@@ -50,10 +50,10 @@
 /*  local function prototypes  */
 
 static void   edit_actions_foreground_changed (GimpContext     *context,
-                                               const GimpRGB   *color,
+                                               GeglColor       *color,
                                                GimpActionGroup *group);
 static void   edit_actions_background_changed (GimpContext     *context,
-                                               const GimpRGB   *color,
+                                               GeglColor       *color,
                                                GimpActionGroup *group);
 static void   edit_actions_pattern_changed    (GimpContext     *context,
                                                GimpPattern     *pattern,
@@ -180,19 +180,31 @@ static const GimpEnumActionEntry edit_paste_actions[] =
     GIMP_HELP_EDIT_PASTE_IN_PLACE },
 
   { "edit-paste-into", GIMP_ICON_EDIT_PASTE_INTO,
-    NC_("edit-action", "Paste _Into Selection"), NULL, { NULL },
+    NC_("edit-action", "Paste as Floating Data _Into Selection"), NULL, { NULL },
     NC_("edit-action",
         "Paste the content of the clipboard into the current selection"),
     GIMP_PASTE_TYPE_FLOATING_INTO, FALSE,
     GIMP_HELP_EDIT_PASTE_INTO },
 
   { "edit-paste-into-in-place", GIMP_ICON_EDIT_PASTE_INTO,
-    NC_("edit-action", "Paste Int_o Selection In Place"), NULL, { NULL },
+    NC_("edit-action", "Paste as Floating Data Int_o Selection In Place"), NULL, { NULL },
     NC_("edit-action",
         "Paste the content of the clipboard into the current selection "
         "at its original position"),
     GIMP_PASTE_TYPE_FLOATING_INTO_IN_PLACE, FALSE,
-    GIMP_HELP_EDIT_PASTE_INTO_IN_PLACE }
+    GIMP_HELP_EDIT_PASTE_INTO_IN_PLACE },
+
+  { "edit-paste-float", GIMP_ICON_EDIT_PASTE,
+    NC_("edit-action", "Paste as _Floating Data"), NULL, { NULL },
+    NC_("edit-action", "Paste the content of the clipboard as Floating Data"),
+    GIMP_PASTE_TYPE_FLOATING, FALSE,
+    GIMP_HELP_EDIT_PASTE_FLOAT },
+
+  { "edit-paste-float-in-place", GIMP_ICON_EDIT_PASTE,
+    NC_("edit-action", "Paste as Floa_ting Data In Place"), NULL, { NULL },
+    NC_("edit-action", "Paste the content of the clipboard as Floating Data at its original position"),
+    GIMP_PASTE_TYPE_FLOATING_IN_PLACE, FALSE,
+    GIMP_HELP_EDIT_PASTE_FLOAT_IN_PLACE }
 };
 
 static const GimpEnumActionEntry edit_fill_actions[] =
@@ -221,7 +233,7 @@ void
 edit_actions_setup (GimpActionGroup *group)
 {
   GimpContext *context = gimp_get_user_context (group->gimp);
-  GimpRGB      color;
+  GeglColor   *color;
   GimpPattern *pattern;
 
   gimp_action_group_add_actions (group, "edit-action",
@@ -248,11 +260,11 @@ edit_actions_setup (GimpActionGroup *group)
                            G_CALLBACK (edit_actions_pattern_changed),
                            group, 0);
 
-  gimp_context_get_foreground (context, &color);
-  edit_actions_foreground_changed (context, &color, group);
+  color = gimp_context_get_foreground (context);
+  edit_actions_foreground_changed (context, color, group);
 
-  gimp_context_get_background (context, &color);
-  edit_actions_background_changed (context, &color, group);
+  color = gimp_context_get_background (context);
+  edit_actions_background_changed (context, color, group);
 
   pattern = gimp_context_get_pattern (context);
   edit_actions_pattern_changed (context, pattern, group);
@@ -368,7 +380,7 @@ edit_actions_update (GimpActionGroup *group,
 
 static void
 edit_actions_foreground_changed (GimpContext     *context,
-                                 const GimpRGB   *color,
+                                 GeglColor       *color,
                                  GimpActionGroup *group)
 {
   gimp_action_group_set_action_color (group, "edit-fill-fg", color, FALSE);
@@ -376,7 +388,7 @@ edit_actions_foreground_changed (GimpContext     *context,
 
 static void
 edit_actions_background_changed (GimpContext     *context,
-                                 const GimpRGB   *color,
+                                 GeglColor       *color,
                                  GimpActionGroup *group)
 {
   gimp_action_group_set_action_color (group, "edit-fill-bg", color, FALSE);
