@@ -262,7 +262,7 @@ typedef enum  /*< pdb-skip >*/
   GIMP_DIRTY_ITEM            = 1 << 4,
   GIMP_DIRTY_ITEM_META       = 1 << 5,
   GIMP_DIRTY_DRAWABLE        = 1 << 6,
-  GIMP_DIRTY_VECTORS         = 1 << 7,
+  GIMP_DIRTY_PATH            = 1 << 7,
   GIMP_DIRTY_SELECTION       = 1 << 8,
   GIMP_DIRTY_ACTIVE_DRAWABLE = 1 << 9,
 
@@ -541,9 +541,10 @@ typedef enum /*< pdb-skip >*/
   GIMP_UNDO_GROUP_IMAGE_ITEM_REMOVE,     /*< desc="Remove item"                    >*/
   GIMP_UNDO_GROUP_IMAGE_ITEM_REORDER,    /*< desc="Reorder item"                   >*/
   GIMP_UNDO_GROUP_IMAGE_LAYERS_MERGE,    /*< desc="Merge layers"                   >*/
-  GIMP_UNDO_GROUP_IMAGE_VECTORS_MERGE,   /*< desc="Merge paths"                    >*/
+  GIMP_UNDO_GROUP_IMAGE_PATHS_MERGE,     /*< desc="Merge paths"                    >*/
   GIMP_UNDO_GROUP_IMAGE_QUICK_MASK,      /*< desc="Quick Mask"                     >*/
   GIMP_UNDO_GROUP_IMAGE_GRID,            /*< desc="Grid"                           >*/
+  GIMP_UNDO_GROUP_IMAGE_COLORMAP_REMAP,  /*< desc="Colormap remapping"             >*/
   GIMP_UNDO_GROUP_GUIDE,                 /*< desc="Guide"                          >*/
   GIMP_UNDO_GROUP_SAMPLE_POINT,          /*< desc="Sample Point"                   >*/
   GIMP_UNDO_GROUP_DRAWABLE,              /*< desc="Layer/Channel"                  >*/
@@ -559,8 +560,8 @@ typedef enum /*< pdb-skip >*/
   GIMP_UNDO_GROUP_ITEM_RESIZE,           /*< desc="Resize item"                    >*/
   GIMP_UNDO_GROUP_LAYER_ADD,             /*< desc="Add layer"                      >*/
   GIMP_UNDO_GROUP_LAYER_ADD_ALPHA,       /*< desc="Add alpha channel"              >*/
-  GIMP_UNDO_GROUP_LAYER_ADD_MASK,        /*< desc="Add layer mask"                 >*/
-  GIMP_UNDO_GROUP_LAYER_APPLY_MASK,      /*< desc="Apply layer mask"               >*/
+  GIMP_UNDO_GROUP_LAYER_ADD_MASK,        /*< desc="Add layer masks"                >*/
+  GIMP_UNDO_GROUP_LAYER_APPLY_MASK,      /*< desc="Apply layer masks"              >*/
   GIMP_UNDO_GROUP_LAYER_REMOVE_ALPHA,    /*< desc="Remove alpha channel"           >*/
   GIMP_UNDO_GROUP_LAYER_LOCK_ALPHA,      /*< desc="Lock/Unlock alpha channels"     >*/
   GIMP_UNDO_GROUP_LAYER_OPACITY,         /*< desc="Set layers opacity"             >*/
@@ -576,7 +577,7 @@ typedef enum /*< pdb-skip >*/
   GIMP_UNDO_GROUP_PAINT,                 /*< desc="Paint"                          >*/
   GIMP_UNDO_GROUP_PARASITE_ATTACH,       /*< desc="Attach parasite"                >*/
   GIMP_UNDO_GROUP_PARASITE_REMOVE,       /*< desc="Remove parasite"                >*/
-  GIMP_UNDO_GROUP_VECTORS_IMPORT,        /*< desc="Import paths"                   >*/
+  GIMP_UNDO_GROUP_PATHS_IMPORT,          /*< desc="Import paths"                   >*/
   GIMP_UNDO_GROUP_MISC,                  /*< desc="Plug-In"                        >*/
 
   GIMP_UNDO_GROUP_LAST = GIMP_UNDO_GROUP_MISC, /*< skip >*/
@@ -604,7 +605,7 @@ typedef enum /*< pdb-skip >*/
   GIMP_UNDO_ITEM_COLOR_TAG,              /*< desc="Item color tag"                 >*/
   GIMP_UNDO_ITEM_LOCK_CONTENT,           /*< desc="Lock/Unlock content"            >*/
   GIMP_UNDO_ITEM_LOCK_POSITION,          /*< desc="Lock/Unlock position"           >*/
-  GIMP_UNDO_ITEM_LOCK_VISIBILITY,        /*< desc="Lock/Unlock visibility"      >*/
+  GIMP_UNDO_ITEM_LOCK_VISIBILITY,        /*< desc="Lock/Unlock visibility"         >*/
   GIMP_UNDO_LAYER_ADD,                   /*< desc="New layer"                      >*/
   GIMP_UNDO_LAYER_REMOVE,                /*< desc="Delete layer"                   >*/
   GIMP_UNDO_LAYER_MODE,                  /*< desc="Set layer mode"                 >*/
@@ -620,16 +621,16 @@ typedef enum /*< pdb-skip >*/
   GIMP_UNDO_TEXT_LAYER,                  /*< desc="Text layer"                     >*/
   GIMP_UNDO_TEXT_LAYER_MODIFIED,         /*< desc="Text layer modification"        >*/
   GIMP_UNDO_TEXT_LAYER_CONVERT,          /*< desc="Convert text layer"             >*/
-  GIMP_UNDO_LAYER_MASK_ADD,              /*< desc="Add layer mask"                 >*/
-  GIMP_UNDO_LAYER_MASK_REMOVE,           /*< desc="Delete layer mask"              >*/
-  GIMP_UNDO_LAYER_MASK_APPLY,            /*< desc="Apply layer mask"               >*/
-  GIMP_UNDO_LAYER_MASK_SHOW,             /*< desc="Show layer mask"                >*/
+  GIMP_UNDO_LAYER_MASK_ADD,              /*< desc="Add layer masks"                >*/
+  GIMP_UNDO_LAYER_MASK_REMOVE,           /*< desc="Delete layer masks"             >*/
+  GIMP_UNDO_LAYER_MASK_APPLY,            /*< desc="Apply layer masks"              >*/
+  GIMP_UNDO_LAYER_MASK_SHOW,             /*< desc="Show layer masks"               >*/
   GIMP_UNDO_CHANNEL_ADD,                 /*< desc="New channel"                    >*/
   GIMP_UNDO_CHANNEL_REMOVE,              /*< desc="Delete channel"                 >*/
   GIMP_UNDO_CHANNEL_COLOR,               /*< desc="Channel color"                  >*/
-  GIMP_UNDO_VECTORS_ADD,                 /*< desc="New path"                       >*/
-  GIMP_UNDO_VECTORS_REMOVE,              /*< desc="Delete path"                    >*/
-  GIMP_UNDO_VECTORS_MOD,                 /*< desc="Path modification"              >*/
+  GIMP_UNDO_PATH_ADD,                    /*< desc="New path"                       >*/
+  GIMP_UNDO_PATH_REMOVE,                 /*< desc="Delete path"                    >*/
+  GIMP_UNDO_PATH_MOD,                    /*< desc="Path modification"              >*/
   GIMP_UNDO_FS_TO_LAYER,                 /*< desc="Floating selection to layer"    >*/
   GIMP_UNDO_TRANSFORM_GRID,              /*< desc="Transform grid"                 >*/
   GIMP_UNDO_PAINT,                       /*< desc="Paint"                          >*/
@@ -637,6 +638,9 @@ typedef enum /*< pdb-skip >*/
   GIMP_UNDO_FOREGROUND_SELECT,           /*< desc="Select foreground"              >*/
   GIMP_UNDO_PARASITE_ATTACH,             /*< desc="Attach parasite"                >*/
   GIMP_UNDO_PARASITE_REMOVE,             /*< desc="Remove parasite"                >*/
+  GIMP_UNDO_FILTER_ADD,                  /*< desc="Add effect"                     >*/
+  GIMP_UNDO_FILTER_REMOVE,               /*< desc="Remove effect"                  >*/
+  GIMP_UNDO_FILTER_REORDER,              /*< desc="Reorder effect"                 >*/
 
   GIMP_UNDO_CANT                         /*< desc="Not undoable"                   >*/
 } GimpUndoType;
@@ -710,6 +714,7 @@ typedef enum  /*< pdb-skip, skip >*/
   GIMP_CONTEXT_PROP_BUFFER      = 18,
   GIMP_CONTEXT_PROP_IMAGEFILE   = 19,
   GIMP_CONTEXT_PROP_TEMPLATE    = 20,
+  GIMP_CONTEXT_PROP_EXPAND      = 21,
 
   GIMP_CONTEXT_PROP_LAST        = GIMP_CONTEXT_PROP_TEMPLATE
 } GimpContextPropType;
@@ -736,6 +741,7 @@ typedef enum  /*< pdb-skip, skip >*/
   GIMP_CONTEXT_PROP_MASK_BUFFER      = 1 << 18,
   GIMP_CONTEXT_PROP_MASK_IMAGEFILE   = 1 << 19,
   GIMP_CONTEXT_PROP_MASK_TEMPLATE    = 1 << 20,
+  GIMP_CONTEXT_PROP_MASK_EXPAND      = 1 << 21,
 
   /*  aliases  */
   GIMP_CONTEXT_PROP_MASK_PAINT = (GIMP_CONTEXT_PROP_MASK_FOREGROUND |
@@ -745,7 +751,8 @@ typedef enum  /*< pdb-skip, skip >*/
                                   GIMP_CONTEXT_PROP_MASK_BRUSH      |
                                   GIMP_CONTEXT_PROP_MASK_DYNAMICS   |
                                   GIMP_CONTEXT_PROP_MASK_PATTERN    |
-                                  GIMP_CONTEXT_PROP_MASK_GRADIENT),
+                                  GIMP_CONTEXT_PROP_MASK_GRADIENT   |
+                                  GIMP_CONTEXT_PROP_MASK_EXPAND),
 
   GIMP_CONTEXT_PROP_MASK_ALL   = (GIMP_CONTEXT_PROP_MASK_IMAGE       |
                                   GIMP_CONTEXT_PROP_MASK_DISPLAY     |
@@ -774,11 +781,11 @@ typedef enum  /*< pdb-skip, skip >*/
 {
   GIMP_ITEM_TYPE_LAYERS   = 1 << 0,
   GIMP_ITEM_TYPE_CHANNELS = 1 << 1,
-  GIMP_ITEM_TYPE_VECTORS  = 1 << 2,
+  GIMP_ITEM_TYPE_PATHS    = 1 << 2,
 
   GIMP_ITEM_TYPE_ALL      = (GIMP_ITEM_TYPE_LAYERS   |
                              GIMP_ITEM_TYPE_CHANNELS |
-                             GIMP_ITEM_TYPE_VECTORS)
+                             GIMP_ITEM_TYPE_PATHS)
 } GimpItemTypeMask;
 
 
