@@ -98,6 +98,11 @@ void          gimp_data_set_file         (GimpData     *data,
                                           gboolean      writable,
                                           gboolean      deletable);
 GFile       * gimp_data_get_file         (GimpData     *data);
+void          gimp_data_set_image        (GimpData     *data,
+                                          GimpImage    *image,
+                                          gboolean      writable,
+                                          gboolean      deletable);
+GimpImage   * gimp_data_get_image        (GimpData     *data);
 
 void          gimp_data_create_filename  (GimpData     *data,
                                           GFile        *dest_dir);
@@ -122,11 +127,19 @@ gboolean      gimp_data_is_duplicatable  (GimpData     *data);
 GimpData    * gimp_data_duplicate        (GimpData     *data);
 
 void          gimp_data_make_internal    (GimpData     *data,
-                                          const gchar  *identifier);
+                                          const gchar  *collection);
 gboolean      gimp_data_is_internal      (GimpData     *data);
 
 gint          gimp_data_compare          (GimpData     *data1,
                                           GimpData     *data2);
+gboolean      gimp_data_identify         (GimpData     *data,
+                                          const gchar  *name,
+                                          const gchar  *collection_id,
+                                          gboolean      is_internal);
+void          gimp_data_get_identifiers  (GimpData     *data,
+                                          gchar       **name,
+                                          gchar       **collection_id,
+                                          gboolean     *is_internal);
 
 #define GIMP_DATA_ERROR (gimp_data_error_quark ())
 
