@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_DRAWABLE_PRIVATE_H__
@@ -20,24 +20,29 @@
 
 struct _GimpDrawablePrivate
 {
-  GimpImageType  type;   /* type of drawable        */
+  GeglBuffer       *buffer; /* buffer for drawable data */
+  GeglBuffer       *shadow; /* shadow buffer            */
 
-  TileManager   *tiles;  /* tiles for drawable data */
-  TileManager   *shadow; /* shadow buffer tiles     */
+  GimpColorProfile *format_profile;
 
-  GeglNode      *source_node;
-  GeglNode      *tile_source_node;
+  GeglNode         *source_node;
+  GeglNode         *buffer_source_node;
+  GimpContainer    *filter_stack;
+  GeglRectangle     bounding_box;
 
-  GimpLayer     *floating_selection;
-  GeglNode      *fs_crop_node;
-  GeglNode      *fs_opacity_node;
-  GeglNode      *fs_offset_node;
-  GeglNode      *fs_mode_node;
+  GimpLayer        *floating_selection;
+  GimpFilter       *fs_filter;
+  GeglNode         *fs_crop_node;
+  GimpApplicator   *fs_applicator;
 
-  GeglNode      *mode_node;
+  GeglNode         *mode_node;
 
-  GSList        *preview_cache; /* preview caches of the channel */
-  gboolean       preview_valid; /* is the preview valid?         */
+  gint              paint_count;
+  GeglBuffer       *paint_buffer;
+  cairo_region_t   *paint_copy_region;
+  cairo_region_t   *paint_update_region;
+
+  gboolean          push_resize_undo;
 };
 
 #endif /* __GIMP_DRAWABLE_PRIVATE_H__ */
