@@ -31,6 +31,7 @@
 #include "gegl/gimp-gegl-apply-operation.h"
 #include "gegl/gimp-gegl-loops.h"
 #include "gegl/gimp-gegl-utils.h"
+#include "gegl/gimptilehandlervalidate.h"
 
 #include "gimp-memsize.h"
 #include "gimp-utils.h"
@@ -38,15 +39,18 @@
 #include "gimpcontext.h"
 #include "gimpdrawable-combine.h"
 #include "gimpdrawable-fill.h"
+#include "gimpdrawable-filters.h"
 #include "gimpdrawable-floating-selection.h"
 #include "gimpdrawable-preview.h"
 #include "gimpdrawable-private.h"
 #include "gimpdrawable-shadow.h"
 #include "gimpdrawable-transform.h"
+#include "gimpdrawablefilter.h"
 #include "gimpfilterstack.h"
 #include "gimpimage.h"
 #include "gimpimage-colormap.h"
 #include "gimpimage-undo-push.h"
+#include "gimplayer.h"
 #include "gimpmarshal.h"
 #include "gimppickable.h"
 #include "gimpprogress.h"
@@ -66,6 +70,7 @@ enum
   FORMAT_CHANGED,
   ALPHA_CHANGED,
   BOUNDING_BOX_CHANGED,
+  FILTERS_CHANGED,
   LAST_SIGNAL
 };
 
@@ -162,6 +167,7 @@ static void       gimp_drawable_real_update        (GimpDrawable      *drawable,
                                                     gint               width,
                                                     gint               height);
 
+static void    gimp_drawable_real_filters_changed  (GimpDrawable      *drawable);
 static gint64  gimp_drawable_real_estimate_memsize (GimpDrawable      *drawable,
                                                     GimpComponentType  component_type,
                                                     gint               width,
@@ -270,6 +276,14 @@ gimp_drawable_class_init (GimpDrawableClass *klass)
                   NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
+  gimp_drawable_signals[FILTERS_CHANGED] =
+    g_signal_new ("filters-changed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GimpDrawableClass, filters_changed),
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
   object_class->dispose           = gimp_drawable_dispose;
   object_class->finalize          = gimp_drawable_finalize;
   object_class->set_property      = gimp_drawable_set_property;
@@ -297,6 +311,7 @@ gimp_drawable_class_init (GimpDrawableClass *klass)
   klass->format_changed           = NULL;
   klass->alpha_changed            = NULL;
   klass->bounding_box_changed     = NULL;
+  klass->filters_changed          = gimp_drawable_real_filters_changed;
   klass->estimate_memsize         = gimp_drawable_real_estimate_memsize;
   klass->update_all               = gimp_drawable_real_update_all;
   klass->invalidate_boundary      = NULL;
@@ -336,12 +351,13 @@ gimp_color_managed_iface_init (GimpColorManagedInterface *iface)
 static void
 gimp_pickable_iface_init (GimpPickableInterface *iface)
 {
-  iface->get_image             = (GimpImage     * (*) (GimpPickable *pickable)) gimp_item_get_image;
-  iface->get_format            = (const Babl    * (*) (GimpPickable *pickable)) gimp_drawable_get_format;
-  iface->get_format_with_alpha = (const Babl    * (*) (GimpPickable *pickable)) gimp_drawable_get_format_with_alpha;
-  iface->get_buffer            = (GeglBuffer    * (*) (GimpPickable *pickable)) gimp_drawable_get_buffer;
-  iface->get_pixel_at          = gimp_drawable_get_pixel_at;
-  iface->get_pixel_average     = gimp_drawable_get_pixel_average;
+  iface->get_image               = (GimpImage     * (*) (GimpPickable *pickable)) gimp_item_get_image;
+  iface->get_format              = (const Babl    * (*) (GimpPickable *pickable)) gimp_drawable_get_format;
+  iface->get_format_with_alpha   = (const Babl    * (*) (GimpPickable *pickable)) gimp_drawable_get_format_with_alpha;
+  iface->get_buffer              = (GeglBuffer    * (*) (GimpPickable *pickable)) gimp_drawable_get_buffer;
+  iface->get_buffer_with_effects = (GeglBuffer    * (*) (GimpPickable *pickable)) gimp_drawable_get_buffer_with_effects;
+  iface->get_pixel_at            = gimp_drawable_get_pixel_at;
+  iface->get_pixel_average       = gimp_drawable_get_pixel_average;
 }
 
 static void
@@ -554,6 +570,30 @@ gimp_drawable_scale (GimpItem              *item,
                                                  0,            0),
                                  TRUE);
   g_object_unref (new_buffer);
+
+  if (GIMP_IS_LAYER (drawable))
+    {
+      GList *list;
+
+      for (list = GIMP_LIST (drawable->private->filter_stack)->queue->tail;
+           list; list = g_list_previous (list))
+        {
+          if (GIMP_IS_DRAWABLE_FILTER (list->data))
+            {
+              GimpDrawableFilter *filter = list->data;
+              GimpChannel        *mask   = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
+              GeglRectangle      *rect   = GEGL_RECTANGLE (0, 0,
+                                                           new_width,
+                                                           new_height);
+
+              /* Don't resize partial layer effects */
+              if (gimp_channel_is_empty (mask))
+                gimp_drawable_filter_refresh_crop (filter, rect);
+            }
+        }
+      if (list)
+        g_list_free (list);
+    }
 }
 
 static void
@@ -606,12 +646,14 @@ gimp_drawable_resize (GimpItem     *item,
     {
       /*  Clear the new buffer if needed  */
 
-      GimpRGB      color;
+      GeglColor   *color;
       GimpPattern *pattern;
 
-      gimp_get_fill_params (context, fill_type, &color, &pattern, NULL);
-      gimp_drawable_fill_buffer (drawable, new_buffer,
-                                 &color, pattern, 0, 0);
+      if (gimp_get_fill_params (context, fill_type, &color, &pattern, NULL))
+        gimp_drawable_fill_buffer (drawable, new_buffer,
+                                   color, pattern, 0, 0);
+
+      g_clear_object (&color);
     }
 
   if (intersect && copy_width && copy_height)
@@ -628,12 +670,37 @@ gimp_drawable_resize (GimpItem     *item,
                         copy_y - new_offset_y, 0, 0));
     }
 
-  gimp_drawable_set_buffer_full (drawable, gimp_item_is_attached (item), NULL,
+  gimp_drawable_set_buffer_full (drawable,
+                                 gimp_item_is_attached (item) &&
+                                 drawable->private->push_resize_undo,
+                                 NULL,
                                  new_buffer,
                                  GEGL_RECTANGLE (new_offset_x, new_offset_y,
                                                  0,            0),
                                  TRUE);
   g_object_unref (new_buffer);
+
+  if (GIMP_IS_LAYER (drawable))
+    {
+      GList *list;
+
+      for (list = GIMP_LIST (drawable->private->filter_stack)->queue->tail;
+           list; list = g_list_previous (list))
+        {
+          if (GIMP_IS_DRAWABLE_FILTER (list->data))
+            {
+              GimpDrawableFilter *filter = list->data;
+              GimpChannel        *mask   = GIMP_CHANNEL (gimp_drawable_filter_get_mask (filter));
+              GeglRectangle       rect   = {0, 0, new_width, new_height};
+
+              /* Don't resize partial layer effects */
+              if (gimp_channel_is_empty (mask))
+                gimp_drawable_filter_refresh_crop (filter, &rect);
+            }
+        }
+      if (list)
+        g_list_free (list);
+    }
 }
 
 static void
@@ -805,6 +872,12 @@ gimp_drawable_real_update (GimpDrawable *drawable,
   gimp_viewable_invalidate_preview (GIMP_VIEWABLE (drawable));
 }
 
+static void
+gimp_drawable_real_filters_changed (GimpDrawable *drawable)
+{
+  gimp_drawable_update_bounding_box (drawable);
+}
+
 static gint64
 gimp_drawable_real_estimate_memsize (GimpDrawable      *drawable,
                                      GimpComponentType  component_type,
@@ -906,6 +979,10 @@ gimp_drawable_real_set_buffer (GimpDrawable        *drawable,
     }
 
   g_set_object (&drawable->private->buffer, buffer);
+
+  if (gimp_drawable_is_painting (drawable))
+    g_set_object (&drawable->private->paint_buffer, buffer);
+
   g_clear_object (&drawable->private->format_profile);
 
   if (drawable->private->buffer_source_node)
@@ -1072,6 +1149,8 @@ gimp_drawable_new (GType          type,
   gimp_drawable_set_buffer (drawable, FALSE, NULL, buffer);
   g_object_unref (buffer);
 
+  gimp_drawable_enable_resize_undo (drawable);
+
   return drawable;
 }
 
@@ -1258,10 +1337,11 @@ gimp_drawable_convert_type (GimpDrawable      *drawable,
                             gboolean           push_undo,
                             GimpProgress      *progress)
 {
-  const Babl *old_format;
-  const Babl *new_format;
-  gint        old_bits;
-  gint        new_bits;
+  const Babl    *old_format;
+  const Babl    *new_format;
+  gint           old_bits;
+  gint           new_bits;
+  GimpContainer *filters;
 
   g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
   g_return_if_fail (GIMP_IS_IMAGE (dest_image));
@@ -1307,6 +1387,44 @@ gimp_drawable_convert_type (GimpDrawable      *drawable,
                                                     push_undo,
                                                     progress);
 
+  /* Update the masks of any filters */
+  /* TODO: Move to gimp_drawable_real_convert_type () once it's updated
+   * to run for all GimpDrawable child classes */
+  filters = gimp_drawable_get_filters (drawable);
+  if (gimp_container_get_n_children (filters) > 0)
+    {
+      const Babl    *mask_format;
+      GList         *filter_list;
+      GimpPrecision  new_mask_precision;
+
+      mask_format        = gimp_image_get_mask_format (dest_image);
+      new_mask_precision = gimp_babl_format_get_precision (mask_format);
+
+      for (filter_list = GIMP_LIST (filters)->queue->tail;
+           filter_list;
+           filter_list = g_list_previous (filter_list))
+        {
+          if (GIMP_IS_DRAWABLE_FILTER (filter_list->data))
+            {
+              GimpDrawableFilter     *filter = filter_list->data;
+              GimpDrawableFilterMask *mask;
+
+              mask = gimp_drawable_filter_get_mask (filter);
+
+              if (new_mask_precision == gimp_drawable_get_precision (GIMP_DRAWABLE (mask)))
+                /* The only change which may happen for masks is the bit-depth. */
+                break;
+
+              gimp_drawable_convert_type (GIMP_DRAWABLE (mask), dest_image,
+                                          GIMP_GRAY, new_mask_precision,
+                                          FALSE,
+                                          NULL, NULL,
+                                          layer_dither_type, mask_dither_type,
+                                          push_undo, progress);
+            }
+        }
+    }
+
   if (progress)
     gimp_progress_set_value (progress, 1.0);
 }
@@ -1430,6 +1548,55 @@ gimp_drawable_set_buffer_full (GimpDrawable        *drawable,
     gimp_drawable_update (drawable, 0, 0, -1, -1);
 }
 
+GeglBuffer *
+gimp_drawable_get_buffer_with_effects (GimpDrawable *drawable)
+{
+  g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
+
+  if (drawable->private->paint_count == 0)
+    {
+      if (gimp_drawable_has_visible_filters (drawable))
+        {
+          GeglNode                *source = NULL;
+          GeglBuffer              *buffer;
+          GimpTileHandlerValidate *validate;
+
+          source = gimp_drawable_get_source_node (drawable);
+          buffer = GIMP_DRAWABLE_GET_CLASS (drawable)->get_buffer (drawable);
+
+          if (source)
+            {
+              buffer = gegl_buffer_new (gegl_buffer_get_extent (buffer),
+                                        gegl_buffer_get_format (buffer));
+
+              validate =
+                GIMP_TILE_HANDLER_VALIDATE (gimp_tile_handler_validate_new (source));
+
+              gimp_tile_handler_validate_assign (validate, buffer);
+
+              g_object_unref (validate);
+
+              gimp_tile_handler_validate_invalidate (validate,
+                                                     gegl_buffer_get_extent (buffer));
+
+              return buffer;
+            }
+          else
+            {
+              return g_object_ref (buffer);
+            }
+        }
+      else
+        {
+          return g_object_ref (GIMP_DRAWABLE_GET_CLASS (drawable)->get_buffer (drawable));
+        }
+    }
+  else
+    {
+      return g_object_ref (drawable->private->paint_buffer);
+    }
+}
+
 void
 gimp_drawable_steal_buffer (GimpDrawable *drawable,
                             GimpDrawable *src_drawable)
@@ -1672,6 +1839,22 @@ gimp_drawable_push_undo (GimpDrawable *drawable,
                                                  x, y, width, height);
 }
 
+void
+gimp_drawable_disable_resize_undo (GimpDrawable *drawable)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
+
+  drawable->private->push_resize_undo = FALSE;
+}
+
+void
+gimp_drawable_enable_resize_undo (GimpDrawable *drawable)
+{
+  g_return_if_fail (GIMP_IS_DRAWABLE (drawable));
+
+  drawable->private->push_resize_undo = TRUE;
+}
+
 const Babl *
 gimp_drawable_get_space (GimpDrawable *drawable)
 {
@@ -1862,18 +2045,6 @@ gimp_drawable_get_component_index (GimpDrawable    *drawable,
   return -1;
 }
 
-guchar *
-gimp_drawable_get_colormap (GimpDrawable *drawable)
-{
-  GimpImage *image;
-
-  g_return_val_if_fail (GIMP_IS_DRAWABLE (drawable), NULL);
-
-  image = gimp_item_get_image (GIMP_ITEM (drawable));
-
-  return image ? gimp_image_get_colormap (image) : NULL;
-}
-
 void
 gimp_drawable_start_paint (GimpDrawable *drawable)
 {
@@ -1911,6 +2082,16 @@ gimp_drawable_end_paint (GimpDrawable *drawable)
 
   drawable->private->paint_count--;
 
+  /* Refresh filters after painting */
+  if (gimp_drawable_has_visible_filters (drawable) &&
+      drawable->private->paint_count == 0)
+    {
+      gimp_item_set_visible (GIMP_ITEM (drawable), FALSE, FALSE);
+      gimp_image_flush (gimp_item_get_image (GIMP_ITEM (drawable)));
+      gimp_item_set_visible (GIMP_ITEM (drawable),TRUE, FALSE);
+      gimp_image_flush (gimp_item_get_image (GIMP_ITEM (drawable)));
+    }
+
   return result;
 }
 
@@ -1979,3 +2160,9 @@ gimp_drawable_is_painting (GimpDrawable *drawable)
 
   return drawable->private->paint_count > 0;
 }
+
+void
+gimp_drawable_filters_changed (GimpDrawable *drawable)
+{
+  g_signal_emit (drawable, gimp_drawable_signals[FILTERS_CHANGED], 0);
+}
