@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -21,6 +21,7 @@
 
 #include <cairo.h>
 #include <gegl.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "libgimpbase/gimpbase.h"
 #include "libgimpcolor/gimpcolor.h"
@@ -28,15 +29,17 @@
 
 #include "core-types.h"
 
-#include "gimp-utils.h"
+#include "gimp-memsize.h"
 #include "gimpdrawable.h"
 #include "gimpgrid.h"
 #include "gimpimage.h"
+#include "gimpimage-color-profile.h"
 #include "gimpimage-colormap.h"
 #include "gimpimage-grid.h"
+#include "gimpimage-metadata.h"
 #include "gimpimage-private.h"
 #include "gimpimageundo.h"
-#include "gimpparasitelist.h"
+#include "gimppalette.h"
 
 
 enum
@@ -149,15 +152,18 @@ gimp_image_undo_constructed (GObject *object)
   GimpImageUndo *image_undo = GIMP_IMAGE_UNDO (object);
   GimpImage     *image;
 
-  if (G_OBJECT_CLASS (parent_class)->constructed)
-    G_OBJECT_CLASS (parent_class)->constructed (object);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 
   image = GIMP_UNDO (object)->image;
 
   switch (GIMP_UNDO (object)->undo_type)
     {
     case GIMP_UNDO_IMAGE_TYPE:
-      image_undo->base_type = gimp_image_base_type (image);
+      image_undo->base_type = gimp_image_get_base_type (image);
+      break;
+
+    case GIMP_UNDO_IMAGE_PRECISION:
+      image_undo->precision = gimp_image_get_precision (image);
       break;
 
     case GIMP_UNDO_IMAGE_SIZE:
@@ -173,25 +179,39 @@ gimp_image_undo_constructed (GObject *object)
       break;
 
     case GIMP_UNDO_IMAGE_GRID:
-      g_assert (GIMP_IS_GRID (image_undo->grid));
+      gimp_assert (GIMP_IS_GRID (image_undo->grid));
       break;
 
     case GIMP_UNDO_IMAGE_COLORMAP:
-      image_undo->num_colors = gimp_image_get_colormap_size (image);
-      image_undo->colormap   = g_memdup (gimp_image_get_colormap (image),
-                                         GIMP_IMAGE_COLORMAP_SIZE);
+        {
+          GimpPalette *palette;
+
+          palette = gimp_image_get_colormap_palette (image);
+          image_undo->colormap = GIMP_PALETTE (gimp_palette_new (NULL, gimp_object_get_name (palette)));
+          gimp_data_copy (GIMP_DATA (image_undo->colormap), GIMP_DATA (palette));
+        }
+      break;
+
+    case GIMP_UNDO_IMAGE_HIDDEN_PROFILE:
+      g_set_object (&image_undo->hidden_profile,
+                    _gimp_image_get_hidden_profile (image));
+      break;
+
+    case GIMP_UNDO_IMAGE_METADATA:
+      image_undo->metadata =
+        gimp_metadata_duplicate (gimp_image_get_metadata (image));
       break;
 
     case GIMP_UNDO_PARASITE_ATTACH:
     case GIMP_UNDO_PARASITE_REMOVE:
-      g_assert (image_undo->parasite_name != NULL);
+      gimp_assert (image_undo->parasite_name != NULL);
 
       image_undo->parasite = gimp_parasite_copy
         (gimp_image_parasite_find (image, image_undo->parasite_name));
       break;
 
     default:
-      g_assert_not_reached ();
+      g_return_if_reached ();
     }
 }
 
@@ -280,6 +300,9 @@ gimp_image_undo_get_memsize (GimpObject *object,
   if (image_undo->colormap)
     memsize += GIMP_IMAGE_COLORMAP_SIZE;
 
+  if (image_undo->metadata)
+    memsize += gimp_g_object_get_memsize (G_OBJECT (image_undo->metadata));
+
   memsize += gimp_object_get_memsize (GIMP_OBJECT (image_undo->grid),
                                       gui_size);
   memsize += gimp_string_get_memsize (image_undo->parasite_name);
@@ -307,13 +330,37 @@ gimp_image_undo_pop (GimpUndo            *undo,
         GimpImageBaseType base_type;
 
         base_type = image_undo->base_type;
-        image_undo->base_type = gimp_image_base_type (image);
+        image_undo->base_type = gimp_image_get_base_type (image);
         g_object_set (image, "base-type", base_type, NULL);
 
         gimp_image_colormap_changed (image, -1);
 
-        if (image_undo->base_type != gimp_image_base_type (image))
-          accum->mode_changed = TRUE;
+        if (image_undo->base_type != gimp_image_get_base_type (image))
+          {
+            if ((image_undo->base_type            == GIMP_GRAY) ||
+                (gimp_image_get_base_type (image) == GIMP_GRAY))
+              {
+                /* in case there was no profile undo, we need to emit
+                 * profile-changed anyway
+                 */
+                gimp_color_managed_profile_changed (GIMP_COLOR_MANAGED (image));
+              }
+
+            accum->mode_changed = TRUE;
+          }
+      }
+      break;
+
+    case GIMP_UNDO_IMAGE_PRECISION:
+      {
+        GimpPrecision precision;
+
+        precision = image_undo->precision;
+        image_undo->precision = gimp_image_get_precision (image);
+        g_object_set (image, "precision", precision, NULL);
+
+        if (image_undo->precision != gimp_image_get_precision (image))
+          accum->precision_changed = TRUE;
       }
       break;
 
@@ -383,7 +430,7 @@ gimp_image_undo_pop (GimpUndo            *undo,
 
       if (image_undo->resolution_unit != gimp_image_get_unit (image))
         {
-          GimpUnit unit;
+          GimpUnit *unit;
 
           unit = gimp_image_get_unit (image);
           private->resolution_unit = image_undo->resolution_unit;
@@ -408,22 +455,51 @@ gimp_image_undo_pop (GimpUndo            *undo,
 
     case GIMP_UNDO_IMAGE_COLORMAP:
       {
-        guchar *colormap;
-        gint    num_colors;
+        GimpPalette *colormap;
 
-        num_colors = gimp_image_get_colormap_size (image);
-        colormap   = g_memdup (gimp_image_get_colormap (image),
-                               GIMP_IMAGE_COLORMAP_SIZE);
+        colormap = gimp_image_get_colormap_palette (image);
+        if (colormap)
+          {
+            GimpPalette *palette = colormap;
 
-        gimp_image_set_colormap (image,
-                                 image_undo->colormap, image_undo->num_colors,
-                                 FALSE);
+            colormap = GIMP_PALETTE (gimp_palette_new (NULL, gimp_object_get_name (palette)));
+            gimp_data_copy (GIMP_DATA (colormap), GIMP_DATA (palette));
+          }
 
         if (image_undo->colormap)
-          g_free (image_undo->colormap);
+          gimp_image_set_colormap_palette (image, image_undo->colormap, FALSE);
+        else
+          gimp_image_unset_colormap (image, FALSE);
 
-        image_undo->num_colors = num_colors;
-        image_undo->colormap   = colormap;
+        if (image_undo->colormap)
+          g_object_unref (image_undo->colormap);
+
+        image_undo->colormap = colormap;
+      }
+      break;
+
+    case GIMP_UNDO_IMAGE_HIDDEN_PROFILE:
+      {
+        GimpColorProfile *hidden_profile = NULL;
+
+        g_set_object (&hidden_profile, _gimp_image_get_hidden_profile (image));
+        _gimp_image_set_hidden_profile (image, image_undo->hidden_profile,
+                                        FALSE);
+        image_undo->hidden_profile = hidden_profile;
+      }
+      break;
+
+    case GIMP_UNDO_IMAGE_METADATA:
+      {
+        GimpMetadata *metadata;
+
+        metadata = gimp_metadata_duplicate (gimp_image_get_metadata (image));
+
+        gimp_image_set_metadata (image, image_undo->metadata, FALSE);
+
+        if (image_undo->metadata)
+          g_object_unref (image_undo->metadata);
+        image_undo->metadata = metadata;
       }
       break;
 
@@ -431,21 +507,14 @@ gimp_image_undo_pop (GimpUndo            *undo,
     case GIMP_UNDO_PARASITE_REMOVE:
       {
         GimpParasite *parasite = image_undo->parasite;
-        const gchar  *name;
 
         image_undo->parasite = gimp_parasite_copy
           (gimp_image_parasite_find (image, image_undo->parasite_name));
 
         if (parasite)
-          gimp_parasite_list_add (private->parasites, parasite);
+          gimp_image_parasite_attach (image, parasite, FALSE);
         else
-          gimp_parasite_list_remove (private->parasites,
-                                     image_undo->parasite_name);
-
-        name = parasite ? parasite->name : image_undo->parasite_name;
-
-        if (strcmp (name, "icc-profile") == 0)
-          gimp_color_managed_profile_changed (GIMP_COLOR_MANAGED (image));
+          gimp_image_parasite_detach (image, image_undo->parasite_name, FALSE);
 
         if (parasite)
           gimp_parasite_free (parasite);
@@ -453,7 +522,7 @@ gimp_image_undo_pop (GimpUndo            *undo,
       break;
 
     default:
-      g_assert_not_reached ();
+      g_return_if_reached ();
     }
 }
 
@@ -463,29 +532,12 @@ gimp_image_undo_free (GimpUndo     *undo,
 {
   GimpImageUndo *image_undo = GIMP_IMAGE_UNDO (undo);
 
-  if (image_undo->grid)
-    {
-      g_object_unref (image_undo->grid);
-      image_undo->grid = NULL;
-    }
-
-  if (image_undo->colormap)
-    {
-      g_free (image_undo->colormap);
-      image_undo->colormap = NULL;
-    }
-
-  if (image_undo->parasite_name)
-    {
-      g_free (image_undo->parasite_name);
-      image_undo->parasite_name = NULL;
-    }
-
-  if (image_undo->parasite)
-    {
-      gimp_parasite_free (image_undo->parasite);
-      image_undo->parasite = NULL;
-    }
+  g_clear_object (&image_undo->grid);
+  g_clear_object (&image_undo->colormap);
+  g_clear_object (&image_undo->hidden_profile);
+  g_clear_object (&image_undo->metadata);
+  g_clear_pointer (&image_undo->parasite_name, g_free);
+  g_clear_pointer (&image_undo->parasite, gimp_parasite_free);
 
   GIMP_UNDO_CLASS (parent_class)->free (undo, undo_mode);
 }
