@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GIMP_IMAGE_UNDO_H__
@@ -30,6 +30,7 @@
 #define GIMP_IMAGE_UNDO_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GIMP_TYPE_IMAGE_UNDO, GimpImageUndoClass))
 
 
+typedef struct _GimpImageUndo      GimpImageUndo;
 typedef struct _GimpImageUndoClass GimpImageUndoClass;
 
 struct _GimpImageUndo
@@ -37,6 +38,7 @@ struct _GimpImageUndo
   GimpUndo           parent_instance;
 
   GimpImageBaseType  base_type;
+  GimpPrecision      precision;
   gint               width;
   gint               height;
   gint               previous_origin_x;
@@ -45,10 +47,12 @@ struct _GimpImageUndo
   gint               previous_height;
   gdouble            xresolution;
   gdouble            yresolution;
-  GimpUnit           resolution_unit;
+  GimpUnit          *resolution_unit;
   GimpGrid          *grid;
   gint               num_colors;
-  guchar            *colormap;
+  GimpPalette       *colormap;
+  GimpColorProfile  *hidden_profile;
+  GimpMetadata      *metadata;
   gchar             *parasite_name;
   GimpParasite      *parasite;
 };
