@@ -36,7 +36,7 @@
 #include "gimplayer.h"
 #include "gimpmarshal.h"
 
-#include "vectors/gimpvectors.h"
+#include "vectors/gimppath.h"
 
 #include "gimp-intl.h"
 
@@ -187,7 +187,7 @@ gimp_item_list_constructed (GObject *object)
 
   gimp_assert (GIMP_IS_IMAGE (set->p->image));
   gimp_assert (set->p->item_type == GIMP_TYPE_LAYER   ||
-               set->p->item_type == GIMP_TYPE_VECTORS ||
+               set->p->item_type == GIMP_TYPE_PATH ||
                set->p->item_type == GIMP_TYPE_CHANNEL);
 
   if (! set->p->is_pattern)
@@ -196,8 +196,8 @@ gimp_item_list_constructed (GObject *object)
 
       if (set->p->item_type == GIMP_TYPE_LAYER)
         container = gimp_image_get_layers (set->p->image);
-      else if (set->p->item_type == GIMP_TYPE_VECTORS)
-        container = gimp_image_get_vectors (set->p->image);
+      else if (set->p->item_type == GIMP_TYPE_PATH)
+        container = gimp_image_get_paths (set->p->image);
       else
         container = gimp_image_get_channels (set->p->image);
       g_signal_connect (container, "remove",
@@ -220,8 +220,8 @@ gimp_item_list_dispose (GObject *object)
 
       if (set->p->item_type == GIMP_TYPE_LAYER)
         container = gimp_image_get_layers (set->p->image);
-      else if (set->p->item_type == GIMP_TYPE_VECTORS)
-        container = gimp_image_get_vectors (set->p->image);
+      else if (set->p->item_type == GIMP_TYPE_PATH)
+        container = gimp_image_get_paths (set->p->image);
       else
         container = gimp_image_get_channels (set->p->image);
       g_signal_handlers_disconnect_by_func (container,
@@ -349,8 +349,8 @@ gimp_item_list_named_new (GimpImage   *image,
     {
       if (item_type == GIMP_TYPE_LAYER)
         items = gimp_image_get_selected_layers (image);
-      else if (item_type == GIMP_TYPE_VECTORS)
-        items = gimp_image_get_selected_vectors (image);
+      else if (item_type == GIMP_TYPE_PATH)
+        items = gimp_image_get_selected_paths (image);
       else if (item_type == GIMP_TYPE_CHANNEL)
         items = gimp_image_get_selected_channels (image);
 
@@ -629,8 +629,7 @@ gimp_item_list_get_items_by_glob (GimpItemList  *set,
   spec = g_pattern_spec_new (pattern);
   for (iter = items; iter; iter = iter->next)
     {
-      if (g_pattern_match_string (spec,
-                                  gimp_object_get_name (iter->data)))
+      if (g_pattern_spec_match_string (spec, gimp_object_get_name (iter->data)))
         match = g_list_prepend (match, iter->data);
     }
   g_pattern_spec_free (spec);
