@@ -24,10 +24,15 @@
 
 #include "core-types.h"
 
+#include "gimpcoreapp.h"
+
 #include "gimp.h"
 #include "gimpbrush.h"
+#include "gimpcontext.h"
 #include "gimpdisplay.h"
+#include "gimpdrawablefilter.h"
 #include "gimpgradient.h"
+#include "gimpgrouplayer.h"
 #include "gimpimage.h"
 #include "gimplayer.h"
 #include "gimplayermask.h"
@@ -39,7 +44,7 @@
 #include "text/gimpfont.h"
 #include "text/gimptextlayer.h"
 
-#include "vectors/gimpvectors.h"
+#include "vectors/gimppath.h"
 
 
 /*
@@ -162,7 +167,7 @@ gimp_param_string_validate (GParamSpec *pspec,
  * @flags:          Flags for the property specified.
  *
  * Creates a new #GimpParamSpecString specifying a
- * #G_TYPE_STRING property.
+ * [type@GLib.String] property.
  *
  * If @allow_non_utf8 is %FALSE, non-valid UTF-8 strings will be
  * replaced by question marks.
