@@ -15,41 +15,47 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
 #include <string.h>
 
-#include <glib-object.h>
+#include <cairo.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
+#include <gegl.h>
 
+#include "libgimpbase/gimpbase.h"
+#include "libgimpcolor/gimpcolor.h"
 #include "libgimpmath/gimpmath.h"
 #include "libgimpconfig/gimpconfig.h"
 
 #include "core-types.h"
 
-#include "base/temp-buf.h"
-
-#include "gimp-utils.h"
+#include "gimp-memsize.h"
+#include "gimpcontainer.h"
 #include "gimpcontext.h"
-#include "gimpmarshal.h"
+#include "gimptempbuf.h"
 #include "gimpviewable.h"
 
-#include "themes/Default/images/gimp-core-pixbufs.h"
-
 
 enum
 {
   PROP_0,
-  PROP_STOCK_ID,
-  PROP_FROZEN
+  PROP_ICON_NAME,
+  PROP_ICON_PIXBUF,
+  PROP_FROZEN,
+  N_PROPS
 };
+static GParamSpec *obj_props[N_PROPS] = { NULL, };
 
 enum
 {
   INVALIDATE_PREVIEW,
   SIZE_CHANGED,
+  EXPANDED_CHANGED,
+  ANCESTRY_CHANGED,
   LAST_SIGNAL
 };
 
@@ -58,17 +64,19 @@ typedef struct _GimpViewablePrivate GimpViewablePrivate;
 
 struct _GimpViewablePrivate
 {
-  gchar        *stock_id;
+  gchar        *icon_name;
+  GdkPixbuf    *icon_pixbuf;
   gint          freeze_count;
+  gboolean      invalidate_pending;
+  gboolean      size_changed_prending;
   GimpViewable *parent;
+  gint          depth;
 
-  TempBuf      *preview_temp_buf;
+  GimpTempBuf  *preview_temp_buf;
   GdkPixbuf    *preview_pixbuf;
 };
 
-#define GET_PRIVATE(viewable) G_TYPE_INSTANCE_GET_PRIVATE (viewable, \
-                                                           GIMP_TYPE_VIEWABLE, \
-                                                           GimpViewablePrivate)
+#define GET_PRIVATE(viewable) ((GimpViewablePrivate *) gimp_viewable_get_instance_private ((GimpViewable *) (viewable)))
 
 
 static void    gimp_viewable_config_iface_init (GimpConfigInterface *iface);
@@ -87,6 +95,7 @@ static gint64  gimp_viewable_get_memsize             (GimpObject    *object,
                                                       gint64        *gui_size);
 
 static void    gimp_viewable_real_invalidate_preview (GimpViewable  *viewable);
+static void    gimp_viewable_real_ancestry_changed   (GimpViewable  *viewable);
 
 static GdkPixbuf * gimp_viewable_real_get_new_pixbuf (GimpViewable  *viewable,
                                                       GimpContext   *context,
@@ -106,6 +115,7 @@ static gboolean gimp_viewable_real_get_popup_size    (GimpViewable  *viewable,
                                                       gint          *popup_height);
 static gchar * gimp_viewable_real_get_description    (GimpViewable  *viewable,
                                                       gchar        **tooltip);
+static gboolean gimp_viewable_real_is_name_editable  (GimpViewable  *viewable);
 static GimpContainer * gimp_viewable_real_get_children (GimpViewable *viewable);
 
 static gboolean gimp_viewable_serialize_property     (GimpConfig    *config,
@@ -113,9 +123,16 @@ static gboolean gimp_viewable_serialize_property     (GimpConfig    *config,
                                                       const GValue  *value,
                                                       GParamSpec    *pspec,
                                                       GimpConfigWriter *writer);
+static gboolean gimp_viewable_deserialize_property   (GimpConfig       *config,
+                                                      guint             property_id,
+                                                      GValue           *value,
+                                                      GParamSpec       *pspec,
+                                                      GScanner         *scanner,
+                                                      GTokenType       *expected);
 
 
 G_DEFINE_TYPE_WITH_CODE (GimpViewable, gimp_viewable, GIMP_TYPE_OBJECT,
+                         G_ADD_PRIVATE (GimpViewable)
                          G_IMPLEMENT_INTERFACE (GIMP_TYPE_CONFIG,
                                                 gimp_viewable_config_iface_init))
 
@@ -135,8 +152,7 @@ gimp_viewable_class_init (GimpViewableClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpViewableClass, invalidate_preview),
-                  NULL, NULL,
-                  gimp_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   viewable_signals[SIZE_CHANGED] =
@@ -144,8 +160,23 @@ gimp_viewable_class_init (GimpViewableClass *klass)
                   G_TYPE_FROM_CLASS (klass),
                   G_SIGNAL_RUN_FIRST,
                   G_STRUCT_OFFSET (GimpViewableClass, size_changed),
-                  NULL, NULL,
-                  gimp_marshal_VOID__VOID,
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
+  viewable_signals[EXPANDED_CHANGED] =
+    g_signal_new ("expanded-changed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GimpViewableClass, expanded_changed),
+                  NULL, NULL, NULL,
+                  G_TYPE_NONE, 0);
+
+  viewable_signals[ANCESTRY_CHANGED] =
+    g_signal_new ("ancestry-changed",
+                  G_TYPE_FROM_CLASS (klass),
+                  G_SIGNAL_RUN_FIRST,
+                  G_STRUCT_OFFSET (GimpViewableClass, ancestry_changed),
+                  NULL, NULL, NULL,
                   G_TYPE_NONE, 0);
 
   object_class->finalize         = gimp_viewable_finalize;
@@ -154,11 +185,14 @@ gimp_viewable_class_init (GimpViewableClass *klass)
 
   gimp_object_class->get_memsize = gimp_viewable_get_memsize;
 
-  klass->default_stock_id        = "gimp-question";
+  klass->default_icon_name       = "gimp-image";
   klass->name_changed_signal     = "name-changed";
+  klass->name_editable           = FALSE;
 
   klass->invalidate_preview      = gimp_viewable_real_invalidate_preview;
   klass->size_changed            = NULL;
+  klass->expanded_changed        = NULL;
+  klass->ancestry_changed        = gimp_viewable_real_ancestry_changed;
 
   klass->get_size                = NULL;
   klass->get_preview_size        = gimp_viewable_real_get_preview_size;
@@ -168,21 +202,29 @@ gimp_viewable_class_init (GimpViewableClass *klass)
   klass->get_pixbuf              = NULL;
   klass->get_new_pixbuf          = gimp_viewable_real_get_new_pixbuf;
   klass->get_description         = gimp_viewable_real_get_description;
+  klass->is_name_editable        = gimp_viewable_real_is_name_editable;
+  klass->preview_freeze          = NULL;
+  klass->preview_thaw            = NULL;
   klass->get_children            = gimp_viewable_real_get_children;
   klass->set_expanded            = NULL;
   klass->get_expanded            = NULL;
 
-  GIMP_CONFIG_INSTALL_PROP_STRING (object_class, PROP_STOCK_ID, "stock-id",
-                                   NULL, NULL,
-                                   GIMP_PARAM_STATIC_STRINGS);
+  obj_props[PROP_ICON_NAME] =
+      g_param_spec_string ("icon-name", NULL, NULL,
+                           NULL,
+                           GIMP_CONFIG_PARAM_FLAGS);
 
-  g_object_class_install_property (object_class, PROP_FROZEN,
-                                   g_param_spec_boolean ("frozen",
-                                                         NULL, NULL,
-                                                         FALSE,
-                                                         GIMP_PARAM_READABLE));
+  obj_props[PROP_ICON_PIXBUF] =
+      g_param_spec_object ("icon-pixbuf", NULL, NULL,
+                           GDK_TYPE_PIXBUF,
+                           GIMP_CONFIG_PARAM_FLAGS);
 
-  g_type_class_add_private (klass, sizeof (GimpViewablePrivate));
+  obj_props[PROP_FROZEN] =
+      g_param_spec_boolean ("frozen", NULL, NULL,
+                            FALSE,
+                            GIMP_PARAM_READABLE);
+
+  g_object_class_install_properties (object_class, N_PROPS, obj_props);
 }
 
 static void
@@ -193,7 +235,8 @@ gimp_viewable_init (GimpViewable *viewable)
 static void
 gimp_viewable_config_iface_init (GimpConfigInterface *iface)
 {
-  iface->serialize_property = gimp_viewable_serialize_property;
+  iface->deserialize_property = gimp_viewable_deserialize_property;
+  iface->serialize_property   = gimp_viewable_serialize_property;
 }
 
 static void
@@ -201,23 +244,10 @@ gimp_viewable_finalize (GObject *object)
 {
   GimpViewablePrivate *private = GET_PRIVATE (object);
 
-  if (private->stock_id)
-    {
-      g_free (private->stock_id);
-      private->stock_id = NULL;
-    }
-
-  if (private->preview_temp_buf)
-    {
-      temp_buf_free (private->preview_temp_buf);
-      private->preview_temp_buf = NULL;
-    }
-
-  if (private->preview_pixbuf)
-    {
-      g_object_unref (private->preview_pixbuf);
-      private->preview_pixbuf = NULL;
-    }
+  g_clear_pointer (&private->icon_name, g_free);
+  g_clear_object (&private->icon_pixbuf);
+  g_clear_pointer (&private->preview_temp_buf, gimp_temp_buf_unref);
+  g_clear_object (&private->preview_pixbuf);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -228,12 +258,19 @@ gimp_viewable_set_property (GObject      *object,
                             const GValue *value,
                             GParamSpec   *pspec)
 {
-  GimpViewable *viewable = GIMP_VIEWABLE (object);
+  GimpViewable        *viewable = GIMP_VIEWABLE (object);
+  GimpViewablePrivate *private  = GET_PRIVATE (object);
 
   switch (property_id)
     {
-    case PROP_STOCK_ID:
-      gimp_viewable_set_stock_id (viewable, g_value_get_string (value));
+    case PROP_ICON_NAME:
+      gimp_viewable_set_icon_name (viewable, g_value_get_string (value));
+      break;
+    case PROP_ICON_PIXBUF:
+      if (private->icon_pixbuf)
+        g_object_unref (private->icon_pixbuf);
+      private->icon_pixbuf = g_value_dup_object (value);
+      gimp_viewable_invalidate_preview (viewable);
       break;
     case PROP_FROZEN:
       /* read-only, fall through */
@@ -250,12 +287,16 @@ gimp_viewable_get_property (GObject    *object,
                             GValue     *value,
                             GParamSpec *pspec)
 {
-  GimpViewable *viewable = GIMP_VIEWABLE (object);
+  GimpViewable        *viewable = GIMP_VIEWABLE (object);
+  GimpViewablePrivate *private  = GET_PRIVATE (object);
 
   switch (property_id)
     {
-    case PROP_STOCK_ID:
-      g_value_set_string (value, gimp_viewable_get_stock_id (viewable));
+    case PROP_ICON_NAME:
+      g_value_set_string (value, gimp_viewable_get_icon_name (viewable));
+      break;
+    case PROP_ICON_PIXBUF:
+      g_value_set_object (value, private->icon_pixbuf);
       break;
     case PROP_FROZEN:
       g_value_set_boolean (value, gimp_viewable_preview_is_frozen (viewable));
@@ -273,7 +314,7 @@ gimp_viewable_get_memsize (GimpObject *object,
 {
   GimpViewablePrivate *private = GET_PRIVATE (object);
 
-  *gui_size += temp_buf_get_memsize (private->preview_temp_buf);
+  *gui_size += gimp_temp_buf_get_memsize (private->preview_temp_buf);
 
   if (private->preview_pixbuf)
     {
@@ -291,16 +332,33 @@ gimp_viewable_real_invalidate_preview (GimpViewable *viewable)
 {
   GimpViewablePrivate *private = GET_PRIVATE (viewable);
 
-  if (private->preview_temp_buf)
-    {
-      temp_buf_free (private->preview_temp_buf);
-      private->preview_temp_buf = NULL;
-    }
+  g_clear_pointer (&private->preview_temp_buf, gimp_temp_buf_unref);
+  g_clear_object (&private->preview_pixbuf);
+}
 
-  if (private->preview_pixbuf)
+static void
+gimp_viewable_real_ancestry_changed_propagate (GimpViewable *viewable,
+                                               GimpViewable *parent)
+{
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
+
+  private->depth = gimp_viewable_get_depth (parent) + 1;
+
+  g_signal_emit (viewable, viewable_signals[ANCESTRY_CHANGED], 0);
+}
+
+static void
+gimp_viewable_real_ancestry_changed (GimpViewable *viewable)
+{
+  GimpContainer *children;
+
+  children = gimp_viewable_get_children (viewable);
+
+  if (children)
     {
-      g_object_unref (private->preview_pixbuf);
-      private->preview_pixbuf = NULL;
+      gimp_container_foreach (children,
+                              (GFunc) gimp_viewable_real_ancestry_changed_propagate,
+                              viewable);
     }
 }
 
@@ -346,48 +404,22 @@ gimp_viewable_real_get_new_pixbuf (GimpViewable *viewable,
                                    gint          width,
                                    gint          height)
 {
-  TempBuf   *temp_buf;
-  GdkPixbuf *pixbuf = NULL;
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
+  GdkPixbuf           *pixbuf  = NULL;
+  GimpTempBuf         *temp_buf;
 
   temp_buf = gimp_viewable_get_preview (viewable, context, width, height);
 
   if (temp_buf)
     {
-      TempBuf *color_buf = NULL;
-      gint     width;
-      gint     height;
-      gint     bytes;
-
-      bytes  = temp_buf->bytes;
-      width  = temp_buf->width;
-      height = temp_buf->height;
-
-      if (bytes == 1 || bytes == 2)
-        {
-          gint color_bytes;
-
-          color_bytes = (bytes == 2) ? 4 : 3;
-
-          color_buf = temp_buf_new (width, height, color_bytes, 0, 0, NULL);
-          temp_buf_copy (temp_buf, color_buf);
-
-          temp_buf = color_buf;
-          bytes    = color_bytes;
-        }
-
-      pixbuf = gdk_pixbuf_new_from_data (g_memdup (temp_buf_get_data (temp_buf),
-                                                   width * height * bytes),
-                                         GDK_COLORSPACE_RGB,
-                                         (bytes == 4),
-                                         8,
-                                         width,
-                                         height,
-                                         width * bytes,
-                                         (GdkPixbufDestroyNotify) g_free,
-                                         NULL);
-
-      if (color_buf)
-        temp_buf_free (color_buf);
+      pixbuf = gimp_temp_buf_create_pixbuf (temp_buf);
+    }
+  else if (private->icon_pixbuf)
+    {
+      pixbuf = gdk_pixbuf_scale_simple (private->icon_pixbuf,
+                                        width,
+                                        height,
+                                        GDK_INTERP_BILINEAR);
     }
 
   return pixbuf;
@@ -400,6 +432,12 @@ gimp_viewable_real_get_description (GimpViewable  *viewable,
   return g_strdup (gimp_object_get_name (viewable));
 }
 
+static gboolean
+gimp_viewable_real_is_name_editable (GimpViewable *viewable)
+{
+  return GIMP_VIEWABLE_GET_CLASS (viewable)->name_editable;
+}
+
 static GimpContainer *
 gimp_viewable_real_get_children (GimpViewable *viewable)
 {
@@ -417,15 +455,97 @@ gimp_viewable_serialize_property (GimpConfig       *config,
 
   switch (property_id)
     {
-    case PROP_STOCK_ID:
-      if (private->stock_id)
+    case PROP_ICON_NAME:
+      if (private->icon_name)
         {
           gimp_config_writer_open (writer, pspec->name);
-          gimp_config_writer_string (writer, private->stock_id);
+          gimp_config_writer_string (writer, private->icon_name);
           gimp_config_writer_close (writer);
         }
       return TRUE;
 
+    case PROP_ICON_PIXBUF:
+      {
+        GdkPixbuf *icon_pixbuf = g_value_get_object (value);
+
+        if (icon_pixbuf)
+          {
+            gchar  *pixbuffer;
+            gsize   pixbuffer_size;
+            GError *error = NULL;
+
+            if (gdk_pixbuf_save_to_buffer (icon_pixbuf,
+                                           &pixbuffer,
+                                           &pixbuffer_size,
+                                           "png", &error, NULL))
+              {
+                gchar *pixbuffer_enc;
+
+                pixbuffer_enc = g_base64_encode ((guchar *)pixbuffer,
+                                                 pixbuffer_size);
+                gimp_config_writer_open (writer, "icon-pixbuf");
+                gimp_config_writer_string (writer, pixbuffer_enc);
+                gimp_config_writer_close (writer);
+
+                g_free (pixbuffer_enc);
+                g_free (pixbuffer);
+              }
+          }
+      }
+      return TRUE;
+
+    default:
+      break;
+    }
+
+  return FALSE;
+}
+
+static gboolean
+gimp_viewable_deserialize_property (GimpConfig *config,
+                                    guint       property_id,
+                                    GValue     *value,
+                                    GParamSpec *pspec,
+                                    GScanner   *scanner,
+                                    GTokenType *expected)
+{
+  switch (property_id)
+    {
+    case PROP_ICON_PIXBUF:
+      {
+        GdkPixbuf *icon_pixbuf = NULL;
+        gchar     *encoded_image;
+
+        if (! gimp_scanner_parse_string (scanner, &encoded_image))
+          {
+            *expected = G_TOKEN_STRING;
+            return TRUE;
+          }
+
+        if (encoded_image && strlen (encoded_image) > 0)
+          {
+            gsize   out_len;
+            guchar *decoded_image = g_base64_decode (encoded_image, &out_len);
+
+            if (decoded_image)
+              {
+                GInputStream *stream;
+
+                stream = g_memory_input_stream_new_from_data (decoded_image,
+                                                              out_len, NULL);
+                icon_pixbuf = gdk_pixbuf_new_from_stream (stream, NULL, NULL);
+                g_object_unref (stream);
+
+                g_free (decoded_image);
+              }
+          }
+
+        g_free (encoded_image);
+
+        g_value_take_object (value, icon_pixbuf);
+      }
+      return TRUE;
+
     default:
       break;
     }
@@ -443,14 +563,15 @@ gimp_viewable_serialize_property (GimpConfig       *config,
 void
 gimp_viewable_invalidate_preview (GimpViewable *viewable)
 {
-  GimpViewablePrivate *private;
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
 
   g_return_if_fail (GIMP_IS_VIEWABLE (viewable));
 
-  private = GET_PRIVATE (viewable);
 
   if (private->freeze_count == 0)
     g_signal_emit (viewable, viewable_signals[INVALIDATE_PREVIEW], 0);
+  else
+    private->invalidate_pending = TRUE;
 }
 
 /**
@@ -464,9 +585,30 @@ gimp_viewable_invalidate_preview (GimpViewable *viewable)
 void
 gimp_viewable_size_changed (GimpViewable *viewable)
 {
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
+
   g_return_if_fail (GIMP_IS_VIEWABLE (viewable));
 
-  g_signal_emit (viewable, viewable_signals[SIZE_CHANGED], 0);
+  if (private->freeze_count == 0)
+    g_signal_emit (viewable, viewable_signals[SIZE_CHANGED], 0);
+  else
+    private->size_changed_prending = TRUE;
+}
+
+/**
+ * gimp_viewable_expanded_changed:
+ * @viewable: a viewable object
+ *
+ * This function sends a signal that is handled at a lower level in the
+ * object hierarchy, and provides a mechanism by which objects derived
+ * from #GimpViewable can respond to expanded state changes.
+ **/
+void
+gimp_viewable_expanded_changed (GimpViewable *viewable)
+{
+  g_return_if_fail (GIMP_IS_VIEWABLE (viewable));
+
+  g_signal_emit (viewable, viewable_signals[EXPANDED_CHANGED], 0);
 }
 
 /**
@@ -475,12 +617,12 @@ gimp_viewable_size_changed (GimpViewable *viewable)
  * @aspect_height:  unscaled height of the preview for an item.
  * @width:          maximum available width for scaled preview.
  * @height:         maximum available height for scaled preview.
- * @dot_for_dot:    if #TRUE, ignore any differences in axis resolution.
+ * @dot_for_dot:    if %TRUE, ignore any differences in axis resolution.
  * @xresolution:    resolution in the horizontal direction.
  * @yresolution:    resolution in the vertical direction.
  * @return_width:   place to return the calculated preview width.
  * @return_height:  place to return the calculated preview height.
- * @scaling_up:     returns #TRUE here if the calculated preview size
+ * @scaling_up:     returns %TRUE here if the calculated preview size
  *                  is larger than the viewable itself.
  *
  * A utility function, for calculating the dimensions of a preview
@@ -491,7 +633,7 @@ gimp_viewable_size_changed (GimpViewable *viewable)
  * preview is scaled to be as large as possible without exceeding
  * these constraints.
  *
- * If @dot_for_dot is #TRUE, and @xresolution and @yresolution are
+ * If @dot_for_dot is %TRUE, and @xresolution and @yresolution are
  * different, then these results are corrected for the difference in
  * resolution on the two axes, so that the requested aspect ratio
  * applies to the appearance of the display rather than to pixel
@@ -565,9 +707,9 @@ gimp_viewable_get_size (GimpViewable  *viewable,
  * @viewable:    the object for which to calculate the preview size.
  * @size:        requested size for preview.
  * @popup:       %TRUE if the preview is intended for a popup window.
- * @dot_for_dot: If #TRUE, ignore any differences in X and Y resolution.
- * @width:       return location for the the calculated width.
- * @height:      return location for the calculated height.
+ * @dot_for_dot: If %TRUE, ignore any differences in X and Y resolution.
+ * @width: (out) (optional):  return location for the the calculated width.
+ * @height: (out) (optional): return location for the calculated height.
  *
  * Retrieve the size of a viewable's preview.  By default, this
  * simply returns the value of the @size argument for both the @width
@@ -605,9 +747,11 @@ gimp_viewable_get_preview_size (GimpViewable *viewable,
  * @viewable:     the object for which to calculate the popup size.
  * @width:        the width of the preview from which the popup will be shown.
  * @height:       the height of the preview from which the popup will be shown.
- * @dot_for_dot:  If #TRUE, ignore any differences in X and Y resolution.
- * @popup_width:  return location for the calculated popup width.
- * @popup_height: return location for the calculated popup height.
+ * @dot_for_dot:  If %TRUE, ignore any differences in X and Y resolution.
+ * @popup_width: (out) (optional): return location for the calculated popup
+ *                                  width.
+ * @popup_height: (out) (optional): return location for the calculated popup
+ *                                  height.
  *
  * Calculate the size of a viewable's preview, for use in making a
  * popup. The arguments @width and @height specify the size of the
@@ -687,33 +831,31 @@ gimp_viewable_get_popup_size (GimpViewable *viewable,
  * Gets a preview for a viewable object, by running through a variety
  * of methods until it finds one that works.  First, if an
  * implementation exists of a "get_preview" method, it is tried, and
- * the result is returned if it is not #NULL.  Second, the function
+ * the result is returned if it is not %NULL.  Second, the function
  * checks to see whether there is a cached preview with the correct
  * dimensions; if so, it is returned.  If neither of these works, then
  * the function looks for an implementation of the "get_new_preview"
  * method, and executes it, caching the result.  If everything fails,
- * #NULL is returned.
+ * %NULL is returned.
  *
- * Returns: A #TempBuf containg the preview image, or #NULL if none can
- *          be found or created.
+ * Returns: (nullable): A #GimpTempBuf containing the preview image, or %NULL if
+ *          none can be found or created.
  **/
-TempBuf *
+GimpTempBuf *
 gimp_viewable_get_preview (GimpViewable *viewable,
                            GimpContext  *context,
                            gint          width,
                            gint          height)
 {
-  GimpViewablePrivate *private;
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
   GimpViewableClass   *viewable_class;
-  TempBuf             *temp_buf = NULL;
+  GimpTempBuf         *temp_buf = NULL;
 
   g_return_val_if_fail (GIMP_IS_VIEWABLE (viewable), NULL);
   g_return_val_if_fail (context == NULL || GIMP_IS_CONTEXT (context), NULL);
   g_return_val_if_fail (width  > 0, NULL);
   g_return_val_if_fail (height > 0, NULL);
 
-  private = GET_PRIVATE (viewable);
-
   if (G_UNLIKELY (context == NULL))
     g_warning ("%s: context is NULL", G_STRFUNC);
 
@@ -727,14 +869,13 @@ gimp_viewable_get_preview (GimpViewable *viewable,
 
   if (private->preview_temp_buf)
     {
-      if (private->preview_temp_buf->width  == width  &&
-          private->preview_temp_buf->height == height)
+      if (gimp_temp_buf_get_width  (private->preview_temp_buf) == width &&
+          gimp_temp_buf_get_height (private->preview_temp_buf) == height)
         {
           return private->preview_temp_buf;
         }
 
-      temp_buf_free (private->preview_temp_buf);
-      private->preview_temp_buf = NULL;
+      g_clear_pointer (&private->preview_temp_buf, gimp_temp_buf_unref);
     }
 
   if (viewable_class->get_new_preview)
@@ -758,17 +899,17 @@ gimp_viewable_get_preview (GimpViewable *viewable,
  * then if that fails for a "get_preview" method.  This function does
  * not look for a cached preview.
  *
- * Returns: A #TempBuf containg the preview image, or #NULL if none can
- *          be found or created.
+ * Returns: (nullable): A #GimpTempBuf containing the preview image, or %NULL if
+ *          none can be found or created.
  **/
-TempBuf *
+GimpTempBuf *
 gimp_viewable_get_new_preview (GimpViewable *viewable,
                                GimpContext  *context,
                                gint          width,
                                gint          height)
 {
   GimpViewableClass *viewable_class;
-  TempBuf           *temp_buf = NULL;
+  GimpTempBuf       *temp_buf = NULL;
 
   g_return_val_if_fail (GIMP_IS_VIEWABLE (viewable), NULL);
   g_return_val_if_fail (context == NULL || GIMP_IS_CONTEXT (context), NULL);
@@ -792,7 +933,7 @@ gimp_viewable_get_new_preview (GimpViewable *viewable,
                                             width, height);
 
   if (temp_buf)
-    return temp_buf_copy (temp_buf, NULL);
+    return gimp_temp_buf_copy (temp_buf);
 
   return NULL;
 }
@@ -809,38 +950,26 @@ gimp_viewable_get_new_preview (GimpViewable *viewable,
  * generate a preview in situations where layer previews have been
  * disabled in the current Gimp configuration.
  *
- * Returns: a #TempBuf containing the preview image.
+ * Returns: a #GimpTempBuf containing the preview image.
  **/
-TempBuf *
-gimp_viewable_get_dummy_preview (GimpViewable  *viewable,
-                                 gint           width,
-                                 gint           height,
-                                 gint           bpp)
+GimpTempBuf *
+gimp_viewable_get_dummy_preview (GimpViewable *viewable,
+                                 gint          width,
+                                 gint          height,
+                                 const Babl   *format)
 {
-  GdkPixbuf *pixbuf;
-  TempBuf   *buf;
-  guchar    *src;
-  guchar    *dest;
+  GdkPixbuf   *pixbuf;
+  GimpTempBuf *buf;
 
   g_return_val_if_fail (GIMP_IS_VIEWABLE (viewable), NULL);
   g_return_val_if_fail (width  > 0, NULL);
   g_return_val_if_fail (height > 0, NULL);
-  g_return_val_if_fail (bpp == 3 || bpp == 4, NULL);
+  g_return_val_if_fail (format != NULL, NULL);
 
-  pixbuf = gimp_viewable_get_dummy_pixbuf (viewable, width, height, bpp);
+  pixbuf = gimp_viewable_get_dummy_pixbuf (viewable, width, height,
+                                           babl_format_has_alpha (format));
 
-  buf = temp_buf_new (width, height, bpp, 0, 0, NULL);
-
-  src  = gdk_pixbuf_get_pixels (pixbuf);
-  dest = temp_buf_get_data (buf);
-
-  while (height--)
-    {
-      memcpy (dest, src, width * bpp);
-
-      src  += gdk_pixbuf_get_rowstride (pixbuf);
-      dest += width * bpp;
-    }
+  buf = gimp_temp_buf_new_from_pixbuf (pixbuf, format);
 
   g_object_unref (pixbuf);
 
@@ -857,15 +986,15 @@ gimp_viewable_get_dummy_preview (GimpViewable  *viewable,
  * Gets a preview for a viewable object, by running through a variety
  * of methods until it finds one that works.  First, if an
  * implementation exists of a "get_pixbuf" method, it is tried, and
- * the result is returned if it is not #NULL.  Second, the function
+ * the result is returned if it is not %NULL.  Second, the function
  * checks to see whether there is a cached preview with the correct
  * dimensions; if so, it is returned.  If neither of these works, then
  * the function looks for an implementation of the "get_new_pixbuf"
  * method, and executes it, caching the result.  If everything fails,
- * #NULL is returned.
+ * %NULL is returned.
  *
- * Returns: A #GdkPixbuf containing the preview pixbuf, or #NULL if none can
- *          be found or created.
+ * Returns: (nullable): A #GdkPixbuf containing the preview pixbuf,
+ *          or %NULL if none can be found or created.
  **/
 GdkPixbuf *
 gimp_viewable_get_pixbuf (GimpViewable *viewable,
@@ -873,7 +1002,7 @@ gimp_viewable_get_pixbuf (GimpViewable *viewable,
                           gint          width,
                           gint          height)
 {
-  GimpViewablePrivate *private;
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
   GimpViewableClass   *viewable_class;
   GdkPixbuf           *pixbuf = NULL;
 
@@ -882,8 +1011,6 @@ gimp_viewable_get_pixbuf (GimpViewable *viewable,
   g_return_val_if_fail (width  > 0, NULL);
   g_return_val_if_fail (height > 0, NULL);
 
-  private = GET_PRIVATE (viewable);
-
   if (G_UNLIKELY (context == NULL))
     g_warning ("%s: context is NULL", G_STRFUNC);
 
@@ -903,8 +1030,7 @@ gimp_viewable_get_pixbuf (GimpViewable *viewable,
           return private->preview_pixbuf;
         }
 
-      g_object_unref (private->preview_pixbuf);
-      private->preview_pixbuf = NULL;
+      g_clear_object (&private->preview_pixbuf);
     }
 
   if (viewable_class->get_new_pixbuf)
@@ -928,8 +1054,8 @@ gimp_viewable_get_pixbuf (GimpViewable *viewable,
  * then if that fails for a "get_pixbuf" method.  This function does
  * not look for a cached pixbuf.
  *
- * Returns: A #GdkPixbuf containing the preview, or #NULL if none can
- *          be created.
+ * Returns: (nullable): A #GdkPixbuf containing the preview,
+ *          or %NULL if none can be created.
  **/
 GdkPixbuf *
 gimp_viewable_get_new_pixbuf (GimpViewable *viewable,
@@ -986,21 +1112,26 @@ GdkPixbuf *
 gimp_viewable_get_dummy_pixbuf (GimpViewable  *viewable,
                                 gint           width,
                                 gint           height,
-                                gint           bpp)
+                                gboolean       with_alpha)
 {
   GdkPixbuf *icon;
   GdkPixbuf *pixbuf;
+  GError    *error = NULL;
   gdouble    ratio;
   gint       w, h;
 
   g_return_val_if_fail (GIMP_IS_VIEWABLE (viewable), NULL);
   g_return_val_if_fail (width  > 0, NULL);
   g_return_val_if_fail (height > 0, NULL);
-  g_return_val_if_fail (bpp == 3 || bpp == 4, NULL);
 
-  icon = gdk_pixbuf_new_from_inline (-1, stock_question_64, FALSE, NULL);
-
-  g_return_val_if_fail (icon != NULL, NULL);
+  icon = gdk_pixbuf_new_from_resource ("/org/gimp/icons/64/dialog-question.png",
+                                       &error);
+  if (! icon)
+    {
+      g_critical ("Failed to create icon image: %s", error->message);
+      g_clear_error (&error);
+      return NULL;
+    }
 
   w = gdk_pixbuf_get_width (icon);
   h = gdk_pixbuf_get_height (icon);
@@ -1011,7 +1142,7 @@ gimp_viewable_get_dummy_pixbuf (GimpViewable  *viewable,
   w = RINT (ratio * (gdouble) w);
   h = RINT (ratio * (gdouble) h);
 
-  pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, (bpp == 4), 8, width, height);
+  pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, with_alpha, 8, width, height);
   gdk_pixbuf_fill (pixbuf, 0xffffffff);
 
   if (w && h)
@@ -1028,7 +1159,8 @@ gimp_viewable_get_dummy_pixbuf (GimpViewable  *viewable,
 /**
  * gimp_viewable_get_description:
  * @viewable: viewable object for which to retrieve a description.
- * @tooltip:  return loaction for an optional tooltip string.
+ * @tooltip: (out) (optional) (nullable): return location for an optional
+ *                                        tooltip string.
  *
  * Retrieves a string containing a description of the viewable object,
  * By default, it simply returns the name of the object, but this can
@@ -1051,97 +1183,124 @@ gimp_viewable_get_description (GimpViewable  *viewable,
 }
 
 /**
- * gimp_viewable_get_stock_id:
- * @viewable: viewable object for which to retrieve a stock ID.
+ * gimp_viewable_is_name_editable:
+ * @viewable: viewable object for which to retrieve a description.
  *
- * Gets the current value of the object's stock ID, for use in
- * constructing an iconic representation of the object.
- *
- * Returns: a pointer to the string containing the stock ID.  The
- *          contents must not be altered or freed.
+ * Returns: whether the viewable's name is editable by the user.
  **/
-const gchar *
-gimp_viewable_get_stock_id (GimpViewable *viewable)
+gboolean
+gimp_viewable_is_name_editable (GimpViewable *viewable)
 {
-  GimpViewablePrivate *private;
+  g_return_val_if_fail (GIMP_IS_VIEWABLE (viewable), FALSE);
 
-  g_return_val_if_fail (GIMP_IS_VIEWABLE (viewable), NULL);
-
-  private = GET_PRIVATE (viewable);
-
-  if (private->stock_id)
-    return (const gchar *) private->stock_id;
-
-  return GIMP_VIEWABLE_GET_CLASS (viewable)->default_stock_id;
+  return GIMP_VIEWABLE_GET_CLASS (viewable)->is_name_editable (viewable);
 }
 
 /**
- * gimp_viewable_set_stock_id:
- * @viewable: viewable object to assign the specified stock ID.
- * @stock_id: string containing a stock identifier.
+ * gimp_viewable_get_icon_name:
+ * @viewable: viewable object for which to retrieve a icon name.
  *
- * Seta the object's stock ID, for use in constructing iconic smbols
- * of the object.  The contents of @stock_id are copied, so you can
+ * Gets the current value of the object's icon name, for use in
+ * constructing an iconic representation of the object.
+ *
+ * Returns: a pointer to the string containing the icon name.  The
+ *          contents must not be altered or freed.
+ **/
+const gchar *
+gimp_viewable_get_icon_name (GimpViewable *viewable)
+{
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
+
+  g_return_val_if_fail (GIMP_IS_VIEWABLE (viewable), NULL);
+
+  if (private->icon_name)
+    return (const gchar *) private->icon_name;
+
+  return GIMP_VIEWABLE_GET_CLASS (viewable)->default_icon_name;
+}
+
+/**
+ * gimp_viewable_set_icon_name:
+ * @viewable: viewable object to assign the specified icon name.
+ * @icon_name: string containing an icon name identifier.
+ *
+ * Set the object's icon name, for use in constructing iconic symbols
+ * of the object. The contents of @icon_name are copied, so you can
  * free it when you are done with it.
  **/
 void
-gimp_viewable_set_stock_id (GimpViewable *viewable,
-                            const gchar  *stock_id)
+gimp_viewable_set_icon_name (GimpViewable *viewable,
+                             const gchar  *icon_name)
 {
-  GimpViewablePrivate *private;
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
   GimpViewableClass   *viewable_class;
 
   g_return_if_fail (GIMP_IS_VIEWABLE (viewable));
 
-  private = GET_PRIVATE (viewable);
-
-  g_free (private->stock_id);
-  private->stock_id = NULL;
+  g_clear_pointer (&private->icon_name, g_free);
 
   viewable_class = GIMP_VIEWABLE_GET_CLASS (viewable);
 
-  if (stock_id)
+  if (icon_name)
     {
-      if (viewable_class->default_stock_id == NULL ||
-          strcmp (stock_id, viewable_class->default_stock_id))
-        private->stock_id = g_strdup (stock_id);
+      if (viewable_class->default_icon_name == NULL ||
+          strcmp (icon_name, viewable_class->default_icon_name))
+        private->icon_name = g_strdup (icon_name);
     }
 
-  g_object_notify (G_OBJECT (viewable), "stock-id");
+  gimp_viewable_invalidate_preview (viewable);
+
+  g_object_notify_by_pspec (G_OBJECT (viewable), obj_props[PROP_ICON_NAME]);
 }
 
 void
 gimp_viewable_preview_freeze (GimpViewable *viewable)
 {
-  GimpViewablePrivate *private;
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
 
   g_return_if_fail (GIMP_IS_VIEWABLE (viewable));
 
-  private = GET_PRIVATE (viewable);
-
   private->freeze_count++;
 
   if (private->freeze_count == 1)
-    g_object_notify (G_OBJECT (viewable), "frozen");
+    {
+      if (GIMP_VIEWABLE_GET_CLASS (viewable)->preview_freeze)
+        GIMP_VIEWABLE_GET_CLASS (viewable)->preview_freeze (viewable);
+
+      g_object_notify_by_pspec (G_OBJECT (viewable), obj_props[PROP_FROZEN]);
+    }
 }
 
 void
 gimp_viewable_preview_thaw (GimpViewable *viewable)
 {
-  GimpViewablePrivate *private;
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
 
   g_return_if_fail (GIMP_IS_VIEWABLE (viewable));
-
-  private = GET_PRIVATE (viewable);
-
   g_return_if_fail (private->freeze_count > 0);
 
   private->freeze_count--;
 
   if (private->freeze_count == 0)
     {
-      gimp_viewable_invalidate_preview (viewable);
-      g_object_notify (G_OBJECT (viewable), "frozen");
+      if (private->size_changed_prending)
+        {
+          private->size_changed_prending = FALSE;
+
+          gimp_viewable_size_changed (viewable);
+        }
+
+      if (private->invalidate_pending)
+        {
+          private->invalidate_pending = FALSE;
+
+          gimp_viewable_invalidate_preview (viewable);
+        }
+
+      g_object_notify_by_pspec (G_OBJECT (viewable), obj_props[PROP_FROZEN]);
+
+      if (GIMP_VIEWABLE_GET_CLASS (viewable)->preview_thaw)
+        GIMP_VIEWABLE_GET_CLASS (viewable)->preview_thaw (viewable);
     }
 }
 
@@ -1165,10 +1324,26 @@ void
 gimp_viewable_set_parent (GimpViewable *viewable,
                           GimpViewable *parent)
 {
+  GimpViewablePrivate *private = GET_PRIVATE (viewable);
+
   g_return_if_fail (GIMP_IS_VIEWABLE (viewable));
   g_return_if_fail (parent == NULL || GIMP_IS_VIEWABLE (parent));
 
-  GET_PRIVATE (viewable)->parent = parent;
+  if (parent != private->parent)
+    {
+      private->parent = parent;
+      private->depth  = parent ? gimp_viewable_get_depth (parent) + 1 : 0;
+
+      g_signal_emit (viewable, viewable_signals[ANCESTRY_CHANGED], 0);
+    }
+}
+
+gint
+gimp_viewable_get_depth (GimpViewable *viewable)
+{
+  g_return_val_if_fail (GIMP_IS_VIEWABLE (viewable), 0);
+
+  return GET_PRIVATE (viewable)->depth;
 }
 
 GimpContainer *
