@@ -29,6 +29,10 @@
 #include "dialogs-types.h"
 
 #include "config/gimpcoreconfig.h"
+#include "config/gimpguiconfig.h"
+
+#include "widgets/gimphelp-ids.h"
+#include "widgets/gimpwidgets-utils.h"
 
 #include "about.h"
 #include "git-version.h"
@@ -50,26 +54,32 @@
 
 typedef struct
 {
-  GtkWidget   *dialog;
+  GtkWidget      *dialog;
+
+  Gimp           *gimp;
 
   GtkWidget      *update_frame;
   GimpCoreConfig *config;
 
-  GtkWidget   *anim_area;
-  PangoLayout *layout;
+  GtkWidget      *anim_area;
+  PangoLayout    *layout;
+  gboolean        use_animation;
 
-  gint         n_authors;
-  gint         shuffle[G_N_ELEMENTS (authors) - 1];  /* NULL terminated */
+  gint            n_authors;
+  gint            shuffle[G_N_ELEMENTS (authors) - 1];  /* NULL terminated */
 
-  guint        timer;
+  guint           timer;
 
-  gint         index;
-  gint         animstep;
-  gint         state;
-  gboolean     visible;
+  gint            index;
+  gint            animstep;
+  gint            state;
+  gboolean        visible;
 } GimpAboutDialog;
 
-
+#ifdef G_OS_WIN32
+static void        about_dialog_realize       (GtkWidget       *widget,
+                                               GimpAboutDialog *dialog);
+#endif
 static void        about_dialog_map           (GtkWidget       *widget,
                                                GimpAboutDialog *dialog);
 static void        about_dialog_unmap         (GtkWidget       *widget,
@@ -99,7 +109,8 @@ static void        about_dialog_download_clicked
                                                const gchar *link);
 
 GtkWidget *
-about_dialog_create (GimpCoreConfig *config)
+about_dialog_create (Gimp           *gimp,
+                     GimpCoreConfig *config)
 {
   static GimpAboutDialog dialog;
 
@@ -112,9 +123,20 @@ about_dialog_create (GimpCoreConfig *config)
       gchar     *copyright;
       gchar     *version;
 
+      dialog.gimp      = gimp;
       dialog.n_authors = G_N_ELEMENTS (authors) - 1;
       dialog.config    = config;
 
+      /* For some people, animated contents may be distracting, or even
+       * disturbing. "Vestibular motion disorders" are an example of
+       * such discomfort. This is why most platforms have a "reduce
+       * animations" option in accessibility settings.
+       * When it's set, we just won't display the fancy animated authors
+       * list. This is redundant anyway as the full list is available in
+       * the Credits tab.
+       */
+      dialog.use_animation = gimp_widget_animation_enabled ();
+
       pixbuf = about_dialog_load_logo ();
 
       copyright = g_strdup_printf (GIMP_COPYRIGHT, GIMP_GIT_LAST_COMMIT_YEAR);
@@ -160,7 +182,11 @@ about_dialog_create (GimpCoreConfig *config)
       g_signal_connect (widget, "response",
                         G_CALLBACK (gtk_widget_destroy),
                         NULL);
-
+#ifdef G_OS_WIN32
+      g_signal_connect (widget, "realize",
+                        G_CALLBACK (about_dialog_realize),
+                        &dialog);
+#endif
       g_signal_connect (widget, "map",
                         G_CALLBACK (about_dialog_map),
                         &dialog);
@@ -174,11 +200,15 @@ about_dialog_create (GimpCoreConfig *config)
 
       if (GTK_IS_BOX (children->data))
         {
-          about_dialog_add_animation (children->data, &dialog);
+          if (dialog.use_animation)
+            about_dialog_add_animation (children->data, &dialog);
 #ifdef GIMP_UNSTABLE
           about_dialog_add_unstable_message (children->data);
 #endif /* GIMP_UNSTABLE */
-          about_dialog_add_update (&dialog, config);
+#ifdef CHECK_UPDATE
+          if (gimp_version_check_update ())
+            about_dialog_add_update (&dialog, config);
+#endif
         }
       else
         g_warning ("%s: ooops, no box in this container?", G_STRLOC);
@@ -186,9 +216,31 @@ about_dialog_create (GimpCoreConfig *config)
       g_list_free (children);
     }
 
+  if (GIMP_GUI_CONFIG (config)->show_help_button)
+    {
+      gimp_help_connect (dialog.dialog, NULL, gimp_standard_help_func,
+                         GIMP_HELP_ABOUT_DIALOG, NULL, NULL);
+
+      gtk_dialog_add_buttons (GTK_DIALOG (dialog.dialog),
+                              _("_Help"), GTK_RESPONSE_HELP,
+                              NULL);
+    }
+
+  gtk_style_context_add_class (gtk_widget_get_style_context (dialog.dialog),
+                               "gimp-about-dialog");
+
   return dialog.dialog;
 }
 
+#ifdef G_OS_WIN32
+static void
+about_dialog_realize (GtkWidget *widget,
+                      GimpAboutDialog *dialog)
+{
+  gimp_window_set_title_bar_theme (dialog->gimp, widget);
+}
+#endif
+
 static void
 about_dialog_map (GtkWidget       *widget,
                   GimpAboutDialog *dialog)
@@ -407,6 +459,8 @@ about_dialog_add_update (GimpAboutDialog *dialog,
                                     "view-refresh",
                                     GTK_ICON_SIZE_MENU);
       gtk_label_set_text (GTK_LABEL (button_label), _("Check for updates"));
+      gtk_style_context_add_class (gtk_widget_get_style_context (button),
+                                   "text-button");
       g_signal_connect_swapped (button, "clicked",
                                 (GCallback) gimp_update_check, config);
 
@@ -656,8 +710,8 @@ decorate_text (GimpAboutDialog *dialog,
 static gboolean
 about_dialog_timer (gpointer data)
 {
-  GimpAboutDialog *dialog  = data;
-  gint             timeout = 0;
+  GimpAboutDialog *dialog        = data;
+  gint             timeout       = 0;
 
   if (dialog->animstep == 0)
     {
@@ -670,7 +724,7 @@ about_dialog_timer (gpointer data)
         case 0:
           dialog->timer = g_timeout_add (30, about_dialog_timer, dialog);
           dialog->state += 1;
-          return FALSE;
+          return G_SOURCE_REMOVE;
 
         case 1:
           text = insert_spacers (_("GIMP is brought to you by"));
@@ -712,8 +766,7 @@ about_dialog_timer (gpointer data)
     }
   else if (dialog->animstep < 33)
     {
-      decorate_text (dialog, 1,
-                     1.0 - ((gfloat) (dialog->animstep - 17)) / 15.0);
+      decorate_text (dialog, 1, 1.0 - ((gfloat) (dialog->animstep - 17)) / 15.0);
     }
   else if (dialog->animstep == 33)
     {
@@ -734,11 +787,11 @@ about_dialog_timer (gpointer data)
   if (timeout > 0)
     {
       dialog->timer = g_timeout_add (timeout, about_dialog_timer, dialog);
-      return FALSE;
+      return G_SOURCE_REMOVE;
     }
 
   /* else keep the current timeout */
-  return TRUE;
+  return G_SOURCE_CONTINUE;
 }
 
 #ifdef GIMP_UNSTABLE
