@@ -439,10 +439,22 @@ palette_import_response (GtkWidget    *dialog,
           gimp_palette_get_n_colors (private->palette) > 0)
         {
           const gchar *name = gtk_entry_get_text (GTK_ENTRY (private->entry));
+          GError      *error = NULL;
 
           if (name && *name)
             gimp_object_set_name (GIMP_OBJECT (private->palette), name);
 
+          if (! gimp_data_factory_data_save_single (gimp->palette_factory,
+                                                    GIMP_DATA (private->palette),
+                                                    &error))
+            {
+              gimp_message (gimp, G_OBJECT (dialog), GIMP_MESSAGE_ERROR,
+                            _("The palette was not imported: %s"),
+                            error->message);
+              g_clear_error (&error);
+              return;
+            }
+
           gimp_container_add (gimp_data_factory_get_container (gimp->palette_factory),
                               GIMP_OBJECT (private->palette));
         }
@@ -847,6 +859,18 @@ palette_import_make_palette (ImportDialog *private)
                                   error->message);
             g_error_free (error);
           }
+        else
+          {
+            gint columns = gimp_palette_get_columns (palette);
+
+            if (columns > 0)
+              {
+                n_columns = columns;
+
+                gimp_label_spin_set_value (GIMP_LABEL_SPIN (private->columns),
+                                           n_columns);
+              }
+          }
       }
       break;
     }
