@@ -51,6 +51,8 @@ struct _GimpDisplayShell
 
   GimpDisplay       *display;
 
+  GBytes            *window_handle;
+
   GimpUIManager     *popup_manager;
   GdkMonitor        *initial_monitor;
 
@@ -58,7 +60,7 @@ struct _GimpDisplayShell
   GimpDisplayOptions *fullscreen_options;
   GimpDisplayOptions *no_image_options;
 
-  GimpUnit           unit;
+  GimpUnit           *unit;
 
   gint               offset_x;         /*  offset of display image            */
   gint               offset_y;
@@ -83,8 +85,6 @@ struct _GimpDisplayShell
   gint               last_offset_x;    /*  offsets used when reverting zoom   */
   gint               last_offset_y;
 
-  gdouble            other_scale;      /*  scale factor entered in Zoom->Other*/
-
   gint               disp_width;       /*  width of drawing area              */
   gint               disp_height;      /*  height of drawing area             */
 
@@ -193,9 +193,9 @@ struct _GimpDisplayShell
   gboolean           size_allocate_center_image;
 
   /*  the state of gimp_display_shell_tool_events()  */
+  GdkSeat           *grab_seat;
   GdkDevice         *grab_pointer;
   GdkDevice         *grab_pointer_source;
-  guint32            grab_pointer_time;
 
   /*  the state of gimp_display_shell_zoom_gesture_*() */
   gdouble            last_zoom_scale;
@@ -230,7 +230,7 @@ struct _GimpDisplayShell
   GeglBuffer        *mask;
   gint               mask_offset_x;
   gint               mask_offset_y;
-  GimpRGB            mask_color;
+  GeglColor         *mask_color;
   gboolean           mask_inverted;
 
   GimpMotionBuffer  *motion_buffer;
@@ -239,6 +239,22 @@ struct _GimpDisplayShell
 
   gboolean           blink;
   guint              blink_timeout_id;
+
+  GimpAlignmentType  snapped_side_horizontal; /* if GIMP_ARRANGE_HFILL then no snapping happen */
+  GimpLayer         *snapped_layer_horizontal;
+
+  GimpAlignmentType  snapped_side_vertical; /* if GIMP_ARRANGE_HFILL then no snapping happen */
+  GimpLayer         *snapped_layer_vertical;
+
+  GimpAlignmentType  equidistance_side_horizontal;
+  GimpLayer         *near_layer_horizontal1;
+  GimpLayer         *near_layer_horizontal2;
+
+  GimpAlignmentType  equidistance_side_vertical;
+  GimpLayer         *near_layer_vertical1;
+  GimpLayer         *near_layer_vertical2;
+
+  gboolean           drawn;
 };
 
 struct _GimpDisplayShellClass
@@ -255,7 +271,7 @@ struct _GimpDisplayShellClass
 GType             gimp_display_shell_get_type      (void) G_GNUC_CONST;
 
 GtkWidget       * gimp_display_shell_new           (GimpDisplay        *display,
-                                                    GimpUnit            unit,
+                                                    GimpUnit           *unit,
                                                     gdouble             scale,
                                                     GimpUIManager      *popup_manager,
                                                     GdkMonitor         *monitor);
@@ -288,7 +304,7 @@ void              gimp_display_shell_reconnect     (GimpDisplayShell   *shell);
 void              gimp_display_shell_empty         (GimpDisplayShell   *shell);
 void              gimp_display_shell_fill          (GimpDisplayShell   *shell,
                                                     GimpImage          *image,
-                                                    GimpUnit            unit,
+                                                    GimpUnit           *unit,
                                                     gdouble             scale);
 
 void              gimp_display_shell_scaled        (GimpDisplayShell   *shell);
@@ -296,8 +312,8 @@ void              gimp_display_shell_scrolled      (GimpDisplayShell   *shell);
 void              gimp_display_shell_rotated       (GimpDisplayShell   *shell);
 
 void              gimp_display_shell_set_unit      (GimpDisplayShell   *shell,
-                                                    GimpUnit            unit);
-GimpUnit          gimp_display_shell_get_unit      (GimpDisplayShell   *shell);
+                                                    GimpUnit           *unit);
+GimpUnit        * gimp_display_shell_get_unit      (GimpDisplayShell   *shell);
 
 gboolean          gimp_display_shell_snap_coords   (GimpDisplayShell   *shell,
                                                     GimpCoords         *coords,
@@ -342,8 +358,10 @@ void              gimp_display_shell_set_mask      (GimpDisplayShell   *shell,
                                                     GeglBuffer         *mask,
                                                     gint                offset_x,
                                                     gint                offset_y,
-                                                    const GimpRGB      *color,
+                                                    GeglColor          *color,
                                                     gboolean            inverted);
 
+gboolean          gimp_display_shell_is_drawn      (GimpDisplayShell   *shell);
+
 
 #endif /* __GIMP_DISPLAY_SHELL_H__ */
