@@ -44,6 +44,9 @@
 #include "core/gimpcontainer.h"
 
 #include "widgets/gimpactiongroup.h"
+#include "widgets/gimpdeviceinfo.h"
+#include "widgets/gimpdevicemanager.h"
+#include "widgets/gimpdevices.h"
 #include "widgets/gimpdialogfactory.h"
 #include "widgets/gimpdock.h"
 #include "widgets/gimpdockbook.h"
@@ -128,6 +131,8 @@ struct _GimpImageWindowPrivate
   GtkWidget         *notebook;
   GtkWidget         *right_docks;
 
+  GHashTable        *pad_controllers;
+
   GdkWindowState     window_state;
 
   const gchar       *entry_id;
@@ -261,6 +266,11 @@ static GtkWidget *
                                                         GimpDisplayShell    *shell);
 static void      gimp_image_window_update_tab_labels   (GimpImageWindow     *window);
 
+static void      gimp_image_window_configure_pad       (GimpImageWindow     *window,
+                                                        GimpDeviceInfo      *info);
+static void      gimp_image_window_init_pad_foreach    (GimpDeviceInfo      *info,
+                                                        GimpImageWindow     *window);
+
 
 G_DEFINE_TYPE_WITH_CODE (GimpImageWindow, gimp_image_window, GIMP_TYPE_WINDOW,
                          G_ADD_PRIVATE (GimpImageWindow)
@@ -351,6 +361,7 @@ gimp_image_window_constructed (GObject *object)
   GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
   GimpUIManager          *menubar_manager;
   GimpGuiConfig          *config;
+  GimpDeviceManager      *device_manager;
   gboolean                use_gtk_menubar = TRUE;
   gboolean                use_app_menu    = TRUE;
 
@@ -547,6 +558,17 @@ gimp_image_window_constructed (GObject *object)
                                     NULL /*new_display*/,
                                     gimp_image_window_config_to_entry_id (config),
                                     private->initial_monitor);
+
+  private->pad_controllers =
+    g_hash_table_new_full (NULL, NULL, NULL, g_object_unref);
+
+  device_manager = gimp_devices_get_manager (private->gimp);
+  g_signal_connect_object (device_manager, "configure-pad",
+                           G_CALLBACK (gimp_image_window_configure_pad),
+                           window, G_CONNECT_SWAPPED);
+  gimp_container_foreach (GIMP_CONTAINER (device_manager),
+                          (GFunc) gimp_image_window_init_pad_foreach,
+                          window);
 }
 
 static void
@@ -568,6 +590,7 @@ gimp_image_window_dispose (GObject *object)
       private->update_ui_manager_idle_id = 0;
     }
 
+  g_clear_pointer (&private->pad_controllers, g_hash_table_unref);
   g_clear_object (&private->menubar_model);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -1170,9 +1193,6 @@ gimp_image_window_set_aux_info (GimpSessionManaged *session_managed,
   else if (StartupInfo.wShowWindow == SW_SHOWMINIMIZED   ||
            StartupInfo.wShowWindow == SW_SHOWMINNOACTIVE ||
            StartupInfo.wShowWindow == SW_MINIMIZE)
-    /* XXX Iconification does not seem to work. I see the
-     * window being iconified and immediately re-raised.
-     * I leave this piece of code for later improvement. */
     gtk_window_iconify (GTK_WINDOW (session_managed));
   else
     /* Another show property not relevant to min/max.
@@ -1966,10 +1986,29 @@ gimp_image_window_hide_tooltip (GimpUIManager   *manager,
 static gboolean
 gimp_image_window_update_ui_manager_idle (GimpImageWindow *window)
 {
-  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
+  GimpImageWindowPrivate *private      = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
+  GimpImage              *active_image = NULL;
+  GimpContext            *context;
 
   gimp_assert (private->active_shell != NULL);
 
+  context = gimp_get_user_context (private->active_shell->display->gimp);
+
+  if (context)
+    active_image = gimp_context_get_image (context);
+
+  /* Since we are running idle, it is possible this runs after the
+   * active display switched, and therefore we may call the wrong
+   * actions for an image. See #10441.
+   */
+  if ((active_image &&
+       (! private->active_shell->display || ! gimp_display_get_image (private->active_shell->display))) ||
+      active_image != gimp_display_get_image (private->active_shell->display))
+    {
+      private->update_ui_manager_idle_id = 0;
+      return G_SOURCE_REMOVE;
+    }
+
   gimp_ui_manager_update (menus_get_image_manager_singleton (private->active_shell->display->gimp),
                           private->active_shell->display);
 
@@ -2074,8 +2113,6 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
 
   gtk_window_set_title (GTK_WINDOW (window), shell->title);
 
-  gimp_display_shell_appearance_update (private->active_shell);
-
   if (gtk_widget_get_window (GTK_WIDGET (window)))
     {
       /*  we are fully initialized, use the window's current monitor
@@ -2100,6 +2137,8 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
   gimp_context_set_display (gimp_get_user_context (private->gimp),
                             active_display);
 
+  gimp_display_shell_appearance_update (private->active_shell);
+
   gimp_image_window_update_ui_manager (window);
 
   gimp_image_window_update_tab_labels (window);
@@ -2466,3 +2505,33 @@ gimp_image_window_update_tab_labels (GimpImageWindow *window)
 
   g_list_free (children);
 }
+
+static void
+gimp_image_window_configure_pad (GimpImageWindow *window,
+                                 GimpDeviceInfo  *info)
+{
+  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
+  GtkPadController       *controller;
+
+  g_hash_table_remove (private->pad_controllers, info);
+
+  if (gimp_device_info_get_device (info, NULL) != NULL)
+    {
+      controller =
+        gimp_device_info_create_pad_controller (info, GIMP_WINDOW (window));
+      if (controller)
+        g_hash_table_insert (private->pad_controllers, info, controller);
+    }
+}
+
+static void
+gimp_image_window_init_pad_foreach (GimpDeviceInfo  *info,
+                                    GimpImageWindow *window)
+{
+  if (gimp_device_info_get_device (info, NULL) == NULL)
+    return;
+  if (gimp_device_info_get_source (info) != GDK_SOURCE_TABLET_PAD)
+    return;
+
+  gimp_image_window_configure_pad (window, info);
+}
