@@ -1,9 +1,9 @@
 /* GIMP - The GNU Image Manipulation Program
  * Copyright (C) 1995 Spencer Kimball and Peter Mattis
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -12,15 +12,19 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __GUI_VTABLE_H__
 #define __GUI_VTABLE_H__
 
 
-void   gui_vtable_init (Gimp *gimp);
+void         gui_vtable_init         (Gimp *gimp);
+
+/*  this function lives in gui.c but must only be used from gui-vtable.c;
+ *  also, gui.h can't contain any Gdk types.
+ */
+GdkMonitor * gui_get_initial_monitor (Gimp *gimp);
 
 
 #endif /* __GUI_VTABLE_H__ */
