@@ -53,6 +53,7 @@
 #include <babl/babl.h>
 
 #include "libgimpbase/gimpbase.h"
+#include "libgimpbase/gimpbase-private.h"
 
 #include "pdb/pdb-types.h"
 
@@ -74,13 +75,6 @@
 #include "unique.h"
 
 #ifdef G_OS_WIN32
-/* To get PROCESS_DEP_* defined we need _WIN32_WINNT at 0x0601. We still
- * use the API optionally only if present, though.
- */
-#ifdef _WIN32_WINNT
-#undef _WIN32_WINNT
-#endif
-#define _WIN32_WINNT 0x0601
 #include <windows.h>
 #include <conio.h>
 #endif
@@ -147,7 +141,7 @@ static gboolean            use_cpu_accel     = TRUE;
 static gboolean            console_messages  = FALSE;
 static gboolean            use_debug_handler = FALSE;
 
-#ifdef GIMP_UNSTABLE
+#if defined (GIMP_UNSTABLE) || ! defined (GIMP_RELEASE)
 static gboolean            show_playground   = TRUE;
 static gboolean            show_debug_menu   = TRUE;
 static GimpStackTraceMode  stack_trace_mode  = GIMP_STACK_TRACE_QUERY;
@@ -434,12 +428,6 @@ gimp_macos_setenv (const char * progname)
       tmp = g_strdup_printf ("%s/share/libwmf/fonts", res_dir);
       g_setenv ("WMF_FONTDIR", tmp, TRUE);
       g_free (tmp);
-      if (g_getenv ("DYLD_LIBRARY_PATH"))
-        tmp = g_strdup_printf ("%s/lib:%s", res_dir, g_getenv ("DYLD_LIBRARY_PATH"));
-      else
-        tmp = g_strdup_printf ("%s/lib", res_dir);
-      g_setenv ("DYLD_LIBRARY_PATH", tmp, TRUE);
-      g_free (tmp);
       if (g_getenv ("XDG_DATA_DIRS"))
         tmp = g_strdup_printf ("%s/share:%s", res_dir, g_getenv ("XDG_DATA_DIRS"));
       else
@@ -498,7 +486,7 @@ gimp_early_configuration (void)
   language = gimp_early_rc_get_language (earlyrc);
 
   /*  change the locale if a language if specified  */
-  language_init (language);
+  language_init (language, NULL);
   if (language)
     g_free (language);
 
@@ -607,6 +595,9 @@ main (int    argc,
   gimp_init_signal_handlers (&backtrace_file);
 
 #ifdef G_OS_WIN32
+  /* Enable Anti-Aliasing*/
+  g_setenv ("PANGOCAIRO_BACKEND", "fc", TRUE);
+
   /* Reduce risks */
   SetDllDirectoryW (L"");
 
