@@ -159,12 +159,7 @@ gimp_smudge_paint (GimpPaintCore    *paint_core,
         if (options->flow > 0.0 &&
             ! gimp_dynamics_is_output_enabled (dynamics, GIMP_DYNAMICS_OUTPUT_COLOR) &&
             ! (brush_core->brush && gimp_brush_get_pixmap (brush_core->brush)))
-          {
-            GimpRGB foreground;
-
-            gimp_context_get_foreground (context, &foreground);
-            gimp_palettes_add_color_history (context->gimp, &foreground);
-          }
+          gimp_palettes_add_color_history (context->gimp, gimp_context_get_foreground (context));
       }
       break;
 
@@ -364,8 +359,7 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
   gdouble             flow;
   gdouble             grad_point;
   /* brush color */
-  GimpRGB             brush_color;
-  GimpRGB            *brush_color_ptr; /* whether use single color or pixmap */
+  GeglColor          *brush_color  = NULL; /* whether use single color or pixmap */
   /* accum buffer */
   gint                x, y;
   GeglBuffer         *accum_buffer;
@@ -401,6 +395,9 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
   coords    = *(gimp_symmetry_get_origin (sym));
   coords.x -= off_x;
   coords.y -= off_y;
+  gimp_symmetry_set_origin (sym, drawable, &coords);
+  paint_core->sym = sym;
+
   opacity = gimp_dynamics_get_linear_value (dynamics,
                                             GIMP_DYNAMICS_OUTPUT_OPACITY,
                                             &coords,
@@ -436,7 +433,6 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
                                                fade_point);
 
   /* Get current gradient color, brush pixmap, or foreground color */
-  brush_color_ptr = &brush_color;
   if (gimp_paint_options_get_gradient_color (paint_options, image,
                                              grad_point,
                                              paint_core->pixel_dist,
@@ -446,26 +442,16 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
     }
   else if (brush_core->brush && gimp_brush_get_pixmap (brush_core->brush))
     {
-      brush_color_ptr = NULL;
     }
   else
     {
-      gimp_context_get_foreground (context, &brush_color);
+      brush_color = g_object_ref (gimp_context_get_foreground (context));
     }
 
-  /* Convert to linear RGBA */
-  if (brush_color_ptr)
-    gimp_pickable_rgb_to_pixel (dest_pickable,
-                                &brush_color,
-                                babl_format ("RGBA double"),
-                                &brush_color);
-
   n_strokes = gimp_symmetry_get_size (sym);
   for (i = 0; i < n_strokes; i++)
     {
       coords    = *(gimp_symmetry_get_coords (sym, i));
-      coords.x -= off_x;
-      coords.y -= off_y;
 
       gimp_brush_core_eval_transform_symmetry (brush_core, sym, i);
 
@@ -508,7 +494,7 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
        * gimp_gegl_smudge_with_paint() instead of calling
        * gegl_buffer_set_color() to reduce gegl's internal processing.
        */
-      if (! brush_color_ptr && flow > 0.0)
+      if (! brush_color && flow > 0.0)
         {
           gimp_brush_core_color_area_with_pixmap (brush_core, drawable,
                                                   &coords,
@@ -530,7 +516,7 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
                                                    dest_pickable_off_y,
                                                    paint_buffer_width,
                                                    paint_buffer_height),
-                                   brush_color_ptr,
+                                   brush_color,
                                    paint_buffer,
                                    options->no_erasing,
                                    flow,
@@ -553,6 +539,8 @@ gimp_smudge_motion (GimpPaintCore    *paint_core,
                                       force,
                                       GIMP_PAINT_INCREMENTAL);
     }
+
+  g_clear_object (&brush_color);
 }
 
 static void
