@@ -12,7 +12,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef __PLUG_IN_TYPES_H__
@@ -24,6 +24,10 @@
 #include "plug-in/plug-in-enums.h"
 
 
+#define GIMP_PLUG_IN_TILE_WIDTH  128
+#define GIMP_PLUG_IN_TILE_HEIGHT 128
+
+
 typedef struct _GimpPlugIn           GimpPlugIn;
 typedef struct _GimpPlugInDebug      GimpPlugInDebug;
 typedef struct _GimpPlugInDef        GimpPlugInDef;
