@@ -34,6 +34,8 @@
 
 #include "text-types.h"
 
+#include "gimpfont.h"
+
 #include "core/gimp.h"
 #include "core/gimp-memsize.h"
 #include "core/gimp-utils.h"
@@ -139,8 +141,8 @@ gimp_text_class_init (GimpTextClass *klass)
 {
   GObjectClass    *object_class      = G_OBJECT_CLASS (klass);
   GimpObjectClass *gimp_object_class = GIMP_OBJECT_CLASS (klass);
-  GimpRGB          black;
-  GimpRGB          gray;
+  GeglColor       *black             = gegl_color_new ("black");
+  GeglColor       *gray              = gegl_color_new ("gray");
   GimpMatrix2      identity;
   gchar           *language;
   GParamSpec      *array_spec;
@@ -160,8 +162,6 @@ gimp_text_class_init (GimpTextClass *klass)
 
   gimp_object_class->get_memsize            = gimp_text_get_memsize;
 
-  gimp_rgba_set (&black, 0.0, 0.0, 0.0, GIMP_OPACITY_OPAQUE);
-  gimp_rgba_set (&gray, 0.75, 0.75, 0.75, GIMP_OPACITY_OPAQUE);
   gimp_matrix2_identity (&identity);
 
   GIMP_CONFIG_PROP_STRING (object_class, PROP_TEXT,
@@ -176,11 +176,9 @@ gimp_text_class_init (GimpTextClass *klass)
                            NULL,
                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_STRING (object_class, PROP_FONT,
-                           "font",
-                           NULL, NULL,
-                           "Sans-serif",
-                           GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_FONT (object_class, PROP_FONT,
+                         "font", NULL, NULL,
+                         GIMP_CONFIG_PARAM_FLAGS);
 
   GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_FONT_SIZE,
                            "font-size",
@@ -194,7 +192,7 @@ gimp_text_class_init (GimpTextClass *klass)
   GIMP_CONFIG_PROP_UNIT (object_class, PROP_UNIT,
                          "font-size-unit",
                          NULL, NULL,
-                         TRUE, FALSE, GIMP_UNIT_PIXEL,
+                         TRUE, FALSE, gimp_unit_pixel (),
                          GIMP_PARAM_STATIC_STRINGS);
 
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_ANTIALIAS,
@@ -235,11 +233,11 @@ gimp_text_class_init (GimpTextClass *klass)
                          GIMP_TEXT_DIRECTION_LTR,
                          GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_RGB (object_class, PROP_COLOR,
-                        "color",
-                        NULL, NULL,
-                        FALSE, &black,
-                        GIMP_PARAM_STATIC_STRINGS);
+  GIMP_CONFIG_PROP_COLOR (object_class, PROP_COLOR,
+                          "color",
+                          NULL, NULL,
+                          FALSE, black,
+                          GIMP_PARAM_STATIC_STRINGS);
 
   GIMP_CONFIG_PROP_ENUM (object_class, PROP_OUTLINE,
                          "outline",
@@ -301,7 +299,7 @@ gimp_text_class_init (GimpTextClass *klass)
   GIMP_CONFIG_PROP_UNIT (object_class, PROP_BOX_UNIT,
                          "box-unit",
                          NULL, NULL,
-                         TRUE, FALSE, GIMP_UNIT_PIXEL,
+                         TRUE, FALSE, gimp_unit_pixel (),
                          GIMP_PARAM_STATIC_STRINGS);
 
   GIMP_CONFIG_PROP_MATRIX2 (object_class, PROP_TRANSFORMATION,
@@ -341,10 +339,10 @@ gimp_text_class_init (GimpTextClass *klass)
                             "outline-pattern", NULL, NULL,
                             GIMP_TYPE_PATTERN,
                             GIMP_PARAM_STATIC_STRINGS);
-   GIMP_CONFIG_PROP_RGB (object_class, PROP_OUTLINE_FOREGROUND,
-                         "outline-foreground", NULL, NULL,
-                         FALSE, &gray,
-                         GIMP_PARAM_STATIC_STRINGS);
+   GIMP_CONFIG_PROP_COLOR (object_class, PROP_OUTLINE_FOREGROUND,
+                           "outline-foreground", NULL, NULL,
+                           FALSE, gray,
+                           GIMP_PARAM_STATIC_STRINGS);
    GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_OUTLINE_WIDTH,
                             "outline-width", NULL, NULL,
                             0.0, 8192.0, 4.0,
@@ -394,11 +392,15 @@ gimp_text_class_init (GimpTextClass *klass)
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_CONSTRUCT_ONLY));
 
+  g_object_unref (black);
+  g_object_unref (gray);
 }
 
 static void
 gimp_text_init (GimpText *text)
 {
+  text->color              = gegl_color_new ("black");
+  text->outline_foreground = gegl_color_new ("gray");
 }
 
 static void
@@ -415,8 +417,10 @@ gimp_text_finalize (GObject *object)
 
   g_clear_pointer (&text->text,     g_free);
   g_clear_pointer (&text->markup,   g_free);
-  g_clear_pointer (&text->font,     g_free);
   g_clear_pointer (&text->language, g_free);
+  g_clear_object (&text->font);
+  g_clear_object (&text->color);
+  g_clear_object (&text->outline_foreground);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -438,13 +442,13 @@ gimp_text_get_property (GObject      *object,
       g_value_set_string (value, text->markup);
       break;
     case PROP_FONT:
-      g_value_set_string (value, text->font);
+      g_value_set_object (value, text->font);
       break;
     case PROP_FONT_SIZE:
       g_value_set_double (value, text->font_size);
       break;
     case PROP_UNIT:
-      g_value_set_int (value, text->unit);
+      g_value_set_object (value, text->unit);
       break;
     case PROP_ANTIALIAS:
       g_value_set_boolean (value, text->antialias);
@@ -462,7 +466,7 @@ gimp_text_get_property (GObject      *object,
       g_value_set_string (value, text->language);
       break;
     case PROP_COLOR:
-      g_value_set_boxed (value, &text->color);
+      g_value_set_object (value, text->color);
       break;
     case PROP_OUTLINE:
       g_value_set_enum (value, text->outline);
@@ -489,7 +493,7 @@ gimp_text_get_property (GObject      *object,
       g_value_set_double (value, text->box_height);
       break;
     case PROP_BOX_UNIT:
-      g_value_set_int (value, text->box_unit);
+      g_value_set_object (value, text->box_unit);
       break;
     case PROP_TRANSFORMATION:
       g_value_set_boxed (value, &text->transformation);
@@ -504,7 +508,7 @@ gimp_text_get_property (GObject      *object,
       g_value_set_enum (value, text->outline_style);
       break;
     case PROP_OUTLINE_FOREGROUND:
-      g_value_set_boxed (value, &text->outline_foreground);
+      g_value_set_object (value, text->outline_foreground);
       break;
     case PROP_OUTLINE_PATTERN:
       g_value_set_object (value, text->outline_pattern);
@@ -555,7 +559,6 @@ gimp_text_set_property (GObject      *object,
                         GParamSpec   *pspec)
 {
   GimpText    *text = GIMP_TEXT (object);
-  GimpRGB     *color;
   GimpMatrix2 *matrix;
 
   switch (property_id)
@@ -580,30 +583,17 @@ gimp_text_set_property (GObject      *object,
       break;
     case PROP_FONT:
       {
-        const gchar *font = g_value_get_string (value);
+        GimpFont *font = g_value_get_object (value);
 
-        g_free (text->font);
-
-        if (font)
-          {
-            gsize len = strlen (font);
-
-            if (g_str_has_suffix (font, " Not-Rotated"))
-              len -= strlen ( " Not-Rotated");
-
-            text->font = g_strndup (font, len);
-          }
-        else
-          {
-            text->font = NULL;
-          }
+        if (font != text->font && font != NULL)
+          g_set_object (&text->font, font);
       }
       break;
     case PROP_FONT_SIZE:
       text->font_size = g_value_get_double (value);
       break;
     case PROP_UNIT:
-      text->unit = g_value_get_int (value);
+      text->unit = g_value_get_object (value);
       break;
     case PROP_ANTIALIAS:
       text->antialias = g_value_get_boolean (value);
@@ -622,8 +612,7 @@ gimp_text_set_property (GObject      *object,
       text->base_dir = g_value_get_enum (value);
       break;
     case PROP_COLOR:
-      color = g_value_get_boxed (value);
-      text->color = *color;
+      g_set_object (&text->color, g_value_get_object (value));;
       break;
     case PROP_OUTLINE:
       text->outline = g_value_get_enum (value);
@@ -650,7 +639,7 @@ gimp_text_set_property (GObject      *object,
       text->box_height = g_value_get_double (value);
       break;
     case PROP_BOX_UNIT:
-      text->box_unit = g_value_get_int (value);
+      text->box_unit = g_value_get_object (value);
       break;
     case PROP_TRANSFORMATION:
       matrix = g_value_get_boxed (value);
@@ -666,8 +655,7 @@ gimp_text_set_property (GObject      *object,
       text->outline_style = g_value_get_enum (value);
       break;
     case PROP_OUTLINE_FOREGROUND:
-      color                    = g_value_get_boxed (value);
-      text->outline_foreground = *color;
+      g_set_object (&text->outline_foreground, g_value_get_object (value));;
       break;
     case PROP_OUTLINE_PATTERN:
       {
@@ -748,7 +736,6 @@ gimp_text_get_memsize (GimpObject *object,
 
   memsize += gimp_string_get_memsize (text->text);
   memsize += gimp_string_get_memsize (text->markup);
-  memsize += gimp_string_get_memsize (text->font);
   memsize += gimp_string_get_memsize (text->language);
 
   return memsize + GIMP_OBJECT_CLASS (parent_class)->get_memsize (object,
@@ -795,6 +782,98 @@ gimp_text_serialize_property (GimpConfig       *config,
 
       gimp_config_writer_close (writer);
 
+      return TRUE;
+    }
+  else if (property_id == PROP_MARKUP)
+    {
+      gchar         *markup = (gchar*)g_value_get_string (value);
+      GRegex        *regex;
+      GimpText      *text;
+      GimpContainer *container;
+      PangoAttrList *attr_list;
+      GimpFont      *font;
+      guint          length;
+      GSList        *list   = NULL;
+      GSList        *fonts  = NULL;
+
+      g_return_val_if_fail (GIMP_IS_TEXT (config), FALSE);
+
+      if (markup == NULL)
+        return FALSE;
+
+      text      = GIMP_TEXT (config);
+      container = gimp_data_factory_get_container (text->gimp->font_factory);
+
+      /*lookupname format is "gimpfont%d" we keep only the "font%d" part
+       * this is to avoid problems when deserializing
+       * e.g. if  there are 2 fonts with lookupname gimpfont17 and gimpfont23
+       * we might replace the first with a font whose lookupname is gimpfont23,
+       * and we might replace the original gimpfont23 with say gimpfont29
+       * this means that all occurences of gimpfont23 turned into gimpfont29
+       */
+      regex  = g_regex_new ("\"gimpfont(\\d+)\"", 0, 0, NULL);
+      markup = g_regex_replace (regex, markup, -1, 0, "\"font\\1\"", 0, NULL);
+
+      gimp_config_writer_open   (writer, "markup");
+      gimp_config_writer_string (writer, markup);
+
+      pango_parse_markup (markup, -1, 0, &attr_list, NULL, NULL, NULL);
+
+      list   = pango_attr_list_get_attributes (attr_list);
+      length = g_slist_length (list);
+
+      for (guint i = 0; i < length; ++i)
+        {
+          PangoAttrFontDesc *attr_font_desc = pango_attribute_as_font_desc ((PangoAttribute*)g_slist_nth_data (list, i));
+
+          if (attr_font_desc != NULL)
+            {
+              gchar *altered_font_name = pango_font_description_to_string (attr_font_desc->desc);
+              gchar *font_name         = g_strdup_printf ("gimp%s", altered_font_name);
+
+              if (g_slist_find_custom (fonts, (gconstpointer) font_name, (GCompareFunc) g_strcmp0) == NULL)
+                {
+                  fonts = g_slist_prepend (fonts, (gpointer) font_name);
+
+                  if (g_str_has_prefix (altered_font_name, "font"))
+                    font = GIMP_FONT (gimp_container_search (container,
+                                                             (GimpContainerSearchFunc) gimp_font_match_by_lookup_name,
+                                                             (gpointer) font_name));
+                  /* in case the font is an alias its lookupname is that alias and doesn't conform to the format "gimpfont%d"*/
+                  else
+                    font = GIMP_FONT (gimp_container_search (container,
+                                                             (GimpContainerSearchFunc) gimp_font_match_by_lookup_name,
+                                                             (gpointer) altered_font_name));
+
+                  gimp_config_writer_open   (writer, "markupfont");
+                  /*lookupname format is "gimpfont%d" we keep only the "font%d",
+                   * and in case the font is an alias, we keep its name (see the above comments)*/
+                  gimp_config_writer_string (writer, font_name+4);
+
+                  gimp_config_writer_open   (writer, "font");
+                  GIMP_CONFIG_GET_IFACE     (GIMP_CONFIG (font))->serialize (GIMP_CONFIG (font),
+                                             writer,
+                                             NULL);
+                  gimp_config_writer_close  (writer);
+                  gimp_config_writer_close  (writer);
+                }
+
+              else
+                {
+                  g_free (font_name);
+                }
+              g_free (altered_font_name);
+            }
+        }
+
+      gimp_config_writer_close  (writer);
+
+      g_slist_free_full (fonts, (GDestroyNotify) g_free);
+      g_slist_free_full (list,  (GDestroyNotify) pango_attribute_destroy);
+      pango_attr_list_unref (attr_list);
+      g_free (markup);
+      g_regex_unref (regex);
+
       return TRUE;
     }
 
@@ -850,6 +929,92 @@ gimp_text_deserialize_property (GimpConfig *object,
           return TRUE;
         }
     }
+  else if (property_id == PROP_MARKUP)
+    {
+      gchar    *markup       = NULL;
+      GString  *markup_str;
+      GimpFont *dummy_object = g_object_new (GIMP_TYPE_FONT, NULL);
+
+      gimp_scanner_parse_string (scanner, &markup);
+
+      markup_str = g_string_new (markup);
+
+      /* This is for backward compatibility with older xcf files.*/
+      if (g_scanner_peek_next_token (scanner) == G_TOKEN_STRING)
+        {
+          while (g_scanner_peek_next_token (scanner) == G_TOKEN_STRING)
+            {
+              gchar    *markup_fontname = NULL;
+              gchar    *replaced_markup;
+              gchar    *new_markup;
+              GimpFont *font;
+
+              gimp_scanner_parse_string (scanner, &markup_fontname);
+
+              font = GIMP_FONT (GIMP_CONFIG_GET_IFACE (dummy_object)->deserialize_create (GIMP_TYPE_FONT,
+                                                                                          scanner,
+                                                                                          -1,
+                                                                                          NULL));
+              replaced_markup = g_markup_printf_escaped (" font=\"%s\"", markup_fontname);
+              new_markup      = g_strdup_printf (" gimpfont=\"%s\"", gimp_font_get_lookup_name (font));
+              g_string_replace (markup_str, replaced_markup, new_markup, 0);
+
+              g_free (markup_fontname);
+              g_free (replaced_markup);
+              g_free (new_markup);
+              g_object_unref (font);
+            }
+          /* We avoid the edge case when actual fonts are called "gimpfont%d"
+           * and their replacement name chains to each other by marking already
+           * processed font as "gimpfont=". Then we clean up this fake attribute
+           * name in the end.
+           * This is not a problem with the new format as font are stored as
+           * "font%d" (see comment in gimp_text_serialize_property()).
+           */
+          g_string_replace (markup_str, " gimpfont=\"", " font=\"", 0);
+        }
+      else
+        {
+          while (g_scanner_peek_next_token (scanner) == G_TOKEN_LEFT_PAREN)
+            {
+              gchar    *lookupname = NULL;
+              gchar    *replaced_markup;
+              gchar    *new_markup;
+              GimpFont *font;
+
+              g_scanner_get_next_token  (scanner); /* ( */
+              g_scanner_get_next_token  (scanner); /* "lookupname" */
+              gimp_scanner_parse_string (scanner, &lookupname);
+
+              g_scanner_get_next_token  (scanner); /* ) */
+              g_scanner_get_next_token  (scanner); /* font */
+
+              font = GIMP_FONT (GIMP_CONFIG_GET_IFACE (dummy_object)->deserialize_create (GIMP_TYPE_FONT,
+                                                                                          scanner,
+                                                                                          -1,
+                                                                                          NULL));
+              g_scanner_get_next_token  (scanner); /* ) */
+              g_scanner_get_next_token  (scanner); /* ) */
+
+              replaced_markup = g_markup_printf_escaped (" font=\"%s\"", lookupname);
+              new_markup      = g_strdup_printf (" font=\"%s\"", gimp_font_get_lookup_name (font));
+              g_string_replace (markup_str, replaced_markup, new_markup, 0);
+
+              g_free (lookupname);
+              g_free (replaced_markup);
+              g_free (new_markup);
+              g_object_unref (font);
+            }
+        }
+
+      g_value_set_string (value, markup_str->str);
+
+      g_object_unref (dummy_object);
+      g_string_free  (markup_str, TRUE);
+      g_free (markup);
+
+      return TRUE;
+    }
 
   return FALSE;
 }
