@@ -38,7 +38,7 @@
 #include "core/gimppickable.h"
 #include "core/gimppickable-auto-shrink.h"
 
-#include "vectors/gimpvectors.h"
+#include "vectors/gimppath.h"
 
 #include "widgets/gimphelp-ids.h"
 #include "widgets/gimpwidgets-utils.h"
@@ -342,7 +342,7 @@ gimp_align_tool_button_release (GimpTool              *tool,
 
       if (object == NULL)
         {
-          GimpVectors *vectors;
+          GimpPath    *path;
           GimpGuide   *guide;
           GimpLayer   *layer;
           GObject     *previously_picked;
@@ -351,12 +351,12 @@ gimp_align_tool_button_release (GimpTool              *tool,
           previously_picked = gimp_align_options_get_reference (options, FALSE);
 
 
-          if ((vectors = gimp_image_pick_vectors (image,
-                                                  coords->x, coords->y,
-                                                  FUNSCALEX (shell, snap_distance),
-                                                  FUNSCALEY (shell, snap_distance))))
+          if ((path = gimp_image_pick_path (image,
+                                               coords->x, coords->y,
+                                               FUNSCALEX (shell, snap_distance),
+                                               FUNSCALEY (shell, snap_distance))))
             {
-              object = G_OBJECT (vectors);
+              object = G_OBJECT (path);
             }
           else if (gimp_display_shell_get_show_guides (shell) &&
                    (guide = gimp_image_pick_guide (image,
@@ -438,10 +438,10 @@ gimp_align_tool_oper_update (GimpTool         *tool,
 
   align_tool->function = ALIGN_TOOL_NO_ACTION;
 
-  if (gimp_image_pick_vectors (image,
-                               coords->x, coords->y,
-                               FUNSCALEX (shell, snap_distance),
-                               FUNSCALEY (shell, snap_distance)))
+  if (gimp_image_pick_path (image,
+                            coords->x, coords->y,
+                            FUNSCALEX (shell, snap_distance),
+                            FUNSCALEY (shell, snap_distance)))
     {
       if (options->align_reference == GIMP_ALIGN_REFERENCE_PICK)
         align_tool->function = ALIGN_TOOL_REF_PICK_PATH;
@@ -632,7 +632,8 @@ gimp_align_tool_draw (GimpDrawTool *draw_tool)
       x += off_x;
       y += off_y;
 
-      if (gimp_align_options_align_contents (options))
+      if (gimp_align_options_align_contents (options) &&
+          GIMP_IS_PICKABLE (reference))
         {
           gint shrunk_x;
           gint shrunk_y;
