@@ -35,6 +35,8 @@
 #include "core/gimpasyncset.h"
 #include "core/gimpcontext.h"
 #include "core/gimpdatafactory.h"
+#include "core/gimpdrawable-filters.h"
+#include "core/gimpdrawablefilter.h"
 #include "core/gimperror.h"
 #include "core/gimpimage.h"
 #include "core/gimp-palettes.h"
@@ -42,20 +44,21 @@
 #include "core/gimpimage-undo.h"
 #include "core/gimpimage-undo-push.h"
 #include "core/gimplayer-floating-selection.h"
+#include "core/gimplist.h"
 #include "core/gimptoolinfo.h"
 #include "core/gimpundostack.h"
 
 #include "menus/menus.h"
 
 #include "text/gimptext.h"
-#include "text/gimptext-vectors.h"
+#include "text/gimptext-path.h"
 #include "text/gimptextlayer.h"
 #include "text/gimptextlayout.h"
 #include "text/gimptextundo.h"
 
 #include "vectors/gimpstroke.h"
-#include "vectors/gimpvectors.h"
-#include "vectors/gimpvectors-warp.h"
+#include "vectors/gimppath.h"
+#include "vectors/gimppath-warp.h"
 
 #include "widgets/gimpdialogfactory.h"
 #include "widgets/gimpdockcontainer.h"
@@ -192,7 +195,7 @@ static void    gimp_text_tool_buffer_end_edit   (GimpTextBuffer    *buffer,
 
 static void    gimp_text_tool_buffer_color_applied
                                                 (GimpTextBuffer    *buffer,
-                                                 const GimpRGB     *color,
+                                                 GeglColor         *color,
                                                  GimpTextTool      *text_tool);
 
 
@@ -290,7 +293,7 @@ gimp_text_tool_constructed (GObject *object)
 
   G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  text_tool->proxy = g_object_new (GIMP_TYPE_TEXT, NULL);
+  text_tool->proxy = g_object_new (GIMP_TYPE_TEXT, "gimp", tool->tool_info->gimp, NULL);
 
   gimp_text_options_connect_text (options, text_tool->proxy);
 
@@ -1089,6 +1092,26 @@ gimp_text_tool_frame_item (GimpTextTool *text_tool)
   gimp_tool_rectangle_frame_item (GIMP_TOOL_RECTANGLE (text_tool->widget),
                                   GIMP_ITEM (text_tool->layer));
 
+  /* Update crop of any filters applied to text */
+  if (text_tool->layer)
+  {
+    GList         *list;
+    GimpDrawable  *drawable = GIMP_DRAWABLE (text_tool->layer);
+    GimpContainer *filters  = gimp_drawable_get_filters (drawable);
+
+    for (list = GIMP_LIST (filters)->queue->tail;
+         list; list = g_list_previous (list))
+      {
+        GimpDrawableFilter *filter = list->data;
+
+        /* TODO: Handle partial layer effect */
+        gimp_drawable_filter_set_region (filter, GIMP_FILTER_REGION_SELECTION);
+        gimp_drawable_filter_set_region (filter, GIMP_FILTER_REGION_DRAWABLE);
+      }
+    if (list)
+      g_list_free (list);
+  }
+
   text_tool->handle_rectangle_change_complete = TRUE;
 }
 
@@ -1140,7 +1163,7 @@ gimp_text_tool_rectangle_change_complete (GimpToolRectangle *rectangle,
       if ((x2 - x1) != gimp_item_get_width  (item) ||
           (y2 - y1) != gimp_item_get_height (item))
         {
-          GimpUnit  box_unit = text_tool->proxy->box_unit;
+          GimpUnit *box_unit = text_tool->proxy->box_unit;
           gdouble   xres, yres;
           gboolean  push_undo = TRUE;
           GimpUndo *undo;
@@ -1658,8 +1681,8 @@ gimp_text_tool_create_layer (GimpTextTool *text_tool,
 
   if (text_tool->text_box_fixed)
     {
-      GimpUnit box_unit = text_tool->proxy->box_unit;
-      gdouble  xres, yres;
+      GimpUnit *box_unit = text_tool->proxy->box_unit;
+      gdouble   xres, yres;
 
       gimp_image_get_resolution (image, &xres, &yres);
 
@@ -1979,11 +2002,10 @@ gimp_text_tool_buffer_end_edit (GimpTextBuffer *buffer,
 
 static void
 gimp_text_tool_buffer_color_applied (GimpTextBuffer *buffer,
-                                     const GimpRGB  *color,
+                                     GeglColor      *color,
                                      GimpTextTool   *text_tool)
 {
-  gimp_palettes_add_color_history (GIMP_TOOL (text_tool)->tool_info->gimp,
-                                   color);
+  gimp_palettes_add_color_history (GIMP_TOOL (text_tool)->tool_info->gimp, color);
 }
 
 
@@ -2008,7 +2030,7 @@ gimp_text_tool_ensure_layout (GimpTextTool *text_tool)
       gimp_image_get_resolution (image, &xres, &yres);
 
       text_tool->layout = gimp_text_layout_new (text_tool->layer->text,
-                                                xres, yres, &error);
+                                                image, xres, yres, &error);
       if (error)
         {
           gimp_message_literal (image->gimp, NULL, GIMP_MESSAGE_ERROR, error->message);
@@ -2291,24 +2313,24 @@ gimp_text_tool_paste_clipboard (GimpTextTool *text_tool)
 void
 gimp_text_tool_create_vectors (GimpTextTool *text_tool)
 {
-  GimpVectors *vectors;
+  GimpPath *path;
 
   g_return_if_fail (GIMP_IS_TEXT_TOOL (text_tool));
 
   if (! text_tool->text || ! text_tool->image)
     return;
 
-  vectors = gimp_text_vectors_new (text_tool->image, text_tool->text);
+  path = gimp_text_path_new (text_tool->image, text_tool->text);
 
   if (text_tool->layer)
     {
       gint x, y;
 
       gimp_item_get_offset (GIMP_ITEM (text_tool->layer), &x, &y);
-      gimp_item_translate (GIMP_ITEM (vectors), x, y, FALSE);
+      gimp_item_translate (GIMP_ITEM (path), x, y, FALSE);
     }
 
-  gimp_image_add_vectors (text_tool->image, vectors,
+  gimp_image_add_path (text_tool->image, path,
                           GIMP_IMAGE_ACTIVE_PARENT, -1, TRUE);
 
   gimp_image_flush (text_tool->image);
@@ -2319,7 +2341,7 @@ gimp_text_tool_create_vectors_warped (GimpTextTool  *text_tool,
                                       GError       **error)
 {
   GList             *vectors0;
-  GimpVectors       *vectors;
+  GimpPath          *vectors;
   gdouble            box_width;
   gdouble            box_height;
   GimpTextDirection  dir;
@@ -2344,7 +2366,7 @@ gimp_text_tool_create_vectors_warped (GimpTextTool  *text_tool,
   box_width  = gimp_item_get_width  (GIMP_ITEM (text_tool->layer));
   box_height = gimp_item_get_height (GIMP_ITEM (text_tool->layer));
 
-  vectors0 = gimp_image_get_selected_vectors (text_tool->image);
+  vectors0 = gimp_image_get_selected_paths (text_tool->image);
   if (g_list_length (vectors0) != 1)
     {
       g_set_error_literal (error, GIMP_ERROR, GIMP_FAILED,
@@ -2352,7 +2374,7 @@ gimp_text_tool_create_vectors_warped (GimpTextTool  *text_tool,
       return FALSE;
     }
 
-  vectors = gimp_text_vectors_new (text_tool->image, text_tool->text);
+  vectors = gimp_text_path_new (text_tool->image, text_tool->text);
 
   offset = 0;
   dir = gimp_text_tool_get_direction (text_tool);
@@ -2369,7 +2391,7 @@ gimp_text_tool_create_vectors_warped (GimpTextTool  *text_tool,
       {
         GimpStroke *stroke = NULL;
 
-        while ((stroke = gimp_vectors_stroke_get_next (vectors, stroke)))
+        while ((stroke = gimp_path_stroke_get_next (vectors, stroke)))
           {
             gimp_stroke_rotate (stroke, 0, 0, 270);
             gimp_stroke_translate (stroke, 0, box_width);
@@ -2379,12 +2401,12 @@ gimp_text_tool_create_vectors_warped (GimpTextTool  *text_tool,
       break;
     }
 
-  gimp_vectors_warp_vectors (vectors0->data, vectors, offset);
+  gimp_path_warp_path (vectors0->data, vectors, offset);
 
   gimp_item_set_visible (GIMP_ITEM (vectors), TRUE, FALSE);
 
-  gimp_image_add_vectors (text_tool->image, vectors,
-                          GIMP_IMAGE_ACTIVE_PARENT, -1, TRUE);
+  gimp_image_add_path (text_tool->image, vectors,
+                       GIMP_IMAGE_ACTIVE_PARENT, -1, TRUE);
 
   gimp_image_flush (text_tool->image);
 
