@@ -24,6 +24,7 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+#include "libgimpbase/gimpbase.h"
 #include "libgimpmath/gimpmath.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
@@ -36,6 +37,8 @@
 
 #include "core/gimp.h"
 #include "core/gimpchannel.h"
+#include "core/gimpcontainer.h"
+#include "core/gimpdrawable-filters.h"
 #include "core/gimpdrawablefilter.h"
 #include "core/gimpimage.h"
 #include "core/gimplayer.h"
@@ -982,7 +985,8 @@ gimp_warp_tool_commit (GimpWarpTool *wt)
 
       gimp_warp_tool_set_sampler (wt, /* commit = */ TRUE);
 
-      gimp_drawable_filter_commit (wt->filter, GIMP_PROGRESS (tool), FALSE);
+      gimp_drawable_filter_commit (wt->filter, FALSE,
+                                   GIMP_PROGRESS (tool), FALSE);
       g_clear_object (&wt->filter);
 
       gimp_tool_control_pop_preserve (tool->control);
@@ -1246,13 +1250,30 @@ gimp_warp_tool_update_area (GimpWarpTool        *wt,
                             const GeglRectangle *area,
                             gboolean             synchronous)
 {
-  GeglRectangle rect;
+  GeglRectangle  rect;
+  GimpContainer *filters;
 
   if (! wt->filter)
     return;
 
   rect = gimp_warp_tool_get_invalidated_by_change (wt, area);
 
+  /* Move this operation below any non-destructive filters that
+   * may be active, so that it's directly affect the raw pixels. */
+  filters =
+    gimp_drawable_get_filters (gimp_drawable_filter_get_drawable (wt->filter));
+
+  if (gimp_container_have (filters, GIMP_OBJECT (wt->filter)))
+  {
+    gint end_index = gimp_container_get_n_children (filters) - 1;
+    gint index     = gimp_container_get_child_index (filters,
+                                                     GIMP_OBJECT (wt->filter));
+
+    if (end_index > 0 && index != end_index)
+      gimp_container_reorder (filters, GIMP_OBJECT (wt->filter),
+                              end_index);
+  }
+
   if (synchronous)
     {
       GimpTool  *tool  = GIMP_TOOL (wt);
@@ -1509,7 +1530,7 @@ gimp_warp_tool_animate (GimpWarpTool *wt)
   gimp_warp_tool_update_stroke (wt, NULL);
 
   widget = GTK_WIDGET (gimp_display_get_shell (tool->display));
-  gimp_create_display (orig_image->gimp, image, GIMP_UNIT_PIXEL, 1.0,
+  gimp_create_display (orig_image->gimp, image, gimp_unit_pixel (), 1.0,
                        G_OBJECT (gimp_widget_get_monitor (widget)));
   g_object_unref (image);
 }
