@@ -76,7 +76,7 @@ struct _GimpActionPrivate
 
   gchar                 *menu_path;
 
-  GimpRGB               *color;
+  GeglColor             *color;
   GimpViewable          *viewable;
   PangoEllipsizeMode     ellipsize;
   gint                   max_width_chars;
@@ -100,6 +100,8 @@ static void   gimp_action_proxy_button_activate  (GtkButton         *button,
 
 static void   gimp_action_update_proxy_sensitive (GimpAction        *action,
                                                   GtkWidget         *proxy);
+static void   gimp_action_update_proxy_visible   (GimpAction        *action,
+                                                  GtkWidget         *proxy);
 static void   gimp_action_update_proxy_tooltip   (GimpAction        *action,
                                                   GtkWidget         *proxy);
 
@@ -112,8 +114,6 @@ static guint action_signals[LAST_SIGNAL];
 static void
 gimp_action_default_init (GimpActionInterface *iface)
 {
-  GimpRGB black;
-
   action_signals[ACTIVATE] =
     g_signal_new ("activate",
                   G_TYPE_FROM_INTERFACE (iface),
@@ -180,12 +180,11 @@ gimp_action_default_init (GimpActionInterface *iface)
                                                             GIMP_PARAM_READWRITE |
                                                             G_PARAM_EXPLICIT_NOTIFY));
 
-  gimp_rgba_set (&black, 0.0, 0.0, 0.0, GIMP_OPACITY_OPAQUE);
   g_object_interface_install_property (iface,
-                                       gimp_param_spec_rgb ("color",
-                                                            NULL, NULL,
-                                                            TRUE, &black,
-                                                            GIMP_PARAM_READWRITE));
+                                       gimp_param_spec_color_from_string ("color",
+                                                                          NULL, NULL,
+                                                                          TRUE, "black",
+                                                                          GIMP_PARAM_READWRITE));
   g_object_interface_install_property (iface,
                                        g_param_spec_object ("viewable",
                                                             NULL, NULL,
@@ -442,9 +441,20 @@ void
 gimp_action_set_visible (GimpAction *action,
                          gboolean    visible)
 {
-  g_object_set (action,
-                "visible", visible,
-                NULL);
+  GimpActionPrivate *priv = GET_PRIVATE (action);
+
+  /* Only notify when the state actually changed. This is important for
+   * handlers such as visibility of menu items in GimpMenuModel which
+   * will assume that the action visibility changed. Otherwise we might
+   * remove items by mistake.
+   */
+  if (priv->visible != visible)
+    {
+      priv->visible = visible;
+
+      gimp_action_update_proxy_visible (action, NULL);
+      g_object_notify (G_OBJECT (action), "visible");
+    }
 }
 
 gboolean
@@ -689,6 +699,39 @@ gimp_action_use_default_accels (GimpAction *action)
   return TRUE;
 }
 
+gboolean
+gimp_action_is_default_accel (GimpAction  *action,
+                              const gchar *accel)
+{
+  gchar           **default_accels;
+  guint             accelerator_key  = 0;
+  GdkModifierType   accelerator_mods = 0;
+
+  g_return_val_if_fail (GIMP_IS_ACTION (action), TRUE);
+  g_return_val_if_fail (accel != NULL, TRUE);
+
+  gtk_accelerator_parse (accel, &accelerator_key, &accelerator_mods);
+  g_return_val_if_fail (accelerator_key != 0 || accelerator_mods == 0, FALSE);
+
+  default_accels = GET_PRIVATE (action)->default_accels;
+
+  if (default_accels == NULL)
+    return FALSE;
+
+  for (gint i = 0; default_accels[i] != NULL; i++)
+    {
+      guint           default_key;
+      GdkModifierType default_mods;
+
+      gtk_accelerator_parse (default_accels[i], &default_key, &default_mods);
+
+      if (default_key == accelerator_key && default_mods == accelerator_mods)
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
 const gchar *
 gimp_action_get_menu_path (GimpAction *action)
 {
@@ -853,7 +896,7 @@ gimp_action_get_property (GObject    *object,
       break;
 
     case GIMP_ACTION_PROP_COLOR:
-      g_value_set_boxed (value, priv->color);
+      g_value_set_object (value, priv->color);
       break;
     case GIMP_ACTION_PROP_VIEWABLE:
       g_value_set_object (value, priv->viewable);
@@ -892,16 +935,8 @@ gimp_action_set_property (GObject      *object,
                                  NULL);
       break;
     case GIMP_ACTION_PROP_VISIBLE:
-      if (priv->visible != g_value_get_boolean (value))
-        {
-          priv->visible = g_value_get_boolean (value);
-          /* Only notify when the state actually changed. This is important for
-           * handlers such as visibility of menu items in GimpMenuModel which
-           * will assume that the action visibility changed. Otherwise we might
-           * remove items by mistake.
-           */
-          g_object_notify (object, "visible");
-        }
+      gimp_action_set_visible (GIMP_ACTION (object),
+                               g_value_get_boolean (value));
       break;
 
     case GIMP_ACTION_PROP_LABEL:
@@ -921,8 +956,10 @@ gimp_action_set_property (GObject      *object,
       break;
 
     case GIMP_ACTION_PROP_COLOR:
-      g_clear_pointer (&priv->color, g_free);
-      priv->color = g_value_dup_boxed (value);
+      g_clear_object (&priv->color);
+      if (g_value_get_object (value))
+        priv->color = gegl_color_duplicate (g_value_get_object (value));
+
       set_proxy = TRUE;
       break;
     case GIMP_ACTION_PROP_VIEWABLE:
@@ -985,8 +1022,7 @@ gimp_action_set_proxy (GimpAction *action,
             {
               gint width, height;
 
-              proxy_image = gimp_color_area_new (priv->color,
-                                                 GIMP_COLOR_AREA_SMALL_CHECKS, 0);
+              proxy_image = gimp_color_area_new (priv->color, GIMP_COLOR_AREA_SMALL_CHECKS, 0);
               gimp_color_area_set_draw_border (GIMP_COLOR_AREA (proxy_image), TRUE);
 
               if (priv->context)
@@ -1206,9 +1242,16 @@ gimp_action_set_menu_path (GimpAction  *action,
 
   if (menu_path)
     {
-      paths = gimp_utils_break_menu_path (menu_path);
+      paths = gimp_utils_break_menu_path (menu_path, NULL, NULL);
+
+      /* MacOS does not support the "rightwards triangle arrowhead" symbol,
+       * so we'll use the MacOS submenu separator symbol per Lukas Oberhuber. */
+#ifdef PLATFORM_OSX
+      priv->menu_path = g_strjoinv (" > ", paths);
+#else
       /* The 4 raw bytes are the "rightwards triangle arrowhead" unicode character. */
       priv->menu_path = g_strjoinv (" \xF0\x9F\xA2\x92 ", paths);
+#endif
       g_strfreev (paths);
     }
 }
@@ -1245,7 +1288,7 @@ gimp_action_private_finalize (GimpActionPrivate *priv)
 {
   g_clear_pointer (&priv->disable_reason, g_free);
   g_clear_object  (&priv->context);
-  g_clear_pointer (&priv->color, g_free);
+  g_clear_object  (&priv->color);
   g_clear_object  (&priv->viewable);
 
   g_free (priv->label);
@@ -1354,6 +1397,24 @@ gimp_action_update_proxy_sensitive (GimpAction *action,
     }
 }
 
+static void
+gimp_action_update_proxy_visible (GimpAction *action,
+                                  GtkWidget  *proxy)
+{
+  GimpActionPrivate *priv    = GET_PRIVATE (action);
+  gboolean           visible = gimp_action_is_visible (action);
+
+  if (proxy)
+    {
+      gtk_widget_set_visible (proxy, visible);
+    }
+  else
+    {
+      for (GList *list = priv->proxies; list; list = list->next)
+        gtk_widget_set_visible (list->data, visible);
+    }
+}
+
 static void
 gimp_action_update_proxy_tooltip (GimpAction *action,
                                   GtkWidget  *proxy)
@@ -1361,9 +1422,10 @@ gimp_action_update_proxy_tooltip (GimpAction *action,
   GimpActionPrivate *priv;
   const gchar       *tooltip;
   const gchar       *help_id;
-  const gchar       *reason         = NULL;
-  gchar             *escaped_reason = NULL;
-  gchar             *markup;
+  const gchar       *reason          = NULL;
+  gchar             *escaped_tooltip = NULL;
+  gchar             *escaped_reason  = NULL;
+  gchar             *markup          = NULL;
 
   g_return_if_fail (GIMP_IS_ACTION (action));
 
@@ -1372,14 +1434,15 @@ gimp_action_update_proxy_tooltip (GimpAction *action,
   help_id = gimp_action_get_help_id (action);
 
   gimp_action_get_sensitive (action, &reason);
-  if (reason)
-    escaped_reason = g_markup_escape_text (reason, -1);
+  escaped_reason  = reason ? g_markup_escape_text (reason, -1) : NULL;
+  escaped_tooltip = tooltip ? g_markup_escape_text (tooltip, -1) : NULL;
 
-  markup = g_strdup_printf ("%s%s"                                   /* Action tooltip  */
-                            "<i><span weight='light'>%s</span></i>", /* Inactive reason */
-                            tooltip,
-                            escaped_reason && tooltip ? "\n" : "",
-                            escaped_reason ? escaped_reason : "");
+  if (escaped_tooltip || escaped_reason)
+    markup = g_strdup_printf ("%s%s"                                   /* Action tooltip  */
+                              "<i><span weight='light'>%s</span></i>", /* Inactive reason */
+                              escaped_tooltip ? escaped_tooltip : "",
+                              escaped_tooltip && escaped_reason ? "\n" : "",
+                              escaped_reason ? escaped_reason : "");
 
   /*  This hack makes sure we don't replace the tooltips of GimpButtons
    *  with extended callbacks (for Shift+Click etc.), because these
@@ -1401,6 +1464,7 @@ gimp_action_update_proxy_tooltip (GimpAction *action,
           gimp_help_set_help_data_with_markup (list->data, markup, help_id);
     }
 
+  g_free (escaped_tooltip);
   g_free (escaped_reason);
   g_free (markup);
 }
