@@ -115,6 +115,8 @@ const gchar    ** gimp_action_get_default_accels  (GimpAction      *action);
 const gchar    ** gimp_action_get_accels          (GimpAction      *action);
 gchar **          gimp_action_get_display_accels  (GimpAction      *action);
 gboolean          gimp_action_use_default_accels  (GimpAction      *action);
+gboolean          gimp_action_is_default_accel    (GimpAction      *action,
+                                                   const gchar     *accel);
 
 const gchar     * gimp_action_get_menu_path       (GimpAction      *action);
 
