@@ -79,6 +79,8 @@ static void   gimp_color_dialog_get_property     (GObject            *object,
                                                   GValue             *value,
                                                   GParamSpec         *pspec);
 
+static void   gimp_color_dialog_style_updated    (GtkWidget          *widget);
+
 static void   gimp_color_dialog_response         (GtkDialog          *dialog,
                                                   gint                response_id);
 
@@ -99,16 +101,14 @@ static void   gimp_color_dialog_color_changed    (GimpColorSelection *selection,
 static void   gimp_color_history_add_clicked     (GtkWidget          *widget,
                                                   GimpColorDialog    *dialog);
 
-static void   gimp_color_dialog_history_selected (GimpColorHistory   *history,
-                                                  const GimpRGB      *rgb,
-                                                  GimpColorDialog    *dialog);
-
 static void   gimp_color_dialog_image_changed    (GimpContext        *context,
                                                   GimpImage          *image,
                                                   GimpColorDialog    *dialog);
+static void   gimp_color_dialog_update_format    (GimpImage          *image,
+                                                  GimpColorDialog    *dialog);
 static void   gimp_color_dialog_update_simulation(GimpImage          *image,
                                                   GimpColorDialog    *dialog);
-static void   gimp_color_dialog_update           (GimpColorDialog    *dialog);
+static void   gimp_color_dialog_update_base_type (GimpColorDialog    *dialog);
 static void   gimp_color_dialog_show             (GimpColorDialog    *dialog);
 static void   gimp_color_dialog_hide             (GimpColorDialog    *dialog);
 
@@ -125,13 +125,16 @@ gimp_color_dialog_class_init (GimpColorDialogClass *klass)
 {
   GObjectClass   *object_class = G_OBJECT_CLASS (klass);
   GtkDialogClass *dialog_class = GTK_DIALOG_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
   object_class->constructed  = gimp_color_dialog_constructed;
   object_class->finalize     = gimp_color_dialog_finalize;
   object_class->set_property = gimp_color_dialog_set_property;
   object_class->get_property = gimp_color_dialog_get_property;
 
-  dialog_class->response     = gimp_color_dialog_response;
+  dialog_class->response      = gimp_color_dialog_response;
+
+  widget_class->style_updated = gimp_color_dialog_style_updated;
 
   color_dialog_signals[UPDATE] =
     g_signal_new ("update",
@@ -141,7 +144,7 @@ gimp_color_dialog_class_init (GimpColorDialogClass *klass)
                   NULL, NULL,
                   gimp_marshal_VOID__BOXED_ENUM,
                   G_TYPE_NONE, 2,
-                  GIMP_TYPE_RGB,
+                  GEGL_TYPE_COLOR,
                   GIMP_TYPE_COLOR_DIALOG_STATE);
 
   g_object_class_install_property (object_class, PROP_USER_CONTEXT_AWARE,
@@ -198,7 +201,7 @@ gimp_color_dialog_constructed (GObject *object)
    * dockable is opened.
    */
   gimp_help_set_help_data_with_markup (button,
-                                       NC_("colormap-action", "Edit this color"),
+                                       C_("colormap-action", "Edit this color"),
                                        GIMP_HELP_INDEXED_PALETTE_EDIT);
   g_signal_connect_swapped (button, "clicked",
                             G_CALLBACK (gimp_color_dialog_colormap_edit_activate),
@@ -216,7 +219,7 @@ gimp_color_dialog_constructed (GObject *object)
   gtk_widget_show (image);
 
   gimp_help_set_help_data_with_markup (button,
-                                       NC_("colormap-action", "Add current foreground color"),
+                                       C_("colormap-action", "Add current foreground color"),
                                        GIMP_HELP_INDEXED_PALETTE_ADD);
 
   g_signal_connect_swapped (button, "clicked",
@@ -259,9 +262,9 @@ gimp_color_dialog_constructed (GObject *object)
   gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET (history), TRUE, TRUE, 0);
   gtk_widget_show (GTK_WIDGET (history));
 
-  g_signal_connect (history, "color-selected",
-                    G_CALLBACK (gimp_color_dialog_history_selected),
-                    dialog);
+  g_signal_connect_swapped (history, "color-selected",
+                            G_CALLBACK (gimp_color_selection_set_color),
+                            dialog->selection);
 
   g_signal_connect (dialog, "show",
                     G_CALLBACK (gimp_color_dialog_show),
@@ -278,6 +281,8 @@ gimp_color_dialog_finalize (GObject *object)
   GimpColorDialog    *dialog          = GIMP_COLOR_DIALOG (object);
   GimpViewableDialog *viewable_dialog = GIMP_VIEWABLE_DIALOG (dialog);
 
+  g_clear_weak_pointer (&dialog->active_image);
+
   if (dialog->user_context_aware && viewable_dialog->context)
     {
       GimpContext *user_context = viewable_dialog->context->gimp->user_context;
@@ -330,6 +335,19 @@ gimp_color_dialog_get_property (GObject    *object,
     }
 }
 
+static void
+gimp_color_dialog_style_updated (GtkWidget *widget)
+{
+  GimpColorDialog   *dialog = GIMP_COLOR_DIALOG (widget);
+  GimpColorNotebook *notebook;
+
+  GTK_WIDGET_CLASS (parent_class)->style_updated (widget);
+
+  notebook =
+    GIMP_COLOR_NOTEBOOK (gimp_color_selection_get_notebook (GIMP_COLOR_SELECTION (dialog->selection)));
+  GTK_WIDGET_GET_CLASS (notebook)->style_updated (GTK_WIDGET (notebook));
+}
+
 static void
 gimp_color_dialog_response (GtkDialog *gtk_dialog,
                             gint       response_id)
@@ -339,7 +357,7 @@ gimp_color_dialog_response (GtkDialog *gtk_dialog,
   GimpImage             *image           = NULL;
   GimpColormapSelection *colormap_selection;
   gint                   col_index;
-  GimpRGB                color;
+  GeglColor             *color           = NULL;
 
   colormap_selection = GIMP_COLORMAP_SELECTION (dialog->colormap_selection);
   col_index = gimp_colormap_selection_get_index (colormap_selection, NULL);
@@ -358,52 +376,56 @@ gimp_color_dialog_response (GtkDialog *gtk_dialog,
       break;
 
     case GTK_RESPONSE_OK:
-      gimp_color_selection_get_color (GIMP_COLOR_SELECTION (dialog->selection),
-                                      &color);
+      color = gimp_color_selection_get_color (GIMP_COLOR_SELECTION (dialog->selection));
 
       if (dialog->colormap_editing && image)
         {
-          GimpRGB old_color;
+          GeglColor *old_color;
 
           dialog->colormap_editing = FALSE;
 
           /* Restore old color for undo */
-          gimp_color_selection_get_old_color (GIMP_COLOR_SELECTION (dialog->selection), &old_color);
-          gimp_image_set_colormap_entry (image, col_index, &old_color, FALSE);
-          gimp_image_set_colormap_entry (image, col_index, &color, TRUE);
+          old_color = gimp_color_selection_get_old_color (GIMP_COLOR_SELECTION (dialog->selection));
+          gimp_image_set_colormap_entry (image, col_index, old_color, FALSE);
+          gimp_image_set_colormap_entry (image, col_index, color, TRUE);
           gimp_image_flush (image);
 
           gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "colormap");
           g_signal_emit (dialog, color_dialog_signals[UPDATE], 0,
-                         &color, GIMP_COLOR_DIALOG_UPDATE);
+                         color, GIMP_COLOR_DIALOG_UPDATE);
+
+          g_object_unref (old_color);
         }
       else
         {
           g_signal_emit (dialog, color_dialog_signals[UPDATE], 0,
-                         &color, GIMP_COLOR_DIALOG_OK);
+                         color, GIMP_COLOR_DIALOG_OK);
         }
+
+      g_object_unref (color);
       break;
 
     default:
-      gimp_color_selection_get_old_color (GIMP_COLOR_SELECTION (dialog->selection),
-                                          &color);
+      color = gimp_color_selection_get_old_color (GIMP_COLOR_SELECTION (dialog->selection));
 
       if (dialog->colormap_editing && image)
         {
           dialog->colormap_editing = FALSE;
 
-          gimp_image_set_colormap_entry (image, col_index, &color, FALSE);
+          gimp_image_set_colormap_entry (image, col_index, color, FALSE);
           gimp_projection_flush (gimp_image_get_projection (image));
 
           gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "colormap");
           g_signal_emit (dialog, color_dialog_signals[UPDATE], 0,
-                         &color, GIMP_COLOR_DIALOG_UPDATE);
+                         color, GIMP_COLOR_DIALOG_UPDATE);
         }
       else
         {
           g_signal_emit (dialog, color_dialog_signals[UPDATE], 0,
-                         &color, GIMP_COLOR_DIALOG_CANCEL);
+                         color, GIMP_COLOR_DIALOG_CANCEL);
         }
+
+      g_object_unref (color);
       break;
     }
 }
@@ -421,7 +443,7 @@ gimp_color_dialog_new (GimpViewable      *viewable,
                        GtkWidget         *parent,
                        GimpDialogFactory *dialog_factory,
                        const gchar       *dialog_identifier,
-                       const GimpRGB     *color,
+                       GeglColor         *color,
                        gboolean           wants_updates,
                        gboolean           show_alpha)
 {
@@ -436,7 +458,7 @@ gimp_color_dialog_new (GimpViewable      *viewable,
                         GIMP_IS_DIALOG_FACTORY (dialog_factory), NULL);
   g_return_val_if_fail (dialog_factory == NULL || dialog_identifier != NULL,
                         NULL);
-  g_return_val_if_fail (color != NULL, NULL);
+  g_return_val_if_fail (GEGL_IS_COLOR (color), NULL);
 
   role = dialog_identifier ? dialog_identifier : "gimp-color-selector";
 
@@ -517,10 +539,10 @@ gimp_color_dialog_new (GimpViewable      *viewable,
 
 void
 gimp_color_dialog_set_color (GimpColorDialog *dialog,
-                             const GimpRGB   *color)
+                             GeglColor       *color)
 {
   g_return_if_fail (GIMP_IS_COLOR_DIALOG (dialog));
-  g_return_if_fail (color != NULL);
+  g_return_if_fail (GEGL_IS_COLOR (color));
 
   g_signal_handlers_block_by_func (dialog->selection,
                                    gimp_color_dialog_color_changed,
@@ -530,21 +552,21 @@ gimp_color_dialog_set_color (GimpColorDialog *dialog,
                                   color);
   gimp_color_selection_set_old_color (GIMP_COLOR_SELECTION (dialog->selection),
                                       color);
+  if (! dialog->user_context_aware)
+    gimp_color_selection_set_format (GIMP_COLOR_SELECTION (dialog->selection),
+                                     gegl_color_get_format (color));
 
   g_signal_handlers_unblock_by_func (dialog->selection,
                                      gimp_color_dialog_color_changed,
                                      dialog);
 }
 
-void
-gimp_color_dialog_get_color (GimpColorDialog *dialog,
-                             GimpRGB         *color)
+GeglColor *
+gimp_color_dialog_get_color (GimpColorDialog *dialog)
 {
-  g_return_if_fail (GIMP_IS_COLOR_DIALOG (dialog));
-  g_return_if_fail (color != NULL);
+  g_return_val_if_fail (GIMP_IS_COLOR_DIALOG (dialog), NULL);
 
-  gimp_color_selection_get_color (GIMP_COLOR_SELECTION (dialog->selection),
-                                  color);
+  return gimp_color_selection_get_color (GIMP_COLOR_SELECTION (dialog->selection));
 }
 
 
@@ -573,28 +595,26 @@ gimp_color_dialog_colormap_clicked (GimpColorDialog  *dialog,
                                     GimpPaletteEntry *entry,
                                     GdkModifierType   state)
 {
-  gimp_color_dialog_set_color (dialog, &entry->color);
+  gimp_color_dialog_set_color (dialog, entry->color);
 
   if (dialog->wants_updates)
-    {
-      g_signal_emit (dialog, color_dialog_signals[UPDATE], 0,
-                     &entry->color, GIMP_COLOR_DIALOG_UPDATE);
-    }
+    g_signal_emit (dialog, color_dialog_signals[UPDATE], 0,
+                   entry->color, GIMP_COLOR_DIALOG_UPDATE);
 }
 
 static void
 gimp_color_dialog_colormap_edit_activate (GimpColorDialog *dialog)
 {
   GimpColormapSelection *colormap_selection;
-  GimpRGB                color;
+  GeglColor             *color;
   gint                   col_index;
 
   dialog->colormap_editing = TRUE;
 
   colormap_selection = GIMP_COLORMAP_SELECTION (dialog->colormap_selection);
   col_index = gimp_colormap_selection_get_index (colormap_selection, NULL);
-  gimp_image_get_colormap_entry (dialog->active_image, col_index, &color);
-  gimp_color_dialog_set_color (dialog, &color);
+  color = gimp_image_get_colormap_entry (dialog->active_image, col_index);
+  gimp_color_dialog_set_color (dialog, color);
 
   gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "color");
 }
@@ -605,15 +625,13 @@ gimp_color_dialog_colormap_add_activate (GimpColorDialog *dialog)
   GimpViewableDialog *viewable_dialog = GIMP_VIEWABLE_DIALOG (dialog);
 
   if (dialog->active_image &&
-      gimp_image_get_colormap_size (dialog->active_image) < 256 &&
+      gimp_palette_get_n_colors (gimp_image_get_colormap_palette (dialog->active_image)) < 256 &&
       viewable_dialog->context)
     {
       GimpContext *user_context = viewable_dialog->context->gimp->user_context;
-      GimpRGB      color;
+      GeglColor   *color        = gimp_context_get_foreground (user_context);
 
-      gimp_context_get_foreground (user_context, &color);
-
-      gimp_image_add_colormap_entry (dialog->active_image, &color);
+      gimp_image_add_colormap_entry (dialog->active_image, color);
       gimp_image_flush (dialog->active_image);
     }
 }
@@ -623,9 +641,9 @@ gimp_color_dialog_color_changed (GimpColorSelection *selection,
                                  GimpColorDialog    *dialog)
 {
   GimpViewableDialog *viewable_dialog = GIMP_VIEWABLE_DIALOG (dialog);
-  GimpRGB             color;
+  GeglColor          *color;
 
-  gimp_color_selection_get_color (selection, &color);
+  color = gimp_color_selection_get_color (selection);
 
   if (dialog->colormap_editing && viewable_dialog->context)
     {
@@ -643,17 +661,17 @@ gimp_color_dialog_color_changed (GimpColorSelection *selection,
           col_index = gimp_colormap_selection_get_index (colormap_selection, NULL);
           if (push_undo)
             {
-              GimpRGB old_color;
+              GeglColor *old_color;
 
-              gimp_color_selection_get_old_color (GIMP_COLOR_SELECTION (dialog->selection), &old_color);
+              old_color = gimp_color_selection_get_old_color (GIMP_COLOR_SELECTION (dialog->selection));
 
               /* Restore old color for undo */
-              gimp_image_set_colormap_entry (image, col_index, &old_color,
-                                             FALSE);
+              gimp_image_set_colormap_entry (image, col_index, old_color, FALSE);
+
+              g_object_unref (old_color);
             }
 
-          gimp_image_set_colormap_entry (image, col_index, &color,
-                                         push_undo);
+          gimp_image_set_colormap_entry (image, col_index, color, push_undo);
 
           if (push_undo)
             gimp_image_flush (image);
@@ -663,10 +681,10 @@ gimp_color_dialog_color_changed (GimpColorSelection *selection,
     }
 
   if (dialog->wants_updates)
-    {
-      g_signal_emit (dialog, color_dialog_signals[UPDATE], 0,
-                     &color, GIMP_COLOR_DIALOG_UPDATE);
-    }
+    g_signal_emit (dialog, color_dialog_signals[UPDATE], 0,
+                   color, GIMP_COLOR_DIALOG_UPDATE);
+
+  g_object_unref (color);
 }
 
 
@@ -678,25 +696,11 @@ gimp_color_history_add_clicked (GtkWidget       *widget,
 {
   GimpViewableDialog *viewable_dialog = GIMP_VIEWABLE_DIALOG (dialog);
   GimpPalette        *history;
-  GimpRGB             color;
+  GeglColor          *color;
 
   history = gimp_palettes_get_color_history (viewable_dialog->context->gimp);
-
-  gimp_color_selection_get_color (GIMP_COLOR_SELECTION (dialog->selection),
-                                  &color);
-
-  gimp_palette_mru_add (GIMP_PALETTE_MRU (history), &color);
-}
-
-/* Color history callback  */
-
-static void
-gimp_color_dialog_history_selected (GimpColorHistory *history,
-                                    const GimpRGB    *rgb,
-                                    GimpColorDialog  *dialog)
-{
-  gimp_color_selection_set_color (GIMP_COLOR_SELECTION (dialog->selection),
-                                  rgb);
+  color   = gimp_color_selection_get_color (GIMP_COLOR_SELECTION (dialog->selection));
+  gimp_palette_mru_add (GIMP_PALETTE_MRU (history), color);
 }
 
 /* Context-related callbacks */
@@ -711,7 +715,7 @@ gimp_color_dialog_image_changed (GimpContext     *context,
       if (dialog->active_image)
         {
           g_signal_handlers_disconnect_by_func (dialog->active_image,
-                                                G_CALLBACK (gimp_color_dialog_update),
+                                                G_CALLBACK (gimp_color_dialog_update_base_type),
                                                 dialog);
           g_signal_handlers_disconnect_by_func (dialog->active_image,
                                                 gimp_color_dialog_update_simulation,
@@ -722,44 +726,67 @@ gimp_color_dialog_image_changed (GimpContext     *context,
 
       if (image)
         {
-          g_signal_connect_swapped (image, "notify::base-type",
-                                    G_CALLBACK (gimp_color_dialog_update),
-                                    dialog);
-          g_signal_connect (image, "simulation-profile-changed",
-                            G_CALLBACK (gimp_color_dialog_update_simulation),
-                            dialog);
-          g_signal_connect (image, "simulation-intent-changed",
-                            G_CALLBACK (gimp_color_dialog_update_simulation),
-                            dialog);
-          g_signal_connect (image, "simulation-bpc-changed",
-                            G_CALLBACK (gimp_color_dialog_update_simulation),
-                            dialog);
+          g_signal_connect_object (image, "notify::base-type",
+                                   G_CALLBACK (gimp_color_dialog_update_base_type),
+                                   dialog, G_CONNECT_SWAPPED);
+          g_signal_connect_object (image, "profile-changed",
+                                   G_CALLBACK (gimp_color_dialog_update_format),
+                                   dialog, 0);
+          g_signal_connect_object (image, "simulation-profile-changed",
+                                   G_CALLBACK (gimp_color_dialog_update_simulation),
+                                   dialog, 0);
+          g_signal_connect_object (image, "simulation-intent-changed",
+                                   G_CALLBACK (gimp_color_dialog_update_simulation),
+                                   dialog, 0);
+          g_signal_connect_object (image, "simulation-bpc-changed",
+                                   G_CALLBACK (gimp_color_dialog_update_simulation),
+                                   dialog, 0);
 
           gimp_color_dialog_update_simulation (image, dialog);
         }
 
-      gimp_color_dialog_update (dialog);
+      gimp_color_dialog_update_base_type (dialog);
+      gimp_color_dialog_update_format (image, dialog);
     }
 }
 
+static void
+gimp_color_dialog_update_format (GimpImage       *image,
+                                 GimpColorDialog *dialog)
+{
+  const Babl *format = NULL;
+
+  g_return_if_fail (GIMP_IS_COLOR_DIALOG (dialog));
+
+  if (image)
+    format = gimp_image_get_layer_format (image, FALSE);
+
+  gimp_color_selection_set_format (GIMP_COLOR_SELECTION (dialog->selection), format);
+}
+
 static void
 gimp_color_dialog_update_simulation (GimpImage       *image,
                                      GimpColorDialog *dialog)
 {
+  GimpColorProfile         *profile = NULL;
+  GimpColorRenderingIntent  intent  = GIMP_COLOR_RENDERING_INTENT_RELATIVE_COLORIMETRIC;
+  gboolean                  bpc     = FALSE;
+
   g_return_if_fail (GIMP_IS_COLOR_DIALOG (dialog));
 
-  if (image && GIMP_IS_COLOR_DIALOG (dialog))
+  if (image)
     {
-      gimp_color_selection_set_simulation (GIMP_COLOR_SELECTION (dialog->selection),
-                                           gimp_image_get_simulation_profile (image),
-                                           gimp_image_get_simulation_intent (image),
-                                           gimp_image_get_simulation_bpc (image));
+      profile = gimp_image_get_simulation_profile (image);
+      intent  = gimp_image_get_simulation_intent (image);
+      bpc     = gimp_image_get_simulation_bpc (image);
     }
+
+  gimp_color_selection_set_simulation (GIMP_COLOR_SELECTION (dialog->selection),
+                                       profile, intent, bpc);
 }
 
-
 static void
-gimp_color_dialog_update (GimpColorDialog *dialog)
+gimp_color_dialog_update_base_type (GimpColorDialog *dialog)
 {
   if (dialog->active_image &&
       gimp_image_get_base_type (dialog->active_image) == GIMP_INDEXED)
@@ -780,13 +807,26 @@ gimp_color_dialog_show (GimpColorDialog *dialog)
       GimpContext *user_context = viewable_dialog->context->gimp->user_context;
       GimpImage   *image        = gimp_context_get_image (user_context);
 
-      g_signal_connect (user_context, "image-changed",
-                        G_CALLBACK (gimp_color_dialog_image_changed),
-                        dialog);
+      g_signal_connect_object (user_context, "image-changed",
+                               G_CALLBACK (gimp_color_dialog_image_changed),
+                               dialog, 0);
+      g_signal_connect_object (viewable_dialog->context->gimp->config,
+                               "notify::theme",
+                               G_CALLBACK (gimp_color_dialog_style_updated),
+                               dialog, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+      g_signal_connect_object (viewable_dialog->context->gimp->config,
+                               "notify::override-theme-icon-size",
+                               G_CALLBACK (gimp_color_dialog_style_updated),
+                               dialog, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+      g_signal_connect_object (viewable_dialog->context->gimp->config,
+                               "notify::custom-icon-size",
+                               G_CALLBACK (gimp_color_dialog_style_updated),
+                               dialog, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 
       gimp_color_dialog_image_changed (viewable_dialog->context,
                                        image, dialog);
-      gimp_color_dialog_update (dialog);
+      gimp_color_dialog_update_base_type (dialog);
+      gimp_color_dialog_style_updated (GTK_WIDGET (dialog));
     }
   else
     {
@@ -805,5 +845,8 @@ gimp_color_dialog_hide (GimpColorDialog *dialog)
       g_signal_handlers_disconnect_by_func (user_context,
                                             G_CALLBACK (gimp_color_dialog_image_changed),
                                             dialog);
+      g_signal_handlers_disconnect_by_func (viewable_dialog->context->gimp->config,
+                                            G_CALLBACK (gimp_color_dialog_style_updated),
+                                            dialog);
     }
 }
