@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -29,6 +29,12 @@
 
 #include "widgets-types.h"
 
+#include "propgui/propgui-types.h"
+
+#include "core/gimp.h"
+
+#include "propgui/gimppropgui.h"
+
 #include "gimpcolordisplayeditor.h"
 #include "gimpeditor.h"
 
@@ -127,6 +133,7 @@ gimp_color_display_editor_init (GimpColorDisplayEditor *editor)
                                   GTK_ORIENTATION_VERTICAL);
 
   paned = gtk_paned_new (GTK_ORIENTATION_VERTICAL);
+  gtk_paned_set_wide_handle (GTK_PANED (paned), TRUE);
   gtk_box_pack_start (GTK_BOX (editor), paned, TRUE, TRUE, 0);
   gtk_widget_show (paned);
 
@@ -147,6 +154,8 @@ gimp_color_display_editor_init (GimpColorDisplayEditor *editor)
                                     G_TYPE_STRING,
                                     G_TYPE_STRING,
                                     G_TYPE_GTYPE);
+  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (editor->src),
+                                        SRC_COLUMN_NAME, GTK_SORT_ASCENDING);
   tv = gtk_tree_view_new_with_model (GTK_TREE_MODEL (editor->src));
   g_object_unref (editor->src);
 
@@ -222,7 +231,7 @@ gimp_color_display_editor_init (GimpColorDisplayEditor *editor)
                             NULL,
                             G_CALLBACK (gimp_color_display_editor_up_clicked),
                             NULL,
-                            editor);
+                            G_OBJECT (editor));
 
   editor->down_button =
     gimp_editor_add_button (GIMP_EDITOR (ed),
@@ -231,7 +240,7 @@ gimp_color_display_editor_init (GimpColorDisplayEditor *editor)
                             NULL,
                             G_CALLBACK (gimp_color_display_editor_down_clicked),
                             NULL,
-                            editor);
+                            G_OBJECT (editor));
 
   gtk_widget_set_sensitive (editor->up_button,   FALSE);
   gtk_widget_set_sensitive (editor->down_button, FALSE);
@@ -320,7 +329,7 @@ gimp_color_display_editor_init (GimpColorDisplayEditor *editor)
   gtk_box_pack_end (GTK_BOX (editor->config_box), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
-  editor->reset_button = gtk_button_new_from_stock (GIMP_ICON_RESET);
+  editor->reset_button = gtk_button_new_with_mnemonic (_("_Reset"));
   gtk_box_pack_end (GTK_BOX (hbox), editor->reset_button, FALSE, FALSE, 0);
   gtk_widget_show (editor->reset_button);
 
@@ -340,36 +349,18 @@ gimp_color_display_editor_dispose (GObject *object)
 {
   GimpColorDisplayEditor *editor = GIMP_COLOR_DISPLAY_EDITOR (object);
 
-  if (editor->selected)
-    {
-      g_object_remove_weak_pointer (G_OBJECT (editor->selected),
-                                    (gpointer) &editor->selected);
-      editor->selected = NULL;
-    }
+  g_clear_weak_pointer (&editor->selected);
 
-  if (editor->stack)
-    {
-      g_object_unref (editor->stack);
-      editor->stack = NULL;
-    }
-
-  if (editor->config)
-    {
-      g_object_unref (editor->config);
-      editor->config = NULL;
-    }
-
-  if (editor->managed)
-    {
-      g_object_unref (editor->managed);
-      editor->managed = NULL;
-    }
+  g_clear_object (&editor->stack);
+  g_clear_object (&editor->config);
+  g_clear_object (&editor->managed);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 GtkWidget *
-gimp_color_display_editor_new (GimpColorDisplayStack *stack,
+gimp_color_display_editor_new (Gimp                  *gimp,
+                               GimpColorDisplayStack *stack,
                                GimpColorConfig       *config,
                                GimpColorManaged      *managed)
 {
@@ -379,12 +370,14 @@ gimp_color_display_editor_new (GimpColorDisplayStack *stack,
   gint                    i;
   GList                  *list;
 
+  g_return_val_if_fail (GIMP_IS_GIMP (gimp), NULL);
   g_return_val_if_fail (GIMP_IS_COLOR_DISPLAY_STACK (stack), NULL);
   g_return_val_if_fail (GIMP_IS_COLOR_CONFIG (config), NULL);
   g_return_val_if_fail (GIMP_IS_COLOR_MANAGED (managed), NULL);
 
   editor = g_object_new (GIMP_TYPE_COLOR_DISPLAY_EDITOR, NULL);
 
+  editor->gimp    = gimp;
   editor->stack   = g_object_ref (stack);
   editor->config  = g_object_ref (config);
   editor->managed = g_object_ref (managed);
@@ -411,7 +404,9 @@ gimp_color_display_editor_new (GimpColorDisplayStack *stack,
 
   g_free (display_types);
 
-  for (list = stack->filters; list; list = g_list_next (list))
+  for (list = gimp_color_display_stack_get_filters (stack);
+       list;
+       list = g_list_next (list))
     {
       GimpColorDisplay *display = list->data;
       GtkTreeIter       iter;
@@ -547,12 +542,7 @@ gimp_color_display_editor_dest_changed (GtkTreeSelection       *sel,
   GimpColorDisplay *display = NULL;
   gchar            *tip     = NULL;
 
-  if (editor->selected)
-    {
-      g_object_remove_weak_pointer (G_OBJECT (editor->selected),
-                                    (gpointer) &editor->selected);
-      editor->selected = NULL;
-    }
+  g_clear_weak_pointer (&editor->selected);
 
   if (gtk_tree_selection_get_selected (sel, &model, &iter))
     {
@@ -580,13 +570,20 @@ gimp_color_display_editor_dest_changed (GtkTreeSelection       *sel,
 
   if (display)
     {
-      editor->selected = display;
-
-      g_object_add_weak_pointer (G_OBJECT (display),
-                                 (gpointer) &editor->selected);
+      g_set_weak_pointer (&editor->selected, display);
 
       editor->config_widget = gimp_color_display_configure (display);
 
+      if (! editor->config_widget)
+        {
+          editor->config_widget =
+            gimp_prop_gui_new (G_OBJECT (display),
+                               G_TYPE_FROM_INSTANCE (display), 0,
+                               NULL,
+                               gimp_get_user_context (editor->gimp),
+                               NULL, NULL, NULL);
+        }
+
       gtk_frame_set_label (GTK_FRAME (editor->config_frame),
                            GIMP_COLOR_DISPLAY_GET_CLASS (display)->name);
     }
@@ -704,6 +701,7 @@ gimp_color_display_editor_reordered (GimpColorDisplayStack  *stack,
 
       if (display == display2)
         {
+          GList       *filters = gimp_color_display_stack_get_filters (stack);
           GtkTreePath *path;
           gint         old_position;
 
@@ -714,7 +712,7 @@ gimp_color_display_editor_reordered (GimpColorDisplayStack  *stack,
           if (position == old_position)
             return;
 
-          if (position == -1 || position == g_list_length (stack->filters) - 1)
+          if (position == -1 || position == g_list_length (filters) - 1)
             {
               gtk_list_store_move_before (editor->dest, &iter, NULL);
             }
@@ -815,11 +813,12 @@ gimp_color_display_editor_update_buttons (GimpColorDisplayEditor *editor)
 
   if (gtk_tree_selection_get_selected (editor->dest_sel, &model, &iter))
     {
+      GList       *filters = gimp_color_display_stack_get_filters (editor->stack);
       GtkTreePath *path    = gtk_tree_model_get_path (model, &iter);
       gint        *indices = gtk_tree_path_get_indices (path);
 
       up_sensitive   = indices[0] > 0;
-      down_sensitive = indices[0] < (g_list_length (editor->stack->filters) - 1);
+      down_sensitive = indices[0] < (g_list_length (filters) - 1);
 
       gtk_tree_path_free (path);
     }
