@@ -40,8 +40,7 @@
 
 
 static void   gimp_color_selector_palette_set_color  (GimpColorSelector *selector,
-                                                      const GimpRGB     *rgb,
-                                                      const GimpHSV     *hsv);
+                                                      GeglColor         *color);
 static void   gimp_color_selector_palette_set_config (GimpColorSelector *selector,
                                                       GimpColorConfig   *config);
 
@@ -74,8 +73,7 @@ gimp_color_selector_palette_init (GimpColorSelectorPalette *select)
 
 static void
 gimp_color_selector_palette_set_color (GimpColorSelector *selector,
-                                       const GimpRGB     *rgb,
-                                       const GimpHSV     *hsv)
+                                       GeglColor         *color)
 {
   GimpColorSelectorPalette *select = GIMP_COLOR_SELECTOR_PALETTE (selector);
 
@@ -87,7 +85,7 @@ gimp_color_selector_palette_set_color (GimpColorSelector *selector,
         {
           GimpPaletteEntry *entry;
 
-          entry = gimp_palette_find_entry (palette, rgb,
+          entry = gimp_palette_find_entry (palette, color,
                                            GIMP_PALETTE_VIEW (select->view)->selected);
 
           if (entry)
@@ -111,10 +109,7 @@ gimp_color_selector_palette_entry_clicked (GimpPaletteView   *view,
                                            GdkModifierType    state,
                                            GimpColorSelector *selector)
 {
-  selector->rgb = entry->color;
-  gimp_rgb_to_hsv (&selector->rgb, &selector->hsv);
-
-  gimp_color_selector_emit_color_changed (selector);
+  gimp_color_selector_set_color (selector, entry->color);
 }
 
 static void
