@@ -34,9 +34,12 @@
 #include "core/gimpcurve.h"
 #include "core/gimpcurve-map.h"
 #include "core/gimpdatafactory.h"
+#include "core/gimppadactions.h"
 #include "core/gimpparamspecs.h"
 #include "core/gimptoolinfo.h"
 
+#include "widgets/gimpaction.h"
+
 #include "gimpdeviceinfo.h"
 
 #include "gimp-intl.h"
@@ -83,7 +86,8 @@ enum
   PROP_TOOL_HARDWARE_ID,
   PROP_AXES,
   PROP_KEYS,
-  PROP_PRESSURE_CURVE
+  PROP_PRESSURE_CURVE,
+  PROP_PAD_ACTIONS
 };
 
 struct _GimpDeviceInfoPrivate
@@ -101,6 +105,8 @@ struct _GimpDeviceInfoPrivate
   gint            n_keys;
   GimpDeviceKey  *keys;
 
+  GimpPadActions *pad_actions;
+
   /*  curves  */
 
   GimpCurve      *pressure_curve;
@@ -249,6 +255,13 @@ gimp_device_info_class_init (GimpDeviceInfoClass *klass)
                            NULL,
                            GIMP_TYPE_CURVE,
                            GIMP_CONFIG_PARAM_AGGREGATE);
+
+  GIMP_CONFIG_PROP_OBJECT (object_class, PROP_PAD_ACTIONS,
+                           "pad-actions",
+                           _("Pad actions"),
+                           NULL,
+                           GIMP_TYPE_PAD_ACTIONS,
+                           GIMP_CONFIG_PARAM_AGGREGATE);
 }
 
 static void
@@ -260,6 +273,7 @@ gimp_device_info_init (GimpDeviceInfo *info)
   info->priv->mode           = GDK_MODE_DISABLED;
   info->priv->pressure_curve = GIMP_CURVE (gimp_curve_new ("pressure curve"));
   info->priv->axes_names     = NULL;
+  info->priv->pad_actions    = gimp_pad_actions_new ();
 
   g_signal_connect (info, "notify::name",
                     G_CALLBACK (gimp_device_info_guess_icon),
@@ -289,6 +303,7 @@ gimp_device_info_finalize (GObject *object)
   g_strfreev (info->priv->axes_names);
 
   g_clear_object (&info->priv->pressure_curve);
+  g_clear_object (&info->priv->pad_actions);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -408,6 +423,11 @@ gimp_device_info_set_property (GObject      *object,
       dest_curve = info->priv->pressure_curve;
       break;
 
+    case PROP_PAD_ACTIONS:
+      g_clear_object (&info->priv->pad_actions);
+      info->priv->pad_actions = g_value_get_object (value);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -542,6 +562,10 @@ gimp_device_info_get_property (GObject    *object,
       g_value_set_object (value, info->priv->pressure_curve);
       break;
 
+    case PROP_PAD_ACTIONS:
+      g_value_set_object (value, info->priv->pad_actions);
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -739,6 +763,48 @@ gimp_device_info_updated (GimpDeviceInfo *info)
   g_object_thaw_notify (G_OBJECT (info));
 }
 
+static void
+gimp_device_info_pad_action_map_foreach (GimpPadActions    *pad_actions,
+                                         GimpPadActionType  action_type,
+                                         guint              number,
+                                         guint              mode,
+                                         const gchar       *action_name,
+                                         gpointer           data)
+{
+  GtkPadController *controller = data;
+  GimpDeviceInfo   *info;
+  Gimp             *gimp;
+  GimpAction       *action;
+  gchar            *label;
+  gchar            *accel_pos;
+
+  info = g_object_get_data (G_OBJECT (controller), "device-info");
+  if (!info)
+    return;
+
+  gimp = GIMP_TOOL_PRESET (info)->gimp;
+
+  action = GIMP_ACTION (g_action_map_lookup_action (G_ACTION_MAP (gimp->app),
+                                                    action_name));
+  if (!action)
+    return;
+
+  /* Trim the accelerator from the feedback string */
+  label = g_strdup (gimp_action_get_label (action));
+  accel_pos = g_utf8_strchr (label, -1, '_');
+  if (accel_pos)
+    strcpy (accel_pos, accel_pos + 1);
+
+  gtk_pad_controller_set_action (controller,
+                                 /* Action type enums are binary compatible */
+                                 (GtkPadActionType) action_type,
+                                 number, mode,
+                                 label,
+                                 gimp_action_get_name (action));
+  g_free (label);
+}
+
+
 /*  public functions  */
 
 GimpDeviceInfo *
@@ -1054,6 +1120,10 @@ gimp_device_info_get_source (GimpDeviceInfo *info)
   if (info->priv->device)
     return gdk_device_get_source (info->priv->device);
 
+  if (info->priv->pad_actions &&
+      gimp_pad_actions_get_n_actions (info->priv->pad_actions) > 0)
+    return GDK_SOURCE_TABLET_PAD;
+
   return GDK_SOURCE_MOUSE;
 }
 
@@ -1396,3 +1466,35 @@ gimp_device_info_compare (GimpDeviceInfo *a,
                                        (GimpObject *) b);
     }
 }
+
+GimpPadActions *
+gimp_device_info_get_pad_actions (GimpDeviceInfo *info)
+{
+  return info->priv->pad_actions;
+}
+
+GtkPadController *
+gimp_device_info_create_pad_controller (GimpDeviceInfo *info,
+                                        GimpWindow     *window)
+{
+  Gimp             *gimp;
+  GimpPadActions   *pad_actions;
+  GtkPadController *controller;
+
+  gimp = GIMP_TOOL_PRESET (info)->gimp;
+
+  pad_actions = gimp_device_info_get_pad_actions (info);
+
+  if (gimp_pad_actions_get_n_actions (pad_actions) == 0)
+    return NULL;
+
+  controller = gtk_pad_controller_new (GTK_WINDOW (window),
+                                       G_ACTION_GROUP (gimp->app),
+                                       info->priv->device);
+  g_object_set_data (G_OBJECT (controller), "device-info", info);
+  gimp_pad_actions_foreach (pad_actions,
+                            gimp_device_info_pad_action_map_foreach,
+                            controller);
+
+  return controller;
+}
