@@ -16,22 +16,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <sys/types.h>
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #include <gegl.h>
 #include <gtk/gtk.h>
-#include <glib/gstdio.h>
 
 #include "libgimpbase/gimpbase.h"
+#include "libgimpcolor/gimpcolor.h"
 #include "libgimpwidgets/gimpwidgets.h"
 
 #include "widgets-types.h"
@@ -41,13 +35,11 @@
 #include "core/gimpimage.h"
 #include "core/gimpimage-colormap.h"
 #include "core/gimpimage-undo.h"
+#include "core/gimppalette.h"
 #include "core/gimpundostack.h"
 #include "core/gimp-utils.h"
 
-#include "file/file-procedure.h"
-#include "file/file-utils.h"
-
-#include "plug-in/gimppluginmanager.h"
+#include "plug-in/gimppluginmanager-file.h"
 #include "plug-in/gimppluginprocedure.h"
 
 #include "gimpimagepropview.h"
@@ -73,7 +65,7 @@ static void        gimp_image_prop_view_get_property (GObject           *object,
                                                       GValue            *value,
                                                       GParamSpec        *pspec);
 
-static GtkWidget * gimp_image_prop_view_add_label    (GtkTable          *table,
+static GtkWidget * gimp_image_prop_view_add_label    (GtkGrid           *grid,
                                                       gint               row,
                                                       const gchar       *text);
 static void        gimp_image_prop_view_undo_event   (GimpImage         *image,
@@ -82,9 +74,11 @@ static void        gimp_image_prop_view_undo_event   (GimpImage         *image,
                                                       GimpImagePropView *view);
 static void        gimp_image_prop_view_update       (GimpImagePropView *view);
 static void        gimp_image_prop_view_file_update  (GimpImagePropView *view);
+static void        gimp_image_prop_view_realize      (GimpImagePropView *view,
+                                                      gpointer           user_data);
 
 
-G_DEFINE_TYPE (GimpImagePropView, gimp_image_prop_view, GTK_TYPE_TABLE)
+G_DEFINE_TYPE (GimpImagePropView, gimp_image_prop_view, GTK_TYPE_GRID)
 
 #define parent_class gimp_image_prop_view_parent_class
 
@@ -108,65 +102,71 @@ gimp_image_prop_view_class_init (GimpImagePropViewClass *klass)
 static void
 gimp_image_prop_view_init (GimpImagePropView *view)
 {
-  GtkTable *table = GTK_TABLE (view);
-  gint      row = 0;
+  GtkGrid *grid = GTK_GRID (view);
+  gint     row = 0;
 
-  gtk_table_resize (table, 14, 2);
-
-  gtk_table_set_col_spacings (table, 6);
-  gtk_table_set_row_spacings (table, 3);
+  gtk_grid_set_column_spacing (grid, 6);
+  gtk_grid_set_row_spacing (grid, 3);
 
   view->pixel_size_label =
-    gimp_image_prop_view_add_label (table, row++, _("Size in pixels:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Size in pixels:"));
 
   view->print_size_label =
-    gimp_image_prop_view_add_label (table, row++, _("Print size:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Print size:"));
 
   view->resolution_label =
-    gimp_image_prop_view_add_label (table, row++, _("Resolution:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Resolution:"));
 
   view->colorspace_label =
-    gimp_image_prop_view_add_label (table, row, _("Color space:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Color space:"));
 
-  gtk_table_set_row_spacing (GTK_TABLE (view), row++, 12);
+  view->precision_label =
+    gimp_image_prop_view_add_label (grid, row++, _("Precision:"));
+
+  gtk_widget_set_margin_bottom (view->precision_label, 12);
 
   view->filename_label =
-    gimp_image_prop_view_add_label (table, row++, _("File Name:"));
+    gimp_image_prop_view_add_label (grid, row++, _("File Name:"));
 
   gtk_label_set_ellipsize (GTK_LABEL (view->filename_label),
                            PANGO_ELLIPSIZE_MIDDLE);
+  /* See gimp_image_prop_view_realize(). */
+  gtk_label_set_max_width_chars (GTK_LABEL (view->filename_label), 25);
 
   view->filesize_label =
-    gimp_image_prop_view_add_label (table, row++, _("File Size:"));
+    gimp_image_prop_view_add_label (grid, row++, _("File Size:"));
 
   view->filetype_label =
-    gimp_image_prop_view_add_label (table, row, _("File Type:"));
+    gimp_image_prop_view_add_label (grid, row++, _("File Type:"));
 
-  gtk_table_set_row_spacing (GTK_TABLE (view), row++, 12);
+  gtk_widget_set_margin_bottom (view->filetype_label, 12);
 
   view->memsize_label =
-    gimp_image_prop_view_add_label (table, row++, _("Size in memory:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Size in memory:"));
 
   view->undo_label =
-    gimp_image_prop_view_add_label (table, row++, _("Undo steps:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Undo steps:"));
 
   view->redo_label =
-    gimp_image_prop_view_add_label (table, row, _("Redo steps:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Redo steps:"));
 
-  gtk_table_set_row_spacing (GTK_TABLE (view), row++, 12);
+  gtk_widget_set_margin_bottom (view->redo_label, 12);
 
   view->pixels_label =
-    gimp_image_prop_view_add_label (table, row++, _("Number of pixels:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Number of pixels:"));
 
   view->layers_label =
-    gimp_image_prop_view_add_label (table, row++, _("Number of layers:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Number of layers:"));
 
   view->channels_label =
-    gimp_image_prop_view_add_label (table, row++, _("Number of channels:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Number of channels:"));
 
   view->vectors_label =
-    gimp_image_prop_view_add_label (table, row++, _("Number of paths:"));
+    gimp_image_prop_view_add_label (grid, row++, _("Number of paths:"));
 
+  g_signal_connect (view, "realize",
+                    G_CALLBACK (gimp_image_prop_view_realize),
+                    NULL);
 }
 
 static void
@@ -174,10 +174,9 @@ gimp_image_prop_view_constructed (GObject *object)
 {
   GimpImagePropView *view = GIMP_IMAGE_PROP_VIEW (object);
 
-  if (G_OBJECT_CLASS (parent_class)->constructed)
-    G_OBJECT_CLASS (parent_class)->constructed (object);
+  G_OBJECT_CLASS (parent_class)->constructed (object);
 
-  g_assert (view->image != NULL);
+  gimp_assert (view->image != NULL);
 
   g_signal_connect_object (view->image, "name-changed",
                            G_CALLBACK (gimp_image_prop_view_file_update),
@@ -264,7 +263,7 @@ gimp_image_prop_view_new (GimpImage *image)
 /*  private functions  */
 
 static GtkWidget *
-gimp_image_prop_view_add_label (GtkTable    *table,
+gimp_image_prop_view_add_label (GtkGrid     *grid,
                                 gint         row,
                                 const gchar *text)
 {
@@ -274,13 +273,12 @@ gimp_image_prop_view_add_label (GtkTable    *table,
   desc = g_object_new (GTK_TYPE_LABEL,
                        "label",  text,
                        "xalign", 1.0,
-                       "yalign", 0.5,
+                       "yalign", 0.0,
                        NULL);
   gimp_label_set_attributes (GTK_LABEL (desc),
                              PANGO_ATTR_WEIGHT, PANGO_WEIGHT_BOLD,
                              -1);
-  gtk_table_attach (table, desc,
-                    0, 1, row, row + 1, GTK_FILL, GTK_FILL, 0, 0);
+  gtk_grid_attach (grid, desc, 0, row, 1, 1);
   gtk_widget_show (desc);
 
   label = g_object_new (GTK_TYPE_LABEL,
@@ -289,8 +287,7 @@ gimp_image_prop_view_add_label (GtkTable    *table,
                         "selectable", TRUE,
                         NULL);
 
-  gtk_table_attach (table, label,
-                    1, 2, row, row + 1, GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
+  gtk_grid_attach (grid, label, 1, row, 1, 1);
 
   gtk_widget_show (label);
 
@@ -310,14 +307,15 @@ static void
 gimp_image_prop_view_label_set_filename (GtkWidget *label,
                                          GimpImage *image)
 {
-  const gchar *uri = gimp_image_get_any_uri (image);
+  GFile *file = gimp_image_get_any_file (image);
 
-  if (uri)
+  if (file)
     {
-      gchar *name = file_utils_uri_display_name (uri);
-
-      gtk_label_set_text (GTK_LABEL (label), name);
-      g_free (name);
+      gtk_label_set_text (GTK_LABEL (label),
+                          gimp_file_get_utf8_name (file));
+      /* In case the label is ellipsized. */
+      gtk_widget_set_tooltip_text (GTK_WIDGET (label),
+                                   gimp_file_get_utf8_name (file));
     }
   else
     {
@@ -330,29 +328,29 @@ static void
 gimp_image_prop_view_label_set_filesize (GtkWidget *label,
                                          GimpImage *image)
 {
-  const gchar *uri      = gimp_image_get_any_uri (image);
-  gchar       *filename = NULL;
+  GFile *file = gimp_image_get_any_file (image);
 
-  if (uri)
-    filename = g_filename_from_uri (uri, NULL, NULL);
-
-  if (filename)
+  if (file)
     {
-      struct stat  buf;
+      GFileInfo *info = g_file_query_info (file,
+                                           G_FILE_ATTRIBUTE_STANDARD_SIZE,
+                                           G_FILE_QUERY_INFO_NONE,
+                                           NULL, NULL);
 
-      if (g_stat (filename, &buf) == 0)
+      if (info)
         {
-          gchar *str = g_format_size (buf.st_size);
+          goffset  size = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_STANDARD_SIZE);
+          gchar   *str  = g_format_size (size);
 
           gtk_label_set_text (GTK_LABEL (label), str);
           g_free (str);
+
+          g_object_unref (info);
         }
       else
         {
           gtk_label_set_text (GTK_LABEL (label), NULL);
         }
-
-      g_free (filename);
     }
   else
     {
@@ -364,27 +362,25 @@ static void
 gimp_image_prop_view_label_set_filetype (GtkWidget *label,
                                          GimpImage *image)
 {
-  GimpPlugInManager   *manager = image->gimp->plug_in_manager;
-  GimpPlugInProcedure *proc;
-
-  proc = gimp_image_get_save_proc (image);
+  GimpPlugInProcedure *proc = gimp_image_get_save_proc (image);
 
   if (! proc)
     proc = gimp_image_get_load_proc (image);
 
   if (! proc)
     {
-      gchar *filename = gimp_image_get_filename (image);
+      GimpPlugInManager *manager = image->gimp->plug_in_manager;
+      GFile             *file    = gimp_image_get_file (image);
 
-      if (filename)
-        {
-          proc = file_procedure_find (manager->load_procs, filename, NULL);
-          g_free (filename);
-        }
+      if (file)
+        proc = gimp_plug_in_manager_file_procedure_find (manager,
+                                                         GIMP_FILE_PROCEDURE_GROUP_OPEN,
+                                                         file, NULL);
     }
 
   gtk_label_set_text (GTK_LABEL (label),
-                      proc ? gimp_plug_in_procedure_get_label (proc) : NULL);
+                      proc ?
+                      gimp_procedure_get_label (GIMP_PROCEDURE (proc)) : NULL);
 }
 
 static void
@@ -425,10 +421,11 @@ static void
 gimp_image_prop_view_update (GimpImagePropView *view)
 {
   GimpImage         *image = view->image;
+  GimpColorProfile  *profile;
   GimpImageBaseType  type;
-  GimpUnit           unit;
+  GimpPrecision      precision;
+  GimpUnit          *unit;
   gdouble            unit_factor;
-  gint               unit_digits;
   const gchar       *desc;
   gchar              format_buf[32];
   gchar              buf[256];
@@ -448,11 +445,10 @@ gimp_image_prop_view_update (GimpImagePropView *view)
   /*  print size  */
   unit = gimp_get_default_unit ();
 
-  unit_digits = gimp_unit_get_digits (unit);
-
   g_snprintf (format_buf, sizeof (format_buf), "%%.%df × %%.%df %s",
-              unit_digits + 1, unit_digits + 1,
-              gimp_unit_get_plural (unit));
+              gimp_unit_get_scaled_digits (unit, xres),
+              gimp_unit_get_scaled_digits (unit, yres),
+              gimp_unit_get_name (unit));
   g_snprintf (buf, sizeof (buf), format_buf,
               gimp_pixels_to_units (gimp_image_get_width  (image), unit, xres),
               gimp_pixels_to_units (gimp_image_get_height (image), unit, yres));
@@ -467,12 +463,11 @@ gimp_image_prop_view_update (GimpImagePropView *view)
   g_snprintf (buf, sizeof (buf), _("%g × %g %s"),
               xres / unit_factor,
               yres / unit_factor,
-              unit == GIMP_UNIT_INCH ? _("ppi") : format_buf);
+              unit == gimp_unit_inch () ? _("ppi") : format_buf);
   gtk_label_set_text (GTK_LABEL (view->resolution_label), buf);
 
-  /*  color type  */
-  type = gimp_image_base_type (image);
-
+  /*  color space  */
+  type = gimp_image_get_base_type (image);
   gimp_enum_get_value (GIMP_TYPE_IMAGE_BASE_TYPE, type,
                        NULL, NULL, &desc, NULL);
 
@@ -480,16 +475,33 @@ gimp_image_prop_view_update (GimpImagePropView *view)
     {
     case GIMP_RGB:
     case GIMP_GRAY:
-      g_snprintf (buf, sizeof (buf), "%s", desc);
+      profile = gimp_color_managed_get_color_profile (GIMP_COLOR_MANAGED (image));
+      g_snprintf (buf, sizeof (buf), "%s: %s", desc,
+                  gimp_color_profile_get_label (profile));
       break;
     case GIMP_INDEXED:
-      g_snprintf (buf, sizeof (buf),
-                  "%s (%d %s)", desc, gimp_image_get_colormap_size (image),
-                  _("colors"));
+        {
+          gint n_colors;
+
+          n_colors = gimp_palette_get_n_colors (gimp_image_get_colormap_palette (image));
+          g_snprintf (buf, sizeof (buf),
+                      ngettext ("Indexed color (monochrome)",
+                                "Indexed color (%d colors)",
+                                n_colors), n_colors);
+        }
       break;
     }
 
   gtk_label_set_text (GTK_LABEL (view->colorspace_label), buf);
+  gtk_label_set_line_wrap (GTK_LABEL (view->colorspace_label), TRUE);
+
+  /*  precision  */
+  precision = gimp_image_get_precision (image);
+
+  gimp_enum_get_value (GIMP_TYPE_PRECISION, precision,
+                       NULL, NULL, &desc, NULL);
+
+  gtk_label_set_text (GTK_LABEL (view->precision_label), desc);
 
   /*  size in memory  */
   gimp_image_prop_view_label_set_memsize (view->memsize_label,
@@ -519,7 +531,7 @@ gimp_image_prop_view_update (GimpImagePropView *view)
 
   /*  number of vectors  */
   g_snprintf (buf, sizeof (buf), "%d",
-              gimp_image_get_n_vectors (image));
+              gimp_image_get_n_paths (image));
   gtk_label_set_text (GTK_LABEL (view->vectors_label), buf);
 }
 
@@ -537,3 +549,18 @@ gimp_image_prop_view_file_update (GimpImagePropView *view)
   /*  filetype  */
   gimp_image_prop_view_label_set_filetype (view->filetype_label, image);
 }
+
+static void
+gimp_image_prop_view_realize (GimpImagePropView *view,
+                              gpointer           user_data)
+{
+  /* Ugly trick to avoid extra-wide dialog at construction because of
+   * overlong file path. Basically I give a reasonable max size at
+   * construction (if the path is longer, it is just ellipsized per set
+   * rules), then once the widget is realized, I remove the max size,
+   * allowing the widget to grow wider if ever the dialog were
+   * manually resized (we don't want to keep the label short and
+   * ellipsized if the dialog is explicitly made to have enough place).
+   */
+  gtk_label_set_max_width_chars (GTK_LABEL (view->filename_label), -1);
+}
