@@ -77,9 +77,11 @@ struct _GimpMenuModelPrivate
    * will not be NULL.
    */
   GMenuItem     *submenu_item;
-  GimpRGB       *submenu_color;
+  GeglColor     *submenu_color;
 
   GList         *items;
+
+  GHashTable    *named_sections;
 };
 
 
@@ -114,12 +116,13 @@ static gboolean     gimp_menu_model_is_mutable               (GMenuModel
 
 static void         gimp_menu_model_initialize               (GimpMenuModel       *model,
                                                               GMenuModel          *gmodel);
-static gboolean     gimp_menu_model_handles_subpath          (GimpMenuModel       *model,
-                                                              const gchar         *path);
+static gchar      * gimp_menu_model_handles_subpath          (GimpMenuModel       *model,
+                                                              const gchar         *canonical_path,
+                                                              const gchar         *mnemonic_path);
 
 static GMenuItem  * gimp_menu_model_get_item                 (GimpMenuModel       *model,
                                                               gint                 idx);
-static GMenuItem *  gimp_menu_model_get_menu_item_rec        (GimpMenuModel       *model,
+static GMenuItem  * gimp_menu_model_get_menu_item_rec        (GimpMenuModel       *model,
                                                               const gchar         *path,
                                                               GimpMenuModel      **menu,
                                                               GMenuItem           *item);
@@ -132,12 +135,11 @@ static void         gimp_menu_model_action_notify_visible    (GimpAction
                                                               GimpMenuModel       *model);
 static void         gimp_menu_model_action_notify_label      (GimpAction          *action,
                                                               GParamSpec          *pspec,
-                                                              GMenuItem           *item);
+                                                              GimpMenuModel       *model);
 
 static gboolean     gimp_menu_model_ui_added                 (GimpUIManager       *manager,
                                                               const gchar         *path,
                                                               const gchar         *action_name,
-                                                              const gchar         *placeholder_key,
                                                               gboolean             top,
                                                               GimpMenuModel       *model);
 static gboolean     gimp_menu_model_ui_removed               (GimpUIManager       *manager,
@@ -199,11 +201,11 @@ gimp_menu_model_class_init (GimpMenuModelClass *klass)
                                                         GIMP_PARAM_READWRITE |
                                                         G_PARAM_EXPLICIT_NOTIFY));
   g_object_class_install_property (object_class, PROP_COLOR,
-                                   gimp_param_spec_rgb ("color",
-                                                        NULL, NULL,
-                                                        TRUE, &(GimpRGB) {},
-                                                        GIMP_PARAM_READWRITE |
-                                                        G_PARAM_EXPLICIT_NOTIFY));
+                                   gimp_param_spec_color ("color",
+                                                          NULL, NULL,
+                                                          TRUE, NULL,
+                                                          GIMP_PARAM_READWRITE |
+                                                          G_PARAM_EXPLICIT_NOTIFY));
 }
 
 static void
@@ -216,6 +218,9 @@ gimp_menu_model_init (GimpMenuModel *model)
   model->priv->is_section    = FALSE;
   model->priv->submenu_item  = NULL;
   model->priv->submenu_color = NULL;
+
+  model->priv->named_sections = g_hash_table_new_full (g_str_hash, g_str_equal,
+                                                       g_free, NULL);
 }
 
 static void
@@ -227,7 +232,8 @@ gimp_menu_model_finalize (GObject *object)
   g_clear_object (&model->priv->model);
   g_list_free_full (model->priv->items, g_object_unref);
   g_free (model->priv->path);
-  g_free (model->priv->submenu_color);
+  g_clear_object (&model->priv->submenu_color);
+  g_hash_table_destroy (model->priv->named_sections);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -258,7 +264,7 @@ gimp_menu_model_get_property (GObject    *object,
         }
       break;
     case PROP_COLOR:
-      g_value_set_boxed (value, model->priv->submenu_color);
+      g_value_set_object (value, model->priv->submenu_color);
       break;
 
     default:
@@ -296,7 +302,7 @@ gimp_menu_model_set_property (GObject      *object,
       break;
     case PROP_COLOR:
       gimp_menu_model_set_color (model, model->priv->path,
-                                 g_value_get_boxed (value));
+                                 g_value_get_object (value));
       break;
 
     default:
@@ -352,6 +358,14 @@ gimp_menu_model_get_item_attributes (GMenuModel  *model,
   value = g_menu_item_get_attribute_value (item, G_MENU_LINK_SECTION, NULL);
   if (value)
     g_hash_table_insert (*attributes, G_MENU_LINK_SECTION, value);
+
+  value = g_menu_item_get_attribute_value (item, G_MENU_ATTRIBUTE_TARGET, NULL);
+  if (value)
+    g_hash_table_insert (*attributes, G_MENU_ATTRIBUTE_TARGET, value);
+
+  value = g_menu_item_get_attribute_value (item, "hidden-when", NULL);
+  if (value)
+    g_hash_table_insert (*attributes, "hidden-when", value);
 }
 
 static GMenuModel*
@@ -538,14 +552,20 @@ gimp_menu_model_get_submodel (GimpMenuModel *model,
       n_items = g_menu_model_get_n_items (G_MENU_MODEL (submodel));
       for (i = 0; i < n_items; i++)
         {
-          GMenuModel *subsubmodel;
-          gchar      *label = NULL;
-          gchar      *canon_label = NULL;
+          GMenuModel  *subsubmodel;
+          gchar       *label       = NULL;
+          gchar       *canon_label = NULL;
+          const gchar *en_label;
 
           subsubmodel = g_menu_model_get_item_link (G_MENU_MODEL (submodel), i, G_MENU_LINK_SUBMENU);
           g_menu_model_get_item_attribute (G_MENU_MODEL (submodel), i, G_MENU_ATTRIBUTE_LABEL, "s", &label);
+
           if (label)
-            canon_label = gimp_utils_make_canonical_menu_label (label);
+            {
+              en_label = g_object_get_data (G_OBJECT (subsubmodel),
+                                            "gimp-menu-model-en-label");
+              canon_label = gimp_utils_make_canonical_menu_label (en_label);
+            }
 
           if (subsubmodel && g_strcmp0 (canon_label, submenu) == 0)
             {
@@ -594,7 +614,7 @@ gimp_menu_model_set_title (GimpMenuModel *model,
 void
 gimp_menu_model_set_color (GimpMenuModel *model,
                            const gchar   *path,
-                           const GimpRGB *color)
+                           GeglColor     *color)
 {
   GMenuItem     *item;
   GimpMenuModel *submenu = NULL;
@@ -603,13 +623,8 @@ gimp_menu_model_set_color (GimpMenuModel *model,
 
   if (item != NULL)
     {
-      if (color == NULL)
-        g_clear_pointer (&submenu->priv->submenu_color, g_free);
-      else if (submenu->priv->submenu_color == NULL)
-        submenu->priv->submenu_color = g_new (GimpRGB, 1);
-
-      if (color != NULL)
-        *submenu->priv->submenu_color = *color;
+      g_clear_object (&submenu->priv->submenu_color);
+      submenu->priv->submenu_color = color ? gegl_color_duplicate (color) : NULL;
 
       g_object_notify (G_OBJECT (submenu), "color");
     }
@@ -675,10 +690,18 @@ gimp_menu_model_initialize (GimpMenuModel *model,
       if (subsection != NULL)
         {
           GimpMenuModel *submodel;
+          gchar         *section_name = NULL;
 
           submodel = gimp_menu_model_new_section (model->priv->manager, subsection,
                                                   model->priv->path);
           item = g_menu_item_new_section (label, G_MENU_MODEL (submodel));
+
+          if (g_menu_model_get_item_attribute (G_MENU_MODEL (gmodel), i,
+                                               "section-name", "s", &section_name))
+            g_hash_table_insert (model->priv->named_sections,
+                                 (gpointer) section_name,
+                                 item);
+
           g_object_unref (submodel);
         }
       else if (submenu != NULL && label == NULL)
@@ -722,6 +745,7 @@ gimp_menu_model_initialize (GimpMenuModel *model,
           gchar         *canon_label;
           gchar         *path;
           const gchar   *en_label;
+          gchar         *en_label_copy;
 
           g_return_if_fail (label != NULL);
           en_label = g_object_get_data (G_OBJECT (submenu),
@@ -735,6 +759,10 @@ gimp_menu_model_initialize (GimpMenuModel *model,
 
           submodel = gimp_menu_model_new_submenu (model->priv->manager, submenu, path);
           item     = g_menu_item_new_submenu (label, G_MENU_MODEL (submodel));
+
+          en_label_copy = g_strdup (en_label);
+          g_object_set_data_full (G_OBJECT (submodel), "gimp-menu-model-en-label",
+                                  en_label_copy, (GDestroyNotify) g_free);
           submodel->priv->submenu_item = item;
 
           g_object_unref (submodel);
@@ -742,46 +770,60 @@ gimp_menu_model_initialize (GimpMenuModel *model,
         }
       else
         {
+          GimpAction *action;
+          gchar      *label_variant = NULL;
+
           item = g_menu_item_new_from_model (G_MENU_MODEL (gmodel), i);
 
-          if (action_name)
+          g_return_if_fail (action_name != NULL);
+
+          action = gimp_ui_manager_find_action (model->priv->manager, NULL, action_name);
+
+          if (model->priv->manager->store_action_paths)
+            /* Special-case the main menu manager when constructing it as
+             * this is the only one which should set the menu path.
+             */
+            gimp_action_set_menu_path (action, gimp_menu_model_get_path (model));
+
+          g_signal_connect_object (action,
+                                   "notify::visible",
+                                   G_CALLBACK (gimp_menu_model_action_notify_visible),
+                                   model, 0);
+
+          g_menu_item_get_attribute (item, "label-variant", "s",
+                                     &label_variant);
+          if (g_strcmp0 (label_variant, "long") == 0)
+            g_menu_item_set_label (item, gimp_action_get_label (action));
+          else
+            g_menu_item_set_label (item, gimp_action_get_short_label (action));
+
+          g_signal_connect_object (action,
+                                   "notify::short-label",
+                                   G_CALLBACK (gimp_menu_model_action_notify_label),
+                                   model, 0);
+          g_signal_connect_object (action,
+                                   "notify::label",
+                                   G_CALLBACK (gimp_menu_model_action_notify_label),
+                                   model, 0);
+
+          /* We want GimpRadioAction to be GTK_MENU_TRACKER_ITEM_ROLE_RADIO,
+           * in order to be displayed as radio menu items (as used to be
+           * GtkRadioAction) even in the gtk_application_set_menubar() code path.
+           *
+           * GTK do this by checking that the item has a "target" parameter,
+           * which we don't add in our .ui file. Instead let's do this
+           * programmatically, hence avoiding human errors.
+           * See: gtk/gtkmenutrackeritem.c
+           */
+          if (GIMP_IS_RADIO_ACTION (action))
             {
-              GimpAction *action;
-              gchar      *label_variant = NULL;
+              gint target;
 
-              action = gimp_ui_manager_find_action (model->priv->manager, NULL, action_name);
-
-              if (model->priv->manager->store_action_paths)
-                /* Special-case the main menu manager when constructing it as
-                 * this is the only one which should set the menu path.
-                 */
-                gimp_action_set_menu_path (action, gimp_menu_model_get_path (model));
-
-              g_signal_connect_object (action,
-                                       "notify::visible",
-                                       G_CALLBACK (gimp_menu_model_action_notify_visible),
-                                       model, 0);
-
-              g_menu_item_get_attribute (item, "label-variant", "s",
-                                         &label_variant);
-              if (g_strcmp0 (label_variant, "long") == 0)
-                g_menu_item_set_label (item, gimp_action_get_label (action));
-              else
-                g_menu_item_set_label (item, gimp_action_get_short_label (action));
-
-              g_signal_connect_object (action,
-                                       "notify::short-label",
-                                       G_CALLBACK (gimp_menu_model_action_notify_label),
-                                       item, 0);
-              g_signal_connect_object (action,
-                                       "notify::label",
-                                       G_CALLBACK (gimp_menu_model_action_notify_label),
-                                       item, 0);
-
-              g_free (label_variant);
+              g_object_get (action, "value", &target, NULL);
+              g_menu_item_set_attribute (item, G_MENU_ATTRIBUTE_TARGET, "i", target);
             }
 
-          /* else we instal a placeholder (no-action and always invisible) item. */
+          g_free (label_variant);
         }
 
       if (item)
@@ -807,22 +849,43 @@ gimp_menu_model_initialize (GimpMenuModel *model,
     }
 }
 
-static gboolean
+/*
+ * Returns the directory to create as a new submodel to @model, with
+ * @canonical_path being the fully canonical path name (all slashes unique,
+ * leading slash, no trailing slash, section name removed and no mnemonic), e.g.
+ * "/some/path/name" and @mnemonic_path is the fully canonical path name, except
+ * that it contains mnemonics, e.g. "/_some/p_ath/_name".
+ *
+ * The code relies on these canonicalized characteristics so you must make sure
+ * to feed properly formatted paths.
+ *
+ * The return value is the canonical path of the first submenu to create,
+ * including mnemonics, e.g. if @model was for path "/some", then this function
+ * returns "p_ath".
+ */
+static gchar *
 gimp_menu_model_handles_subpath (GimpMenuModel *model,
-                                 const gchar   *path)
+                                 const gchar   *canonical_path,
+                                 const gchar   *mnemonic_path)
 {
-  if (model->priv->path != NULL && ! g_str_has_prefix (path, model->priv->path))
-    return FALSE;
+  gchar *new_dir;
+  gchar *end_new_dir;
+  gint   n_slash = 0;
+
+  if (model->priv->path != NULL &&
+      (! g_str_has_prefix (canonical_path, model->priv->path) ||
+       canonical_path[strlen (model->priv->path)] != '/'))
+    {
+      return FALSE;
+    }
 
   for (GList *iter = model->priv->items; iter; iter = iter->next)
     {
-      GMenuModel  *submenu    = NULL;
-      GMenuModel  *subsection = NULL;
-      gchar       *label      = NULL;
-      GMenuItem   *item       = iter->data;
+      GMenuModel *submenu    = NULL;
+      GMenuModel *subsection = NULL;
+      GMenuItem  *item       = iter->data;
 
       subsection = g_menu_item_get_link (item, G_MENU_LINK_SECTION);
-      g_menu_item_get_attribute (item, G_MENU_ATTRIBUTE_LABEL, "s", &label);
 
       if (subsection != NULL)
         {
@@ -830,42 +893,61 @@ gimp_menu_model_handles_subpath (GimpMenuModel *model,
            * the whole menu. So we only handle the negative result (which means
            * we found a submenu model which will handle this path instead).
            */
-          if (! gimp_menu_model_handles_subpath (GIMP_MENU_MODEL (subsection), path))
+          new_dir = gimp_menu_model_handles_subpath (GIMP_MENU_MODEL (subsection),
+                                                     canonical_path, mnemonic_path);
+          if (new_dir == NULL)
             {
-              g_free (label);
               g_clear_object (&subsection);
-              return FALSE;
+              return NULL;
+            }
+          else
+            {
+              g_free (new_dir);
             }
         }
       else if ((submenu = g_menu_item_get_link (item, G_MENU_LINK_SUBMENU)) != NULL)
         {
           gchar *subpath;
 
-          g_return_val_if_fail (label != NULL, FALSE);
           subpath = g_strdup_printf ("%s/", GIMP_MENU_MODEL (submenu)->priv->path);
 
-          if (g_strcmp0 (path, GIMP_MENU_MODEL (submenu)->priv->path) == 0 ||
-              g_str_has_prefix (path, subpath))
+          if (g_strcmp0 (canonical_path, GIMP_MENU_MODEL (submenu)->priv->path) == 0 ||
+              g_str_has_prefix (canonical_path, subpath))
             {
               /* A submodel will handle the new path. */
               g_free (subpath);
-              g_free (label);
               g_clear_object (&subsection);
               g_clear_object (&submenu);
 
-              return FALSE;
+              return NULL;
             }
 
           g_free (subpath);
         }
 
-      g_free (label);
       g_clear_object (&subsection);
       g_clear_object (&submenu);
     }
 
-  /* This is a subpath with submenus to be created! */
-  return TRUE;
+  if (model->priv->path != NULL)
+    {
+      n_slash = 1;
+      new_dir = model->priv->path;
+      while ((new_dir = strstr (new_dir + 1, "/")) != NULL)
+        n_slash++;
+    }
+
+  new_dir = (gchar *) mnemonic_path + 1;
+  while (n_slash-- > 0)
+    new_dir = strstr (new_dir, "/") + 1;
+
+  end_new_dir = strstr (new_dir, "/");
+  if (end_new_dir)
+    new_dir = g_strndup (new_dir, end_new_dir - new_dir);
+  else
+    new_dir = g_strdup (new_dir);
+
+  return new_dir;
 }
 
 static GMenuItem *
@@ -888,9 +970,7 @@ gimp_menu_model_get_item (GimpMenuModel *model,
         {
           cur++;
         }
-      /* Count neither placeholders (items with no action_name), nor invisible
-       * actions.
-       */
+      /* Do not count invisible actions. */
       else if (g_menu_item_get_attribute (iter->data,
                                           G_MENU_ATTRIBUTE_ACTION,
                                           "&s", &action_name))
@@ -923,7 +1003,7 @@ gimp_menu_model_get_menu_item_rec (GimpMenuModel  *model,
   g_return_val_if_fail (item == model->priv->submenu_item, NULL);
   g_return_val_if_fail (menu != NULL && *menu == NULL, NULL);
 
-  if (gimp_utils_are_menu_path_identical (path, model->priv->path))
+  if (gimp_utils_are_menu_path_identical (path, model->priv->path, NULL, NULL, NULL))
     {
       *menu = model;
       return item;
@@ -998,20 +1078,32 @@ gimp_menu_model_action_notify_visible (GimpAction    *action,
 }
 
 static void
-gimp_menu_model_action_notify_label (GimpAction *action,
-                                     GParamSpec *pspec,
-                                     GMenuItem  *item)
+gimp_menu_model_action_notify_label (GimpAction    *action,
+                                     GParamSpec    *pspec,
+                                     GimpMenuModel *model)
 {
-  gchar *label_variant = NULL;
+  gchar     *label_variant = NULL;
+  GMenuItem *item;
+  gint       pos;
+  gboolean   visible;
 
   g_return_if_fail (GIMP_IS_ACTION (action));
-  g_return_if_fail (G_IS_MENU_ITEM (item));
+  g_return_if_fail (GIMP_IS_MENU_MODEL (model));
+
+  pos  = gimp_menu_model_get_position (model, gimp_action_get_name (action), &visible);
+  item = g_list_nth_data (model->priv->items, pos);
+
+  g_return_if_fail (item != NULL);
 
   g_menu_item_get_attribute (item, "label-variant", "s", &label_variant);
   if (g_strcmp0 (label_variant, "long") == 0)
     g_menu_item_set_label (item, gimp_action_get_label (action));
   else
     g_menu_item_set_label (item, gimp_action_get_short_label (action));
+
+  if (visible)
+    g_menu_model_items_changed (G_MENU_MODEL (model), pos, 1, 1);
+
   g_free (label_variant);
 }
 
@@ -1019,13 +1111,18 @@ static gboolean
 gimp_menu_model_ui_added (GimpUIManager *manager,
                           const gchar   *path,
                           const gchar   *action_name,
-                          const gchar   *placeholder_key,
                           gboolean       top,
                           GimpMenuModel *model)
 {
-  gboolean added = FALSE;
+  gchar         *canonical_path = NULL;
+  gchar         *mnemonic_path  = NULL;
+  gchar         *section_name   = NULL;
+  gboolean       added          = FALSE;
+  GimpMenuModel *mod_model      = g_object_ref (model);
+  gchar         *new_dir;
 
-  if (gimp_utils_are_menu_path_identical (path, model->priv->path))
+  if (gimp_utils_are_menu_path_identical (path, model->priv->path, &canonical_path,
+                                          &mnemonic_path, &section_name))
     {
       GApplication *app = model->priv->manager->gimp->app;
       GAction      *action;
@@ -1051,9 +1148,21 @@ gimp_menu_model_ui_added (GimpUIManager *manager,
 
       added = TRUE;
 
+      if (section_name != NULL)
+        {
+          GMenuItem *section_item;
+
+          section_item = g_hash_table_lookup (model->priv->named_sections, section_name);
+          if (section_item)
+            {
+              g_object_unref (mod_model);
+              mod_model = GIMP_MENU_MODEL (g_menu_item_get_link (section_item, G_MENU_LINK_SECTION));
+            }
+        }
+
       g_signal_handlers_disconnect_by_func (action,
                                             G_CALLBACK (gimp_menu_model_action_notify_visible),
-                                            model);
+                                            mod_model);
       detailed_action_name = g_strdup_printf ("%s.%s", action_prefix, g_action_get_name (action));
       item = g_menu_item_new (gimp_action_get_short_label (GIMP_ACTION (action)), detailed_action_name);
       /* TODO: add also G_MENU_ATTRIBUTE_ICON attribute? */
@@ -1062,107 +1171,44 @@ gimp_menu_model_ui_added (GimpUIManager *manager,
       if (model->priv->manager->store_action_paths)
         gimp_action_set_menu_path (GIMP_ACTION (action), gimp_menu_model_get_path (model));
 
-      if (placeholder_key)
+      if (top)
         {
-          GList      *placeholder = NULL;
-          GMenuModel *subsection  = NULL;
-
-          for (GList *iter = model->priv->items; iter; iter = iter->next)
-            {
-              const gchar *label;
-
-              subsection = g_menu_item_get_link (iter->data, G_MENU_LINK_SECTION);
-
-              if (subsection != NULL)
-                {
-                  if (gimp_menu_model_ui_added (manager, path, action_name,
-                                                placeholder_key, top,
-                                                GIMP_MENU_MODEL (subsection)))
-                    break;
-                  else
-                    g_clear_object (&subsection);
-                }
-              else if (g_menu_item_get_attribute (iter->data,
-                                                  G_MENU_ATTRIBUTE_LABEL,
-                                                  "&s", &label) &&
-                       g_strcmp0 (label, placeholder_key) == 0)
-                {
-                  placeholder = iter;
-                  break;
-                }
-            }
-
-          if (placeholder)
-            {
-              if (top)
-                model->priv->items = g_list_insert_before (model->priv->items,
-                                                           placeholder, item);
-              else
-                model->priv->items = g_list_insert_before (model->priv->items,
-                                                           placeholder->next, item);
-            }
-          else
-            {
-              added = FALSE;
-
-              if (subsection == NULL && ! model->priv->is_section)
-                g_warning ("%s: no placeholder item '%s'.", G_STRFUNC, placeholder_key);
-            }
-          /* else: added in a subsection. */
-
-          g_clear_object (&subsection);
-        }
-      else if (top)
-        {
-          model->priv->items = g_list_prepend (model->priv->items, item);
+          mod_model->priv->items = g_list_prepend (mod_model->priv->items, item);
         }
       else
         {
-          model->priv->items = g_list_append (model->priv->items, item);
+          mod_model->priv->items = g_list_append (mod_model->priv->items, item);
         }
 
       if (added)
         {
-          gint position = gimp_menu_model_get_position (model, action_name, NULL);
+          gint position = gimp_menu_model_get_position (mod_model, action_name, NULL);
 
           g_signal_connect_object (action,
                                    "notify::visible",
                                    G_CALLBACK (gimp_menu_model_action_notify_visible),
-                                   model, 0);
+                                   mod_model, 0);
           g_signal_connect_object (action,
                                    "notify::short-label",
                                    G_CALLBACK (gimp_menu_model_action_notify_label),
-                                   item, 0);
+                                   mod_model, 0);
           g_signal_connect_object (action,
                                    "notify::label",
                                    G_CALLBACK (gimp_menu_model_action_notify_label),
-                                   item, 0);
-          g_menu_model_items_changed (G_MENU_MODEL (model), position, 0, 1);
+                                   mod_model, 0);
+          g_menu_model_items_changed (G_MENU_MODEL (mod_model), position, 0, 1);
         }
       else
         {
           g_object_unref (item);
         }
     }
-  else if (gimp_menu_model_handles_subpath (model, path))
+  else if ((new_dir = gimp_menu_model_handles_subpath (model, canonical_path, mnemonic_path)))
     {
       GimpMenuModel *submodel;
       GMenuItem     *item;
       gchar         *canon_label;
       gchar         *submodel_path;
-      gchar         *new_dirs;
-      gchar         *new_dir;
-      gchar         *end_new_dir;
-
-      new_dirs = g_strdup (path + (model->priv->path ? strlen (model->priv->path) + 1 : 1));
-      new_dir  = new_dirs;
-
-      while (*new_dir == '/')
-        new_dir++;
-
-      end_new_dir = strstr (new_dir, "/");
-      if (end_new_dir)
-        *end_new_dir = '\0';
 
       canon_label   = gimp_utils_make_canonical_menu_label (new_dir);
       submodel_path = g_strdup_printf ("%s/%s",
@@ -1172,19 +1218,27 @@ gimp_menu_model_ui_added (GimpUIManager *manager,
       submodel = gimp_menu_model_new_submenu (model->priv->manager, NULL, submodel_path);
       item     = g_menu_item_new_submenu (new_dir, G_MENU_MODEL (submodel));
 
-      model->priv->items = g_list_append (model->priv->items, item);
+      if (model->priv->path == NULL)
+        model->priv->items = g_list_insert (model->priv->items, item,
+                                            g_list_length (model->priv->items) - 2);
+      else
+        model->priv->items = g_list_append (model->priv->items, item);
 
       g_free (canon_label);
       g_object_unref (submodel);
       g_free (submodel_path);
+      g_free (new_dir);
 
       g_menu_model_items_changed (G_MENU_MODEL (model),
                                   gimp_menu_model_get_position (model, NULL, NULL),
                                   1, 0);
-
-      g_free (new_dirs);
     }
 
+  g_clear_object (&mod_model);
+  g_free (canonical_path);
+  g_free (mnemonic_path);
+  g_free (section_name);
+
   return added;
 }
 
@@ -1194,9 +1248,12 @@ gimp_menu_model_ui_removed (GimpUIManager *manager,
                             const gchar   *action_name,
                             GimpMenuModel *model)
 {
-  gboolean removed = FALSE;
+  gchar    *section_name = NULL;
+  gboolean  removed      = FALSE;
 
-  if (g_strcmp0 (path, model->priv->path) == 0)
+  g_return_val_if_fail (GIMP_IS_MENU_MODEL (model), FALSE);
+
+  if (gimp_utils_are_menu_path_identical (path, model->priv->path, NULL, NULL, &section_name))
     {
       GApplication *app         = model->priv->manager->gimp->app;
       GMenuItem    *item        = NULL;
@@ -1210,7 +1267,7 @@ gimp_menu_model_ui_removed (GimpUIManager *manager,
 
       for (iter = model->priv->items; iter; iter = iter->next)
         {
-          const gchar *action;
+          const gchar *cur_action_name;
 
           subsection = g_menu_item_get_link (iter->data, G_MENU_LINK_SECTION);
 
@@ -1224,12 +1281,23 @@ gimp_menu_model_ui_removed (GimpUIManager *manager,
             }
           else if (g_menu_item_get_attribute (iter->data,
                                               G_MENU_ATTRIBUTE_ACTION,
-                                              "&s", &action) &&
-                   /* "action" attribute will start with "app." prefix. */
-                   g_strcmp0 (action + 4, action_name) == 0)
+                                              "&s", &cur_action_name))
             {
-              item = iter->data;
-              break;
+              gchar *dot = strstr (cur_action_name, ".");
+
+              g_return_val_if_fail (dot, FALSE);
+
+              /* "action" attribute will start with "app." prefix for main
+               * actions or with another prefix in some other cases (e.g.
+               * "tool-options.tool-options-restore-preset-000"). We need to
+               * clean this up, but for the time being, let's just look at
+               * what's after the dot.
+               */
+              if (g_strcmp0 (dot + 1, action_name) == 0)
+                {
+                  item = iter->data;
+                  break;
+                }
             }
         }
 
@@ -1246,13 +1314,14 @@ gimp_menu_model_ui_removed (GimpUIManager *manager,
                                                     model);
               g_signal_handlers_disconnect_by_func (action,
                                                     G_CALLBACK (gimp_menu_model_action_notify_label),
-                                                    item);
+                                                    model);
             }
           g_object_unref (item);
 
           model->priv->items = g_list_delete_link (model->priv->items, iter);
 
-          g_menu_model_items_changed (G_MENU_MODEL (model), position, 1, 0);
+          if (! action || gimp_action_is_visible (GIMP_ACTION (action)))
+            g_menu_model_items_changed (G_MENU_MODEL (model), position, 1, 0);
         }
       else
         {
@@ -1265,6 +1334,7 @@ gimp_menu_model_ui_removed (GimpUIManager *manager,
 
       g_clear_object (&subsection);
     }
+  g_free (section_name);
 
   return removed;
 }
