@@ -4,10 +4,10 @@
  * gimpbase-private.c
  * Copyright (C) 2003 Sven Neumann <sven@gimp.org>
  *
- * This library is free software; you can redistribute it and/or
+ * This library is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 3 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,23 +15,22 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library.  If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
-#include <glib-object.h>
-
-#include "libgimpbase/gimpbase.h"
+#include <gio/gio.h>
 
 #include "gimpbasetypes.h"
 
 #include "gimpbase-private.h"
+#include "gimpcompatenums.h"
 
 
-GimpUnitVtable _gimp_unit_vtable = { NULL, };
+GHashTable     *_gimp_units       = NULL;
+GimpUnitVtable  _gimp_unit_vtable = { NULL, };
 
 
 void
@@ -46,5 +45,36 @@ gimp_base_init (GimpUnitVtable *vtable)
 
   _gimp_unit_vtable = *vtable;
 
+  gimp_base_compat_enums_init ();
+
   gimp_base_initialized = TRUE;
 }
+
+void
+gimp_base_exit (void)
+{
+  g_clear_pointer (&_gimp_units, g_hash_table_unref);
+}
+
+void
+gimp_base_compat_enums_init (void)
+{
+#if 0
+  static gboolean gimp_base_compat_initialized = FALSE;
+  GQuark          quark;
+
+  if (gimp_base_compat_initialized)
+    return;
+
+  quark = g_quark_from_static_string ("gimp-compat-enum");
+
+  /*  This is how a compat enum is registered, leave one here for
+   *  documentation purposes, remove it as soon as we get a real
+   *  compat enum again
+   */
+  g_type_set_qdata (GIMP_TYPE_ADD_MASK_TYPE, quark,
+                    (gpointer) GIMP_TYPE_ADD_MASK_TYPE_COMPAT);
+
+  gimp_base_compat_initialized = TRUE;
+#endif
+}
