@@ -32,7 +32,6 @@
 #include "gimpconfigtypes.h"
 
 #include "gimpconfigwriter.h"
-#include "gimpconfig-array.h"
 #include "gimpconfig-iface.h"
 #include "gimpconfig-params.h"
 #include "gimpconfig-path.h"
@@ -49,9 +48,10 @@
  **/
 
 
-static gboolean  gimp_config_serialize_rgb (const GValue *value,
-                                            GString      *str,
-                                            gboolean      has_alpha);
+static gboolean gimp_config_serialize_strv  (const GValue *value,
+                                             GString      *str);
+static gboolean gimp_config_serialize_array (const GValue *value,
+                                             GString      *str);
 
 
 /**
@@ -73,6 +73,7 @@ gimp_config_serialize_properties (GimpConfig       *config,
   GParamSpec   **property_specs;
   guint          n_property_specs;
   guint          i;
+  gboolean       success = TRUE;
 
   g_return_val_if_fail (G_IS_OBJECT (config), FALSE);
 
@@ -81,7 +82,7 @@ gimp_config_serialize_properties (GimpConfig       *config,
   property_specs = g_object_class_list_properties (klass, &n_property_specs);
 
   if (! property_specs)
-    return TRUE;
+    return success;
 
   for (i = 0; i < n_property_specs; i++)
     {
@@ -90,13 +91,16 @@ gimp_config_serialize_properties (GimpConfig       *config,
       if (! (prop_spec->flags & GIMP_CONFIG_PARAM_SERIALIZE))
         continue;
 
+      /* Some properties may fail writing, which shouldn't break serializing
+       * more properties, yet final result would be a (partial) failure.
+       */
       if (! gimp_config_serialize_property (config, prop_spec, writer))
-        return FALSE;
+        success = FALSE;
     }
 
   g_free (property_specs);
 
-  return TRUE;
+  return success;
 }
 
 /**
@@ -119,7 +123,8 @@ gimp_config_serialize_changed_properties (GimpConfig       *config,
   GParamSpec   **property_specs;
   guint          n_property_specs;
   guint          i;
-  GValue         value = G_VALUE_INIT;
+  GValue         value   = G_VALUE_INIT;
+  gboolean       success = TRUE;
 
   g_return_val_if_fail (G_IS_OBJECT (config), FALSE);
 
@@ -128,7 +133,7 @@ gimp_config_serialize_changed_properties (GimpConfig       *config,
   property_specs = g_object_class_list_properties (klass, &n_property_specs);
 
   if (! property_specs)
-    return TRUE;
+    return success;
 
   for (i = 0; i < n_property_specs; i++)
     {
@@ -142,8 +147,11 @@ gimp_config_serialize_changed_properties (GimpConfig       *config,
 
       if (! g_param_value_defaults (prop_spec, &value))
         {
+          /* Some properties may fail writing, which shouldn't break serializing
+           * more properties, yet final result would be a (partial) failure.
+           */
           if (! gimp_config_serialize_property (config, prop_spec, writer))
-            return FALSE;
+            success = FALSE;
         }
 
       g_value_unset (&value);
@@ -151,7 +159,7 @@ gimp_config_serialize_changed_properties (GimpConfig       *config,
 
   g_free (property_specs);
 
-  return TRUE;
+  return success;
 }
 
 /**
@@ -179,7 +187,8 @@ gimp_config_serialize_property (GimpConfig       *config,
   if (! (param_spec->flags & GIMP_CONFIG_PARAM_SERIALIZE))
     return FALSE;
 
-  if (param_spec->flags & GIMP_CONFIG_PARAM_IGNORE)
+  if (param_spec->flags & GIMP_CONFIG_PARAM_IGNORE ||
+      param_spec->flags & GIMP_PARAM_DONT_SERIALIZE)
     return TRUE;
 
   g_value_init (&value, param_spec->value_type);
@@ -267,6 +276,120 @@ gimp_config_serialize_property (GimpConfig       *config,
           else
             gimp_config_writer_revert (writer);
         }
+      else if (G_VALUE_TYPE (&value) == G_TYPE_BYTES)
+        {
+          GBytes *bytes = g_value_get_boxed (&value);
+
+          gimp_config_writer_open (writer, param_spec->name);
+
+          if (bytes)
+            {
+              gconstpointer data;
+              gsize         data_length;
+
+              data = g_bytes_get_data (bytes, &data_length);
+
+              gimp_config_writer_printf (writer, "%" G_GSIZE_FORMAT, data_length);
+              gimp_config_writer_data (writer, data_length, data);
+            }
+          else
+            {
+              gimp_config_writer_printf (writer, "%s", "NULL");
+            }
+
+          success = TRUE;
+          gimp_config_writer_close (writer);
+        }
+      else if (GIMP_VALUE_HOLDS_COLOR (&value))
+        {
+          GeglColor *color      = g_value_get_object (&value);
+          gboolean   free_color = FALSE;
+
+          gimp_config_writer_open (writer, param_spec->name);
+
+          if (color)
+            {
+              const gchar   *encoding;
+              const Babl    *format = gegl_color_get_format (color);
+              const Babl    *space;
+              GBytes        *bytes;
+              gconstpointer  data;
+              gsize          data_length;
+              int            profile_length = 0;
+
+              gimp_config_writer_open (writer, "color");
+
+              if (babl_format_is_palette (format))
+                {
+                  guint8 pixel[40];
+
+                  /* As a special case, we don't want to serialize
+                   * palette colors, because they are just too much
+                   * dependent on external data and cannot be
+                   * deserialized back safely. So we convert them first.
+                   */
+                   free_color = TRUE;
+                   color = gegl_color_duplicate (color);
+
+                   format = babl_format_with_space ("R'G'B'A u8", format);
+                   gegl_color_get_pixel (color, format, pixel);
+                   gegl_color_set_pixel (color, format, pixel);
+                }
+
+              encoding = babl_format_get_encoding (format);
+              gimp_config_writer_string (writer, encoding);
+
+              bytes = gegl_color_get_bytes (color, format);
+              data  = g_bytes_get_data (bytes, &data_length);
+
+              gimp_config_writer_printf (writer, "%" G_GSIZE_FORMAT, data_length);
+              gimp_config_writer_data (writer, data_length, data);
+
+              space = babl_format_get_space (format);
+              if (space != babl_space ("sRGB"))
+                {
+                  guint8 *profile_data;
+
+                  profile_data = (guint8 *) babl_space_get_icc (babl_format_get_space (format),
+                                                                &profile_length);
+                  gimp_config_writer_printf (writer, "%u", profile_length);
+                  if (profile_data)
+                    gimp_config_writer_data (writer, profile_length, profile_data);
+                }
+              else
+                {
+                  gimp_config_writer_printf (writer, "%u", profile_length);
+                }
+
+              g_bytes_unref (bytes);
+
+              gimp_config_writer_close (writer);
+            }
+          else
+            {
+              gimp_config_writer_printf (writer, "%s", "NULL");
+            }
+
+          success = TRUE;
+          gimp_config_writer_close (writer);
+
+          if (free_color)
+            g_object_unref (color);
+        }
+      else if (GIMP_VALUE_HOLDS_UNIT (&value))
+        {
+          GimpUnit *unit = g_value_get_object (&value);
+
+          gimp_config_writer_open (writer, param_spec->name);
+
+          if (unit)
+            gimp_config_writer_printf (writer, "%s", gimp_unit_get_name (unit));
+          else
+            gimp_config_writer_printf (writer, "%s", "NULL");
+
+          success = TRUE;
+          gimp_config_writer_close (writer);
+        }
       else if (G_VALUE_HOLDS_OBJECT (&value) &&
                G_VALUE_TYPE (&value) != G_TYPE_FILE)
         {
@@ -307,20 +430,7 @@ gimp_config_serialize_property (GimpConfig       *config,
         {
           GString *str = g_string_new (NULL);
 
-          if (GIMP_VALUE_HOLDS_RGB (&value))
-            {
-              gboolean has_alpha = gimp_param_spec_rgb_has_alpha (param_spec);
-
-              success = gimp_config_serialize_rgb (&value, str, has_alpha);
-            }
-          else if (G_VALUE_TYPE (&value) == G_TYPE_STRV)
-            {
-              success = gimp_config_serialize_strv (&value, str);
-            }
-          else
-            {
-              success = gimp_config_serialize_value (&value, str, TRUE);
-            }
+          success = gimp_config_serialize_value (&value, str, TRUE);
 
           if (success)
             {
@@ -399,6 +509,17 @@ gimp_config_serialize_value (const GValue *value,
                              GString      *str,
                              gboolean      escaped)
 {
+  if (G_VALUE_TYPE (value) == G_TYPE_STRV)
+    {
+      return gimp_config_serialize_strv (value, str);
+    }
+
+  if (GIMP_VALUE_HOLDS_INT32_ARRAY (value) ||
+      GIMP_VALUE_HOLDS_DOUBLE_ARRAY (value))
+    {
+      return gimp_config_serialize_array (value, str);
+    }
+
   if (G_VALUE_HOLDS_BOOLEAN (value))
     {
       gboolean bool;
@@ -457,11 +578,6 @@ gimp_config_serialize_value (const GValue *value,
       return TRUE;
     }
 
-  if (GIMP_VALUE_HOLDS_RGB (value))
-    {
-      return gimp_config_serialize_rgb (value, str, TRUE);
-    }
-
   if (GIMP_VALUE_HOLDS_MATRIX2 (value))
     {
       GimpMatrix2 *trafo;
@@ -526,7 +642,7 @@ gimp_config_serialize_value (const GValue *value,
 
       if (file)
         {
-          gchar *path     = g_file_get_path (file);
+          gchar *path     = g_file_get_uri (file);
           gchar *unexpand = NULL;
 
           if (path)
@@ -573,38 +689,103 @@ gimp_config_serialize_value (const GValue *value,
   return FALSE;
 }
 
+
+/* Private functions */
+
+/**
+ * gimp_config_serialize_strv:
+ * @value: source #GValue holding a #GStrv
+ * @str:   destination string
+ *
+ * Appends a string repr of the #GStrv value of #GValue to @str.
+ * Repr is an integer literal greater than or equal to zero,
+ * followed by a possibly empty sequence
+ * of quoted and escaped string literals.
+ *
+ * Returns: %TRUE always
+ *
+ * Since: 3.0
+ **/
 static gboolean
-gimp_config_serialize_rgb (const GValue *value,
-                           GString      *str,
-                           gboolean      has_alpha)
+gimp_config_serialize_strv (const GValue *value,
+                            GString      *str)
 {
-  GimpRGB *rgb;
+  GStrv gstrv;
 
-  rgb = g_value_get_boxed (value);
+  gstrv = g_value_get_boxed (value);
 
-  if (rgb)
+  if (gstrv)
     {
-      gchar buf[4][G_ASCII_DTOSTR_BUF_SIZE];
+      gint length = g_strv_length (gstrv);
 
-      g_ascii_dtostr (buf[0], G_ASCII_DTOSTR_BUF_SIZE, rgb->r);
-      g_ascii_dtostr (buf[1], G_ASCII_DTOSTR_BUF_SIZE, rgb->g);
-      g_ascii_dtostr (buf[2], G_ASCII_DTOSTR_BUF_SIZE, rgb->b);
+      /* Write length */
+      g_string_append_printf (str, "%d", length);
 
-      if (has_alpha)
+      for (gint i = 0; i < length; i++)
         {
-          g_ascii_dtostr (buf[3], G_ASCII_DTOSTR_BUF_SIZE, rgb->a);
-
-          g_string_append_printf (str, "(color-rgba %s %s %s %s)",
-                                  buf[0], buf[1], buf[2], buf[3]);
+          g_string_append (str, " "); /* separator */
+          gimp_config_string_append_escaped (str, gstrv[i]);
         }
-      else
-        {
-          g_string_append_printf (str, "(color-rgb %s %s %s)",
-                                  buf[0], buf[1], buf[2]);
-        }
-
-      return TRUE;
+    }
+  else
+    {
+      /* GValue has NULL value. Not quite the same as an empty GStrv.
+       * But handle it quietly as an empty GStrv: write a length of zero.
+       */
+      g_string_append (str, "0");
     }
 
-  return FALSE;
+  return TRUE;
+}
+
+static gboolean
+gimp_config_serialize_array (const GValue *value,
+                             GString      *str)
+{
+  GimpArray *array;
+
+  g_return_val_if_fail (GIMP_VALUE_HOLDS_INT32_ARRAY (value) ||
+                        GIMP_VALUE_HOLDS_DOUBLE_ARRAY (value), FALSE);
+
+  array = g_value_get_boxed (value);
+
+  if (array)
+    {
+      gint32 *values = (gint32 *) array->data;
+      gint    length;
+
+      if (GIMP_VALUE_HOLDS_INT32_ARRAY (value))
+        length = array->length / sizeof (gint32);
+      else
+        length = array->length / sizeof (gdouble);
+
+      /* Write length */
+      g_string_append_printf (str, "%d", length);
+
+      for (gint i = 0; i < length; i++)
+        {
+          gchar *num_str;
+
+          if (GIMP_VALUE_HOLDS_INT32_ARRAY (value))
+            {
+              num_str = g_strdup_printf (" %d", values[i]);
+            }
+          else
+            {
+              gchar buf[G_ASCII_DTOSTR_BUF_SIZE];
+
+              g_ascii_dtostr (buf, sizeof (buf), ((gdouble *) values)[i]);
+              num_str = g_strdup_printf (" %s", buf);
+            }
+
+          g_string_append (str, num_str);
+          g_free (num_str);
+        }
+    }
+  else
+    {
+      g_string_append (str, "0");
+    }
+
+  return TRUE;
 }
