@@ -2,7 +2,7 @@
  * Copyright (C) 1995-1997 Peter Mattis and Spencer Kimball
  *
  * Thumbnail handling according to the Thumbnail Managing Standard.
- * http://triq.net/~pearl/thumbnail-spec/
+ * https://specifications.freedesktop.org/thumbnail-spec/
  *
  * Copyright (C) 2001-2003  Sven Neumann <sven@gimp.org>
  *                          Michael Natterer <mitch@gimp.org>
@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library.  If not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
@@ -28,15 +28,11 @@
 #include <string.h>
 #include <sys/types.h>
 
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
 #ifdef PLATFORM_OSX
 #include <AppKit/AppKit.h>
 #endif
 
-#include <glib-object.h>
+#include <gio/gio.h>
 #include <glib/gstdio.h>
 
 #ifdef G_OS_WIN32
@@ -99,7 +95,7 @@ static gchar        *thumb_fail_subdir      = NULL;
  * application-specific thumbnails, you can specify a different base
  * directory here.
  *
- * Return value: %TRUE if the library was successfully initialized.
+ * Returns: %TRUE if the library was successfully initialized.
  **/
 gboolean
 gimp_thumb_init (const gchar *creator,
@@ -107,7 +103,7 @@ gimp_thumb_init (const gchar *creator,
 {
   GEnumClass *enum_class;
   GEnumValue *enum_value;
-  gint        i;
+  guint        i;
 
   g_return_val_if_fail (creator != NULL, FALSE);
   g_return_val_if_fail (thumb_basedir == NULL ||
@@ -141,11 +137,11 @@ gimp_thumb_init (const gchar *creator,
 
 #else
 
-      const gchar *home_dir = g_get_home_dir ();
+      const gchar *cache_dir = g_get_user_cache_dir ();
 
-      if (home_dir && g_file_test (home_dir, G_FILE_TEST_IS_DIR))
+      if (cache_dir && g_file_test (cache_dir, G_FILE_TEST_IS_DIR))
         {
-          thumb_dir = g_build_filename (home_dir, ".thumbnails", NULL);
+          thumb_dir = g_build_filename (cache_dir, "thumbnails", NULL);
         }
 
 #endif
@@ -154,7 +150,7 @@ gimp_thumb_init (const gchar *creator,
         {
           gchar *name = g_filename_display_name (g_get_tmp_dir ());
 
-          g_message (_("Cannot determine a valid home directory.\n"
+          g_message (_("Cannot determine a valid thumbnails directory.\n"
                        "Thumbnails will be stored in the folder for "
                        "temporary files (%s) instead."), name);
           g_free (name);
@@ -190,6 +186,28 @@ gimp_thumb_init (const gchar *creator,
   return gimp_thumb_initialized;
 }
 
+/**
+ * gimp_thumb_get_thumb_base_dir:
+ *
+ * Returns the base directory of thumbnails cache.
+ * It uses the Freedesktop Thumbnail Managing Standard on UNIX,
+ * "~/Library/Caches/org.freedesktop.thumbnails" on OSX, and a cache
+ * folder determined by glib on Windows (currently the common repository
+ * for temporary Internet files).
+ * The returned string belongs to GIMP and must not be changed nor freed.
+ *
+ * Returns: the thumbnails cache directory.
+ *
+ * Since: 2.10
+ **/
+const gchar *
+gimp_thumb_get_thumb_base_dir (void)
+{
+  g_return_val_if_fail (gimp_thumb_initialized, NULL);
+
+  return thumb_dir;
+}
+
 /**
  * gimp_thumb_get_thumb_dir:
  * @size: a GimpThumbSize
@@ -198,7 +216,7 @@ gimp_thumb_init (const gchar *creator,
  * returned pointer will become invalid if gimp_thumb_init() is used
  * again. It must not be changed or freed.
  *
- * Return value: the thumbnail directory in the encoding of the filesystem
+ * Returns: the thumbnail directory in the encoding of the filesystem
  **/
 const gchar *
 gimp_thumb_get_thumb_dir (GimpThumbSize  size)
@@ -219,9 +237,9 @@ gimp_thumb_get_thumb_dir (GimpThumbSize  size)
  * size.  Unlike gimp_thumb_get_thumb_dir() the returned string is not
  * constant and should be free'd when it is not any longer needed.
  *
- * Return value: the thumbnail directory in the encoding of the filesystem
+ * Returns: the thumbnail directory in the encoding of the filesystem
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gchar *
 gimp_thumb_get_thumb_dir_local (const gchar   *dirname,
@@ -249,8 +267,8 @@ gimp_thumb_get_thumb_dir_local (const gchar   *dirname,
  * gimp_thumbnail_save_thumb() and gimp_thumbnail_save_failure() will
  * do this for you.
  *
- * Return value: %TRUE is the directory exists, %FALSE if it could not
- *               be created
+ * Returns: %TRUE is the directory exists, %FALSE if it could not
+ *          be created
  **/
 gboolean
 gimp_thumb_ensure_thumb_dir (GimpThumbSize   size,
@@ -297,10 +315,10 @@ gimp_thumb_ensure_thumb_dir (GimpThumbSize   size,
  * You shouldn't have to call this function directly since
  * gimp_thumbnail_save_thumb_local() will do this for you.
  *
- * Return value: %TRUE is the directory exists, %FALSE if it could not
+ * Returns: %TRUE is the directory exists, %FALSE if it could not
  *               be created
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gboolean
 gimp_thumb_ensure_thumb_dir_local (const gchar    *dirname,
@@ -361,7 +379,7 @@ gimp_thumb_ensure_thumb_dir_local (const gchar    *dirname,
  * Creates the name of the thumbnail file of the specified @size that
  * belongs to an image file located at the given @uri.
  *
- * Return value: a newly allocated filename in the encoding of the
+ * Returns: a newly allocated filename in the encoding of the
  *               filesystem or %NULL if @uri points to the user's
  *               thumbnail repository.
  **/
@@ -391,11 +409,11 @@ gimp_thumb_name_from_uri (const gchar   *uri,
  * that belongs to an image file located at the given @uri. Local
  * thumbnails have been introduced with version 0.7 of the spec.
  *
- * Return value: a newly allocated filename in the encoding of the
+ * Returns: a newly allocated filename in the encoding of the
  *               filesystem or %NULL if @uri is a remote file or
  *               points to the user's thumbnail repository.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 gchar *
 gimp_thumb_name_from_uri_local (const gchar   *uri,
@@ -454,7 +472,7 @@ gimp_thumb_name_from_uri_local (const gchar   *uri,
  * If a thumbnail is found, it's size is written to the variable
  * pointer to by @size and the file location is returned.
  *
- * Return value: a newly allocated string in the encoding of the
+ * Returns: a newly allocated string in the encoding of the
  *               filesystem or %NULL if no thumbnail for @uri was found
  **/
 gchar *
@@ -506,7 +524,7 @@ gimp_thumb_find_thumb (const gchar   *uri,
  * checks if the given @filename exists and returns modification time
  * and file size in 64bit integer values.
  *
- * Return value: The type of the file, or #GIMP_THUMB_FILE_TYPE_NONE if
+ * Returns: The type of the file, or #GIMP_THUMB_FILE_TYPE_NONE if
  *               the file doesn't exist.
  **/
 GimpThumbFileType
@@ -515,33 +533,62 @@ gimp_thumb_file_test (const gchar *filename,
                       gint64      *size,
                       gint        *err_no)
 {
-  struct stat s;
+  GimpThumbFileType  type = GIMP_THUMB_FILE_TYPE_NONE;
+  GFile             *file;
+  GFileInfo         *info;
 
   g_return_val_if_fail (filename != NULL, FALSE);
 
-  if (g_stat (filename, &s) == 0)
+  file = g_file_new_for_path (filename);
+
+  info = g_file_query_info (file,
+                            G_FILE_ATTRIBUTE_STANDARD_TYPE ","
+                            G_FILE_ATTRIBUTE_STANDARD_SIZE ","
+                            G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                            G_FILE_QUERY_INFO_NONE,
+                            NULL, NULL);
+
+  if (info)
     {
-      if (mtime)  *mtime  = s.st_mtime;
-      if (size)   *size   = s.st_size;
-      if (err_no) *err_no = 0;
+      if (mtime)
+        *mtime =
+          g_file_info_get_attribute_uint64 (info,
+                                            G_FILE_ATTRIBUTE_TIME_MODIFIED);
 
-      if (S_ISREG (s.st_mode))
+      if (size)
+        *size = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_STANDARD_SIZE);
+
+      if (err_no)
+        *err_no = 0;
+
+      switch (g_file_info_get_attribute_uint32 (info,
+                                                G_FILE_ATTRIBUTE_STANDARD_TYPE))
         {
-          return GIMP_THUMB_FILE_TYPE_REGULAR;
-        }
-      else if (S_ISDIR (s.st_mode))
-        {
-          return GIMP_THUMB_FILE_TYPE_FOLDER;
+        case G_FILE_TYPE_REGULAR:
+          type = GIMP_THUMB_FILE_TYPE_REGULAR;
+          break;
+
+        case G_FILE_TYPE_DIRECTORY:
+          type = GIMP_THUMB_FILE_TYPE_FOLDER;
+          break;
+
+        default:
+          type = GIMP_THUMB_FILE_TYPE_SPECIAL;
+          break;
         }
 
-      return GIMP_THUMB_FILE_TYPE_SPECIAL;
+      g_object_unref (info);
+    }
+  else
+    {
+      if (mtime)  *mtime  = 0;
+      if (size)   *size   = 0;
+      if (err_no) *err_no = ENOENT;
     }
 
-  if (mtime)  *mtime  = 0;
-  if (size)   *size   = 0;
-  if (err_no) *err_no = errno;
+  g_object_unref (file);
 
-  return GIMP_THUMB_FILE_TYPE_NONE;
+  return type;
 }
 
 /**
@@ -551,7 +598,7 @@ gimp_thumb_file_test (const gchar *filename,
  * Deletes all thumbnails for the image file specified by @uri from the
  * user's thumbnail repository.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_thumbs_delete_for_uri (const gchar *uri)
@@ -580,7 +627,7 @@ gimp_thumbs_delete_for_uri (const gchar *uri)
  * Deletes all thumbnails for the image file specified by @uri from
  * the local thumbnail repository.
  *
- * Since: GIMP 2.2
+ * Since: 2.2
  **/
 void
 gimp_thumbs_delete_for_uri_local (const gchar *uri)
