@@ -12,11 +12,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <gegl.h>
 #include <gtk/gtk.h>
 
 #include "libgimpcolor/gimpcolor.h"
@@ -46,7 +47,6 @@ struct _CdisplayContrast
   GimpColorDisplay  parent_instance;
 
   gdouble           contrast;
-  guchar            lookup[256];
 };
 
 struct _CdisplayContrastClass
@@ -73,9 +73,9 @@ static void        cdisplay_contrast_get_property    (GObject          *object,
                                                       GValue           *value,
                                                       GParamSpec       *pspec);
 
-static void        cdisplay_contrast_convert_surface (GimpColorDisplay *display,
-                                                      cairo_surface_t  *surface);
-static GtkWidget * cdisplay_contrast_configure       (GimpColorDisplay *display);
+static void        cdisplay_contrast_convert_buffer  (GimpColorDisplay *display,
+                                                      GeglBuffer       *buffer,
+                                                      GeglRectangle    *area);
 static void        cdisplay_contrast_set_contrast    (CdisplayContrast *contrast,
                                                       gdouble           value);
 
@@ -90,6 +90,7 @@ static const GimpModuleInfo cdisplay_contrast_info =
   "October 14, 2000"
 };
 
+
 G_DEFINE_DYNAMIC_TYPE (CdisplayContrast, cdisplay_contrast,
                        GIMP_TYPE_COLOR_DISPLAY)
 
@@ -117,17 +118,18 @@ cdisplay_contrast_class_init (CdisplayContrastClass *klass)
   object_class->get_property     = cdisplay_contrast_get_property;
   object_class->set_property     = cdisplay_contrast_set_property;
 
-  GIMP_CONFIG_INSTALL_PROP_DOUBLE (object_class, PROP_CONTRAST,
-                                   "contrast", NULL,
-                                   0.01, 10.0, DEFAULT_CONTRAST,
-                                   0);
+  GIMP_CONFIG_PROP_DOUBLE (object_class, PROP_CONTRAST,
+                           "contrast",
+                           _("Contrast cycles"),
+                           NULL,
+                           0.01, 10.0, DEFAULT_CONTRAST,
+                           0);
 
   display_class->name            = _("Contrast");
   display_class->help_id         = "gimp-colordisplay-contrast";
-  display_class->stock_id        = GIMP_STOCK_DISPLAY_FILTER_CONTRAST;
+  display_class->icon_name       = GIMP_ICON_DISPLAY_FILTER_CONTRAST;
 
-  display_class->convert_surface = cdisplay_contrast_convert_surface;
-  display_class->configure       = cdisplay_contrast_configure;
+  display_class->convert_buffer  = cdisplay_contrast_convert_buffer;
 }
 
 static void
@@ -179,75 +181,36 @@ cdisplay_contrast_set_property (GObject      *object,
 }
 
 static void
-cdisplay_contrast_convert_surface (GimpColorDisplay *display,
-                                   cairo_surface_t  *surface)
+cdisplay_contrast_convert_buffer (GimpColorDisplay *display,
+                                  GeglBuffer       *buffer,
+                                  GeglRectangle    *area)
 {
-  CdisplayContrast *contrast = CDISPLAY_CONTRAST (display);
-  gint              width    = cairo_image_surface_get_width (surface);
-  gint              height   = cairo_image_surface_get_height (surface);
-  gint              stride   = cairo_image_surface_get_stride (surface);
-  guchar           *buf      = cairo_image_surface_get_data (surface);
-  cairo_format_t    fmt      = cairo_image_surface_get_format (surface);
-  gint              i, j, skip;
-  gint              r, g, b, a;
+  CdisplayContrast   *contrast = CDISPLAY_CONTRAST (display);
+  GeglBufferIterator *iter;
+  gfloat              c;
 
-  if (fmt != CAIRO_FORMAT_ARGB32)
-    return;
+  c = contrast->contrast * 2 * G_PI;
 
-  /* You will not be using the entire buffer most of the time.
-   * Hence, the simplistic code for this is as follows:
-   *
-   * for (j = 0; j < height; j++)
-   *   {
-   *     for (i = 0; i < width * bpp; i++)
-   *       buf[i] = lookup[buf[i]];
-   *     buf += bpl;
-   *   }
-   */
+  iter = gegl_buffer_iterator_new (buffer, area, 0,
+                                   babl_format ("R'G'B'A float"),
+                                   GEGL_ACCESS_READWRITE, GEGL_ABYSS_NONE, 1);
 
-  j = height;
-  skip = stride - 4 * width;
-
-  while (j--)
+  while (gegl_buffer_iterator_next (iter))
     {
-      i = width;
-      while (i--)
+      gfloat *data  = iter->items[0].data;
+      gint    count = iter->length;
+
+      while (count--)
         {
-          GIMP_CAIRO_ARGB32_GET_PIXEL (buf, r, g, b, a);
-          r = contrast->lookup[r];
-          g = contrast->lookup[g];
-          b = contrast->lookup[b];
-          GIMP_CAIRO_ARGB32_SET_PIXEL (buf, r, g, b, a);
-          buf += 4;
+          *data = 0.5 * (1.0 + sin (c * *data)); data++;
+          *data = 0.5 * (1.0 + sin (c * *data)); data++;
+          *data = 0.5 * (1.0 + sin (c * *data)); data++;
+
+          data++;
         }
-      buf += skip;
     }
 }
 
-static GtkWidget *
-cdisplay_contrast_configure (GimpColorDisplay *display)
-{
-  CdisplayContrast *contrast = CDISPLAY_CONTRAST (display);
-  GtkWidget        *hbox;
-  GtkWidget        *label;
-  GtkWidget        *spinbutton;
-
-  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
-
-  label = gtk_label_new_with_mnemonic (_("Contrast c_ycles:"));
-  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-  gtk_widget_show (label);
-
-  spinbutton = gimp_prop_spin_button_new (G_OBJECT (contrast), "contrast",
-                                          0.1, 1.0, 3);
-  gtk_box_pack_start (GTK_BOX (hbox), spinbutton, FALSE, FALSE, 0);
-  gtk_widget_show (spinbutton);
-
-  gtk_label_set_mnemonic_widget (GTK_LABEL (label), spinbutton);
-
-  return hbox;
-}
-
 static void
 cdisplay_contrast_set_contrast (CdisplayContrast *contrast,
                                 gdouble           value)
@@ -257,16 +220,8 @@ cdisplay_contrast_set_contrast (CdisplayContrast *contrast,
 
   if (value != contrast->contrast)
     {
-      gint i;
-
       contrast->contrast = value;
 
-      for (i = 0; i < 256; i++)
-        {
-          contrast->lookup[i] = (guchar) (gint)
-            (255 * .5 * (1 + sin (value * 2 * G_PI * i / 255.0)));
-        }
-
       g_object_notify (G_OBJECT (contrast), "contrast");
       gimp_color_display_changed (GIMP_COLOR_DISPLAY (contrast));
     }
