@@ -44,7 +44,7 @@
 #define PLUG_IN_BINARY "blinds"
 #define PLUG_IN_ROLE   "gimp-blinds"
 
-#define MAX_FANS       100
+#define MAX_FANS       1024
 
 
 typedef struct _Blinds      Blinds;
@@ -62,7 +62,7 @@ struct _BlindsClass
 
 
 #define BLINDS_TYPE  (blinds_get_type ())
-#define BLINDS (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BLINDS_TYPE, Blinds))
+#define BLINDS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), BLINDS_TYPE, Blinds))
 
 GType                   blinds_get_type         (void) G_GNUC_CONST;
 
@@ -73,9 +73,8 @@ static GimpProcedure  * blinds_create_procedure (GimpPlugIn           *plug_in,
 static GimpValueArray * blinds_run              (GimpProcedure        *procedure,
                                                  GimpRunMode           run_mode,
                                                  GimpImage            *image,
-                                                 gint                  n_drawables,
                                                  GimpDrawable        **drawables,
-                                                 const GimpValueArray *args,
+                                                 GimpProcedureConfig  *config,
                                                  gpointer              run_data);
 
 static gboolean         blinds_dialog           (GimpProcedure        *procedure,
@@ -150,31 +149,32 @@ blinds_create_procedure (GimpPlugIn  *plug_in,
                                       "Andy Thomas",
                                       "1997");
 
-      GIMP_PROC_ARG_INT (procedure, "angle-displacement",
-                         _("_Displacement"),
-                         _("Angle of Displacement"),
-                         0, 360, 30,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "angle-displacement",
+                                       _("_Displacement"),
+                                       _("Angle of Displacement"),
+                                       0, 90, 30,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "num-segments",
-                         _("_Number of segments"),
-                         _("Number of segments in blinds"),
-                         1, 1024, 3,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "num-segments",
+                                       _("_Number of segments"),
+                                       _("Number of segments in blinds"),
+                                       1, MAX_FANS, 3,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "orientation",
-                         _("Orientation"),
-                         _("The orientation"),
-                         GIMP_ORIENTATION_HORIZONTAL,
-                         GIMP_ORIENTATION_VERTICAL,
-                         GIMP_ORIENTATION_HORIZONTAL,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "orientation",
+                                          _("Orient_ation"),
+                                          _("The orientation"),
+                                          gimp_choice_new_with_values ("horizontal", GIMP_ORIENTATION_HORIZONTAL, _("Horizontal"),  NULL,
+                                                                       "vertical",   GIMP_ORIENTATION_VERTICAL,   _("Vertical"),    NULL,
+                                                                       NULL),
+                                          "horizontal",
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_BOOLEAN (procedure, "bg-transparent",
-                             _("_Transparent"),
-                             _("Background transparent"),
-                             FALSE,
-                             G_PARAM_READWRITE);
+      gimp_procedure_add_boolean_argument (procedure, "bg-transparent",
+                                           _("_Transparent"),
+                                           _("Background transparent"),
+                                           FALSE,
+                                           G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -184,17 +184,15 @@ static GimpValueArray *
 blinds_run (GimpProcedure        *procedure,
             GimpRunMode           run_mode,
             GimpImage            *image,
-            gint                  n_drawables,
             GimpDrawable        **drawables,
-            const GimpValueArray *args,
+            GimpProcedureConfig  *config,
             gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpDrawable        *drawable;
+  GimpDrawable *drawable;
 
   gegl_init (NULL, NULL);
 
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
@@ -211,26 +209,10 @@ blinds_run (GimpProcedure        *procedure,
       drawable = drawables[0];
     }
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-      if (! blinds_dialog (procedure, G_OBJECT (config), drawable))
-        {
-          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-          g_object_unref (config);
-
-          return gimp_procedure_new_return_values (procedure,
-                                                   GIMP_PDB_CANCEL,
-                                                   NULL);
-        }
-      break;
-
-    default:
-      break;
-    }
+  if (run_mode == GIMP_RUN_INTERACTIVE && ! blinds_dialog (procedure, G_OBJECT (config), drawable))
+    return gimp_procedure_new_return_values (procedure,
+                                             GIMP_PDB_CANCEL,
+                                             NULL);
 
   if (gimp_drawable_is_rgb  (drawable) ||
       gimp_drawable_is_gray (drawable))
@@ -244,17 +226,11 @@ blinds_run (GimpProcedure        *procedure,
     }
   else
     {
-      gimp_procedure_config_end_run (config, GIMP_PDB_EXECUTION_ERROR);
-      g_object_unref (config);
-
       return gimp_procedure_new_return_values (procedure,
                                                GIMP_PDB_EXECUTION_ERROR,
                                                NULL);
     }
 
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
@@ -269,7 +245,6 @@ blinds_dialog (GimpProcedure *procedure,
   GtkWidget    *vbox;
   GtkWidget    *hbox;
   GtkWidget    *scale;
-  GtkListStore *store;
   gboolean      run;
 
   gimp_ui_init (PLUG_IN_BINARY);
@@ -278,27 +253,11 @@ blinds_dialog (GimpProcedure *procedure,
                                       GIMP_PROCEDURE_CONFIG (config),
                                       _("Blinds"));
 
-  gimp_dialog_set_alternative_button_order (GTK_DIALOG (dialog),
-                                            GTK_RESPONSE_OK,
-                                            GTK_RESPONSE_CANCEL,
-                                           -1);
-
-  gimp_window_set_transient (GTK_WINDOW (dialog));
-
-  preview = gimp_drawable_preview_new_from_drawable (drawable);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      preview, TRUE, TRUE, 0);
-  gtk_widget_set_margin_bottom (preview, 12);
-  gtk_widget_show (preview);
-
-  store = gimp_int_store_new (_("Horizontal"), GIMP_ORIENTATION_HORIZONTAL,
-                              _("Vertical"),   GIMP_ORIENTATION_VERTICAL,
-                              NULL);
-  gimp_procedure_dialog_get_int_radio (GIMP_PROCEDURE_DIALOG (dialog),
-                                       "orientation", GIMP_INT_STORE (store));
+  gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                    "orientation", GIMP_TYPE_INT_RADIO_FRAME);
 
   gimp_procedure_dialog_get_label (GIMP_PROCEDURE_DIALOG (dialog),
-                                   "bg_label", _("Background"));
+                                   "bg_label", _("Background"), FALSE, FALSE);
 
   gimp_procedure_dialog_fill_frame (GIMP_PROCEDURE_DIALOG (dialog),
                                     "bg-frame", "bg_label", FALSE,
@@ -330,7 +289,9 @@ blinds_dialog (GimpProcedure *procedure,
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
   gtk_widget_set_size_request (vbox, 320, -1);
 
-  g_object_set_data (config, "drawable", drawable);
+  preview = gimp_procedure_dialog_get_drawable_preview (GIMP_PROCEDURE_DIALOG (dialog),
+                                                        "preview", drawable);
+  gtk_widget_set_margin_bottom (preview, 12);
 
   g_signal_connect (preview, "invalidated",
                     G_CALLBACK (dialog_update_preview),
@@ -341,10 +302,10 @@ blinds_dialog (GimpProcedure *procedure,
                             preview);
 
   gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
-                              "blinds-vbox",
+                              "preview", "blinds-vbox",
                               NULL);
 
-  gtk_widget_show (dialog);
+  gtk_widget_set_visible (dialog, TRUE);
 
   run = gimp_procedure_dialog_run (GIMP_PROCEDURE_DIALOG (dialog));
 
@@ -470,46 +431,51 @@ static void
 dialog_update_preview (GtkWidget *widget,
                        GObject   *config)
 {
-  GimpPreview        *preview  = GIMP_PREVIEW (widget);
-  GimpDrawable       *drawable = g_object_get_data (config, "drawable");
-  gint                y;
-  const guchar       *p;
-  guchar             *buffer;
-  GBytes             *cache;
-  const guchar       *cache_start;
-  GimpRGB             background;
-  guchar              bg[4];
-  gint                width;
-  gint                height;
-  gint                bpp;
-  GimpOrientationType orientation;
-  gint                bg_trans;
+  GimpDrawablePreview *preview  = GIMP_DRAWABLE_PREVIEW (widget);
+  GimpDrawable        *drawable = gimp_drawable_preview_get_drawable (preview);
+  gint                 y;
+  const guchar        *p;
+  guchar              *buffer;
+  GBytes              *cache;
+  const guchar        *cache_start;
+  GeglColor           *color;
+  guchar               bg[4];
+  gint                 width;
+  gint                 height;
+  gint                 bpp;
+  GimpOrientationType  orientation;
+  gint                 bg_trans;
 
   g_object_get (config,
                 "bg-transparent", &bg_trans,
-                "orientation",    &orientation,
                 NULL);
+  orientation = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                     "orientation");
 
-  gimp_preview_get_size (preview, &width, &height);
+  gimp_preview_get_size (GIMP_PREVIEW (preview), &width, &height);
   cache = gimp_drawable_get_thumbnail_data (drawable,
                                             width, height,
                                             &width, &height, &bpp);
   p = cache_start = g_bytes_get_data (cache, NULL);
 
-  gimp_context_get_background (&background);
+  color = gimp_context_get_background ();
 
   if (bg_trans)
-    gimp_rgb_set_alpha (&background, 0.0);
+    gimp_color_set_alpha (color, 0.0);
 
   if (gimp_drawable_is_gray (drawable))
     {
-      bg[0] = gimp_rgb_luminance_uchar (&background);
-      gimp_rgba_get_uchar (&background, NULL, NULL, NULL, bg + 3);
+      guchar luminance[2];
+
+      gegl_color_get_pixel (color, babl_format ("Y'A u8"), luminance);
+      bg[0] = luminance[0];
+      bg[3] = luminance[1];
     }
   else
     {
-      gimp_rgba_get_uchar (&background, bg, bg + 1, bg + 2, bg + 3);
+      gegl_color_get_pixel (color, babl_format ("R'G'B'A u8"), bg);
     }
+  g_object_unref (color);
 
   buffer = g_new (guchar, width * height * bpp);
 
@@ -582,7 +548,9 @@ dialog_update_preview (GtkWidget *widget,
       g_free (dr);
     }
 
-  gimp_preview_draw_buffer (preview, buffer, width * bpp);
+  gimp_preview_draw_buffer (GIMP_PREVIEW (preview), buffer, width * bpp);
+  gimp_preview_set_bounds (GIMP_PREVIEW (preview), 0, 0, width, height);
+  gimp_preview_set_size (GIMP_PREVIEW (preview), width, height);
 
   g_free (buffer);
   g_bytes_unref (cache);
@@ -605,7 +573,7 @@ apply_blinds (GObject      *config,
   guchar              *src_rows, *des_rows;
   gint                 bytes;
   gint                 x, y;
-  GimpRGB              background;
+  GeglColor           *background;
   guchar               bg[4];
   gint                 sel_x1, sel_y1;
   gint                 sel_width, sel_height;
@@ -614,15 +582,17 @@ apply_blinds (GObject      *config,
 
   g_object_get (config,
                 "bg-transparent", &bg_trans,
-                "orientation",    &orientation,
                 NULL);
+  orientation = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                     "orientation");
 
-  gimp_context_get_background (&background);
+  background = gimp_context_get_background ();
 
   if (bg_trans)
-    gimp_rgb_set_alpha (&background, 0.0);
+    gimp_color_set_alpha (background, 0.0);
 
-  gimp_rgba_get_uchar (&background, bg, bg + 1, bg + 2, bg + 3);
+  gegl_color_get_pixel (background, babl_format_with_space ("R'G'B'A u8", NULL), bg);
+  g_object_unref (background);
 
   if (! gimp_drawable_mask_intersect (drawable,
                                       &sel_x1, &sel_y1,
