@@ -24,6 +24,8 @@
 
 #include <stdlib.h>
 
+#include "libgimpcolor/gimpcolor-private.h"
+
 #include <libgimp/gimp.h>
 #include <libgimp/gimpui.h>
 
@@ -81,7 +83,7 @@ struct _DespeckleClass
 
 
 #define DESPECKLE_TYPE  (despeckle_get_type ())
-#define DESPECKLE (obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESPECKLE_TYPE, Despeckle))
+#define DESPECKLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), DESPECKLE_TYPE, Despeckle))
 
 GType                   despeckle_get_type         (void) G_GNUC_CONST;
 
@@ -92,9 +94,8 @@ static GimpProcedure  * despeckle_create_procedure (GimpPlugIn           *plug_i
 static GimpValueArray * despeckle_run              (GimpProcedure        *procedure,
                                                     GimpRunMode           run_mode,
                                                     GimpImage            *image,
-                                                    gint                  n_drawables,
                                                     GimpDrawable        **drawables,
-                                                    const GimpValueArray *args,
+                                                    GimpProcedureConfig  *config,
                                                     gpointer              run_data);
 
 static void             despeckle                  (GimpDrawable         *drawable,
@@ -181,30 +182,34 @@ despeckle_create_procedure (GimpPlugIn  *plug_in,
                                       "Copyright 1997-1998 by Michael Sweet",
                                       PLUG_IN_VERSION);
 
-      GIMP_PROC_ARG_INT (procedure, "radius",
-                         _("R_adius"),
-                         _("Filter box radius"),
-                         1, MAX_RADIUS, 3,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "radius",
+                                       _("R_adius"),
+                                       _("Filter box radius"),
+                                       1, MAX_RADIUS, 3,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "type",
-                         _("_Filter Type"),
-                         _("Filter type { MEDIAN (0), ADAPTIVE (1), "
-                         "RECURSIVE-MEDIAN (2), RECURSIVE-ADAPTIVE (3) }"),
-                         0, 3, FILTER_ADAPTIVE,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_choice_argument (procedure, "type",
+                                          _("_Filter Type"),
+                                          _("Filter type"),
+                                          gimp_choice_new_with_values ("median",             0,                 _("Median"),             NULL,
+                                                                       "adaptive",           FILTER_ADAPTIVE,   _("Adaptive"),           NULL,
+                                                                       "recursive-median",   FILTER_RECURSIVE, _("Recursive-Median"),   NULL,
+                                                                       "recursive-adaptive", 3,                 _("Recursive-Adaptive"), NULL,
+                                                                       NULL),
+                                          "adaptive",
+                                          G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "black",
-                         _("_Black level"),
-                         _("Black level"),
-                         -1, 255, 7,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "black",
+                                       _("_Black level"),
+                                       _("Black level"),
+                                       -1, 255, 7,
+                                       G_PARAM_READWRITE);
 
-      GIMP_PROC_ARG_INT (procedure, "white",
-                         _("_White level"),
-                         _("White level"),
-                         0, 256, 248,
-                         G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, "white",
+                                       _("_White level"),
+                                       _("White level"),
+                                       0, 256, 248,
+                                       G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -214,17 +219,15 @@ static GimpValueArray *
 despeckle_run (GimpProcedure        *procedure,
                GimpRunMode           run_mode,
                GimpImage            *image,
-               gint                  n_drawables,
                GimpDrawable        **drawables,
-               const GimpValueArray *args,
+               GimpProcedureConfig  *config,
                gpointer              run_data)
 {
-  GimpProcedureConfig *config;
-  GimpDrawable        *drawable;
+  GimpDrawable *drawable;
 
   gegl_init (NULL, NULL);
 
-  if (n_drawables != 1)
+  if (gimp_core_object_array_get_length ((GObject **) drawables) != 1)
     {
       GError *error = NULL;
 
@@ -241,35 +244,14 @@ despeckle_run (GimpProcedure        *procedure,
       drawable = drawables[0];
     }
 
-  if (! gimp_drawable_is_rgb  (drawable) &&
-      ! gimp_drawable_is_gray (drawable))
-    {
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_EXECUTION_ERROR,
-                                               NULL);
-    }
+  if (! gimp_drawable_is_rgb (drawable) && ! gimp_drawable_is_gray (drawable))
+    return gimp_procedure_new_return_values (procedure, GIMP_PDB_EXECUTION_ERROR, NULL);
 
-  config = gimp_procedure_create_config (procedure);
-  gimp_procedure_config_begin_run (config, NULL, run_mode, args);
-
-  if (run_mode == GIMP_RUN_INTERACTIVE)
-    {
-      if (! despeckle_dialog (procedure, G_OBJECT (config), drawable))
-        {
-          gimp_procedure_config_end_run (config, GIMP_PDB_CANCEL);
-          g_object_unref (config);
-
-          return gimp_procedure_new_return_values (procedure,
-                                                   GIMP_PDB_CANCEL,
-                                                   NULL);
-        }
-    }
+  if (run_mode == GIMP_RUN_INTERACTIVE && ! despeckle_dialog (procedure, G_OBJECT (config), drawable))
+    return gimp_procedure_new_return_values (procedure, GIMP_PDB_CANCEL, NULL);
 
   despeckle (drawable, G_OBJECT (config));
 
-  gimp_procedure_config_end_run (config, GIMP_PDB_SUCCESS);
-  g_object_unref (config);
-
   return gimp_procedure_new_return_values (procedure, GIMP_PDB_SUCCESS, NULL);
 }
 
@@ -393,13 +375,12 @@ despeckle_dialog (GimpProcedure *procedure,
                   GObject       *config,
                   GimpDrawable  *drawable)
 {
-  GtkWidget    *dialog;
-  GtkWidget    *preview;
-  GtkWidget    *vbox;
-  GtkWidget    *scale;
-  GtkWidget    *combo;
-  GtkListStore *store;
-  gboolean      run;
+  GtkWidget *dialog;
+  GtkWidget *preview;
+  GtkWidget *vbox;
+  GtkWidget *scale;
+  GtkWidget *combo;
+  gboolean   run;
 
   gimp_ui_init (PLUG_IN_BINARY);
 
@@ -407,22 +388,8 @@ despeckle_dialog (GimpProcedure *procedure,
                                       GIMP_PROCEDURE_CONFIG (config),
                                       _("Despeckle"));
 
-  gimp_window_set_transient (GTK_WINDOW (dialog));
-
-  preview = gimp_drawable_preview_new_from_drawable (drawable);
-  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))),
-                      preview, TRUE, TRUE, 0);
-  gtk_widget_set_margin_bottom (preview, 12);
-  gtk_widget_show (preview);
-
-  store = gimp_int_store_new (_("Median"),             0,
-                              _("Adaptive"),           FILTER_ADAPTIVE,
-                              _("Recursive-Median"),   FILTER_RECURSIVE,
-                              _("Recursive-Adaptive"), 3,
-                              NULL);
-  combo = gimp_procedure_dialog_get_int_combo (GIMP_PROCEDURE_DIALOG (dialog),
-                                               "type",
-                                               GIMP_INT_STORE (store));
+  combo = gimp_procedure_dialog_get_widget (GIMP_PROCEDURE_DIALOG (dialog),
+                                            "type", G_TYPE_NONE);
   gtk_widget_set_margin_bottom (combo, 12);
 
   /*
@@ -449,6 +416,10 @@ despeckle_dialog (GimpProcedure *procedure,
                                          NULL);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
 
+  preview = gimp_procedure_dialog_get_drawable_preview (GIMP_PROCEDURE_DIALOG (dialog),
+                                                        "preview", drawable);
+  gtk_widget_set_margin_bottom (preview, 12);
+
   g_object_set_data (config, "drawable", drawable);
 
   g_signal_connect (preview, "invalidated",
@@ -460,7 +431,7 @@ despeckle_dialog (GimpProcedure *procedure,
                             preview);
 
   gimp_procedure_dialog_fill (GIMP_PROCEDURE_DIALOG (dialog),
-                              "despeckle-vbox",
+                              "preview", "despeckle-vbox",
                               NULL);
   gtk_widget_show (dialog);
 
@@ -771,10 +742,11 @@ despeckle_median (GObject  *config,
 
   g_object_get (config,
                 "radius", &radius,
-                "type",   &filter_type,
                 "black",  &black_level,
                 "white",  &white_level,
                 NULL);
+  filter_type = gimp_procedure_config_get_choice_id (GIMP_PROCEDURE_CONFIG (config),
+                                                     "type");
 
   memset (&histogram, 0, sizeof(histogram));
   progress     = 0;
