@@ -67,7 +67,7 @@ Previous...Inherited code from Ray Lehtiniemi, who inherited it from S & P.
 
 
 #define LOAD_PROC      "file-xpm-load"
-#define SAVE_PROC      "file-xpm-save"
+#define EXPORT_PROC    "file-xpm-export"
 #define PLUG_IN_BINARY "file-xpm"
 #define PLUG_IN_ROLE   "gimp-file-xpm"
 #define SCALE_WIDTH    125
@@ -111,12 +111,11 @@ static GimpValueArray * xpm_load             (GimpProcedure         *procedure,
                                               GimpMetadataLoadFlags *flags,
                                               GimpProcedureConfig   *config,
                                               gpointer               run_data);
-static GimpValueArray * xpm_save             (GimpProcedure         *procedure,
+static GimpValueArray * xpm_export           (GimpProcedure         *procedure,
                                               GimpRunMode            run_mode,
                                               GimpImage             *image,
-                                              gint                   n_drawables,
-                                              GimpDrawable         **drawables,
                                               GFile                 *file,
+                                              GimpExportOptions     *options,
                                               GimpMetadata          *metadata,
                                               GimpProcedureConfig   *config,
                                               gpointer               run_data);
@@ -127,7 +126,7 @@ static guchar         * parse_colors         (XpmImage               *xpm_image)
 static void             parse_image          (GimpImage              *image,
                                               XpmImage               *xpm_image,
                                               guchar                 *cmap);
-static gboolean         save_image           (GFile                  *file,
+static gboolean         export_image         (GFile                  *file,
                                               GimpImage              *image,
                                               GimpDrawable           *drawable,
                                               GObject                *config,
@@ -179,7 +178,7 @@ xpm_query_procedures (GimpPlugIn *plug_in)
   GList *list = NULL;
 
   list = g_list_append (list, g_strdup (LOAD_PROC));
-  list = g_list_append (list, g_strdup (SAVE_PROC));
+  list = g_list_append (list, g_strdup (EXPORT_PROC));
 
   return list;
 }
@@ -199,16 +198,16 @@ xpm_create_procedure (GimpPlugIn  *plug_in,
       gimp_procedure_set_menu_label (procedure, _("X PixMap image"));
 
       gimp_procedure_set_documentation (procedure,
-                                        "Load files in XPM (X11 Pixmap) format.",
-                                        "Load files in XPM (X11 Pixmap) format. "
-                                        "XPM is a portable image format "
-                                        "designed to be included in C source "
-                                        "code. XLib provides utility functions "
-                                        "to read this format. Newer code should "
-                                        "however be using gdk-pixbuf-csource "
-                                        "instead. XPM supports colored images, "
-                                        "unlike the XBM format which XPM was "
-                                        "designed to replace.",
+                                        _("Load files in XPM (X11 Pixmap) format."),
+                                        _("Load files in XPM (X11 Pixmap) format. "
+                                          "XPM is a portable image format "
+                                          "designed to be included in C source "
+                                          "code. XLib provides utility functions "
+                                          "to read this format. Newer code should "
+                                          "however be using gdk-pixbuf-csource "
+                                          "instead. XPM supports colored images, "
+                                          "unlike the XBM format which XPM was "
+                                          "designed to replace."),
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Spencer Kimball & Peter Mattis & "
@@ -223,27 +222,27 @@ xpm_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_magics (GIMP_FILE_PROCEDURE (procedure),
                                       "0, string,/*\\040XPM\\040*/");
     }
-  else if (! strcmp (name, SAVE_PROC))
+  else if (! strcmp (name, EXPORT_PROC))
     {
-      procedure = gimp_save_procedure_new (plug_in, name,
-                                           GIMP_PDB_PROC_TYPE_PLUGIN,
-                                           FALSE, xpm_save, NULL, NULL);
+      procedure = gimp_export_procedure_new (plug_in, name,
+                                             GIMP_PDB_PROC_TYPE_PLUGIN,
+                                             FALSE, xpm_export, NULL, NULL);
 
       gimp_procedure_set_image_types (procedure, "*");
 
       gimp_procedure_set_menu_label (procedure, _("X PixMap image"));
 
       gimp_procedure_set_documentation (procedure,
-                                        "Export files in XPM (X11 Pixmap) format.",
-                                        "Export files in XPM (X11 Pixmap) format. "
-                                        "XPM is a portable image format "
-                                        "designed to be included in C source "
-                                        "code. XLib provides utility functions "
-                                        "to read this format. Newer code should "
-                                        "however be using gdk-pixbuf-csource "
-                                        "instead. XPM supports colored images, "
-                                        "unlike the XBM format which XPM was "
-                                        "designed to replace.",
+                                        _("Export files in XPM (X11 Pixmap) format."),
+                                        _("Export files in XPM (X11 Pixmap) format. "
+                                          "XPM is a portable image format "
+                                          "designed to be included in C source "
+                                          "code. XLib provides utility functions "
+                                          "to read this format. Newer code should "
+                                          "however be using gdk-pixbuf-csource "
+                                          "instead. XPM supports colored images, "
+                                          "unlike the XBM format which XPM was "
+                                          "designed to replace."),
                                         name);
       gimp_procedure_set_attribution (procedure,
                                       "Spencer Kimball & Peter Mattis & "
@@ -258,11 +257,18 @@ xpm_create_procedure (GimpPlugIn  *plug_in,
       gimp_file_procedure_set_extensions (GIMP_FILE_PROCEDURE (procedure),
                                           "xpm");
 
-      GIMP_PROC_ARG_INT (procedure, "threshold",
-                         _("_Threshold"),
-                         _("Alpha threshold"),
-                         0, 255, 127,
-                         G_PARAM_READWRITE);
+      gimp_export_procedure_set_capabilities (GIMP_EXPORT_PROCEDURE (procedure),
+                                              GIMP_EXPORT_CAN_HANDLE_RGB     |
+                                              GIMP_EXPORT_CAN_HANDLE_GRAY    |
+                                              GIMP_EXPORT_CAN_HANDLE_INDEXED |
+                                              GIMP_EXPORT_CAN_HANDLE_ALPHA,
+                                              NULL, NULL, NULL);
+
+      gimp_procedure_add_int_argument (procedure, "threshold",
+                                       _("_Threshold"),
+                                       _("Alpha threshold"),
+                                       0, 255, 127,
+                                       G_PARAM_READWRITE);
     }
 
   return procedure;
@@ -300,76 +306,47 @@ xpm_load (GimpProcedure         *procedure,
 }
 
 static GimpValueArray *
-xpm_save (GimpProcedure        *procedure,
-          GimpRunMode           run_mode,
-          GimpImage            *image,
-          gint                  n_drawables,
-          GimpDrawable        **drawables,
-          GFile                *file,
-          GimpMetadata         *metadata,
-          GimpProcedureConfig  *config,
-          gpointer              run_data)
+xpm_export (GimpProcedure        *procedure,
+            GimpRunMode           run_mode,
+            GimpImage            *image,
+            GFile                *file,
+            GimpExportOptions    *options,
+            GimpMetadata         *metadata,
+            GimpProcedureConfig  *config,
+            gpointer              run_data)
 {
   GimpPDBStatusType  status = GIMP_PDB_SUCCESS;
-  GimpExportReturn   export = GIMP_EXPORT_CANCEL;
-  GError            *error = NULL;
+  GimpExportReturn   export = GIMP_EXPORT_IGNORE;
+  GList             *drawables;
+  GError            *error  = NULL;
 
   gegl_init (NULL, NULL);
 
-  switch (run_mode)
-    {
-    case GIMP_RUN_INTERACTIVE:
-    case GIMP_RUN_WITH_LAST_VALS:
-      gimp_ui_init (PLUG_IN_BINARY);
-
-      export = gimp_export_image (&image, &n_drawables, &drawables, "XPM",
-                                  GIMP_EXPORT_CAN_HANDLE_RGB     |
-                                  GIMP_EXPORT_CAN_HANDLE_GRAY    |
-                                  GIMP_EXPORT_CAN_HANDLE_INDEXED |
-                                  GIMP_EXPORT_CAN_HANDLE_ALPHA);
-
-      if (export == GIMP_EXPORT_CANCEL)
-        return gimp_procedure_new_return_values (procedure,
-                                                 GIMP_PDB_CANCEL,
-                                                 NULL);
-      break;
-
-    default:
-      break;
-    }
-
-  if (n_drawables != 1)
-    {
-      g_set_error (&error, G_FILE_ERROR, 0,
-                   _("XPM format does not support multiple layers."));
-
-      return gimp_procedure_new_return_values (procedure,
-                                               GIMP_PDB_CALLING_ERROR,
-                                               error);
-    }
+  export = gimp_export_options_get_image (options, &image);
+  drawables = gimp_image_list_layers (image);
 
   if (run_mode == GIMP_RUN_INTERACTIVE)
     {
-      if (gimp_drawable_has_alpha (drawables[0]))
+      gimp_ui_init (PLUG_IN_BINARY);
+
+      if (gimp_drawable_has_alpha (drawables->data))
         if (! save_dialog (image, procedure, G_OBJECT (config)))
           status = GIMP_PDB_CANCEL;
     }
 
   if (status == GIMP_PDB_SUCCESS)
     {
-      if (! save_image (file, image, drawables[0], G_OBJECT (config),
-                        &error))
+      if (! export_image (file, image, drawables->data, G_OBJECT (config),
+                          &error))
         {
           status = GIMP_PDB_EXECUTION_ERROR;
         }
     }
 
   if (export == GIMP_EXPORT_EXPORT)
-    {
-      gimp_image_delete (image);
-      g_free (drawables);
-    }
+    gimp_image_delete (image);
 
+  g_list_free (drawables);
   return gimp_procedure_new_return_values (procedure, status, error);
 }
 
@@ -629,11 +606,11 @@ decrement_hash_values (gpointer gkey,
 }
 
 static gboolean
-save_image (GFile         *file,
-            GimpImage     *image,
-            GimpDrawable  *drawable,
-            GObject       *config,
-            GError       **error)
+export_image (GFile         *file,
+              GimpImage     *image,
+              GimpDrawable  *drawable,
+              GObject       *config,
+              GError       **error)
 {
   GeglBuffer *buffer;
   const Babl *format;
@@ -790,7 +767,7 @@ save_image (GFile         *file,
 
   if (indexed)
     {
-      guchar *cmap = gimp_image_get_colormap (image, NULL, &ncolors);
+      guchar *cmap = gimp_palette_get_colormap (gimp_image_get_palette (image), babl_format ("R'G'B' u8"), &ncolors, NULL);
       guchar *c;
 
       c = cmap;
@@ -842,7 +819,7 @@ save_image (GFile         *file,
   xpm_image->colorTable = colormap;
   xpm_image->data       = ibuff;
 
-  /* do the save */
+  /* do the export */
   switch (XpmWriteFileFromXpmImage ((char *) g_file_peek_path (file),
                                     xpm_image, NULL))
     {
@@ -884,9 +861,9 @@ save_dialog (GimpImage     *image,
   GtkWidget *dialog;
   gboolean   run;
 
-  dialog = gimp_save_procedure_dialog_new (GIMP_SAVE_PROCEDURE (procedure),
-                                           GIMP_PROCEDURE_CONFIG (config),
-                                           image);
+  dialog = gimp_export_procedure_dialog_new (GIMP_EXPORT_PROCEDURE (procedure),
+                                             GIMP_PROCEDURE_CONFIG (config),
+                                             image);
 
   gimp_procedure_dialog_get_scale_entry (GIMP_PROCEDURE_DIALOG (dialog),
                                          "threshold", 1.0);
