@@ -7,9 +7,9 @@
  *
  * Copyright (C) 1997 Andy Thomas  alt@picnic.demon.co.uk
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,8 +18,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  */
 
@@ -30,7 +29,8 @@ void    d_circle_object_class_init (void);
 
 void    d_circle_start             (GdkPoint *pnt,
                                     gboolean  shift_down);
-void    d_circle_end               (GdkPoint *pnt,
+void    d_circle_end               (GimpGfig *gfig,
+                                    GdkPoint *pnt,
                                     gboolean  shift_down);
 
 #endif /* __GFIG_CIRCLE_H__ */
