@@ -18,11 +18,12 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include "config.h"
 
+#include <math.h>
 #include <stdlib.h>
 
 #include <libgimp/gimp.h>
@@ -187,7 +188,7 @@ find_grid_pos_polar (GdkPoint *p,
 /* return the new position in the passed point */
 
 void
-gfig_grid_colours (GtkWidget *widget)
+gfig_grid_colors (GtkWidget *widget)
 {
 }
 
@@ -309,8 +310,8 @@ find_grid_pos (GdkPoint *p,
       m_hi_n_lo = m_lo_n_lo + 1;
 
      /* figure out which is the better candidate */
-      if (abs ((m_lo_n_lo * r + (0.5 * r * (n_lo % 2))) - y) <
-          abs ((m_hi_n_lo * r + (0.5 * r * (n_lo % 2))) - y))
+      if (fabs ((m_lo_n_lo * r + (0.5 * r * (n_lo % 2))) - y) <
+          fabs ((m_hi_n_lo * r + (0.5 * r * (n_lo % 2))) - y))
         {
           m_n_lo = m_lo_n_lo;
         }
@@ -324,8 +325,8 @@ find_grid_pos (GdkPoint *p,
       m_hi_n_hi = m_lo_n_hi + 1;
 
       /* figure out which is the better candidate */
-      if (abs((m_lo_n_hi * r + (0.5 * r * (n_hi % 2))) - y) <
-          abs((m_hi_n_hi * r + (0.5 * r * (n_hi % 2))) - y))
+      if (fabs((m_lo_n_hi * r + (0.5 * r * (n_hi % 2))) - y) <
+          fabs((m_hi_n_hi * r + (0.5 * r * (n_hi % 2))) - y))
         {
           m_n_hi = m_lo_n_hi;
         }
@@ -373,8 +374,8 @@ draw_grid_polar (cairo_t *cr)
         gdouble t;
         gdouble sector_size = 2 * G_PI / current_sectors;
         cairo_arc (cr,
-                   0.5 + preview_width / 2,
-                   0.5 + preview_height / 2,
+                   0.5 + preview_width / 2.0,
+                   0.5 + preview_height / 2.0,
                    outer_radius, 0, 2 * G_PI);
         cairo_stroke (cr);
 
@@ -391,11 +392,11 @@ draw_grid_polar (cairo_t *cr)
             gdouble normal_x = cos (selvals.opts.grid_rotation+t);
             gdouble normal_y = sin (selvals.opts.grid_rotation+t);
             cairo_move_to (cr,
-                           0.5 + (preview_width / 2 + inner_radius * normal_x),
-                           0.5 + (preview_height / 2 - inner_radius * normal_y));
+                           0.5 + (preview_width / 2.0 + inner_radius * normal_x),
+                           0.5 + (preview_height / 2.0 - inner_radius * normal_y));
             cairo_line_to (cr,
-                           0.5 + (preview_width / 2 + outer_radius * normal_x),
-                           0.5 + (preview_height / 2 - outer_radius * normal_y));
+                           0.5 + (preview_width / 2.0 + outer_radius * normal_x),
+                           0.5 + (preview_height / 2.0 - outer_radius * normal_y));
             cairo_stroke (cr);
           }
       }
