@@ -24,6 +24,7 @@
 #include "script-fu-types.h"
 #include "script-fu-arg.h"
 #include "script-fu-utils.h"
+#include "script-fu-version.h"
 
 
 /*
@@ -65,12 +66,6 @@
  * While Scheme speakers understand Scheme uses "numeric" for both float and int,
  * this might be confusing to script authors using other programming languages.
  *
- * SF_VALUE probably should be obsoleted.
- * Search ChangeLog for mention of "SF_VALUE"
- * See below, the only difference is that one get string escaped.
- * Otherwise, SF_VALUE is identical to SF_STRING.
- * Probably SF_VALUE still exists just for backward compatibility.
- *
  * SFArgType denotes not only a C type, but also a Scheme type.
  * For example, SF_ADJUSTMENT denotes the C type "float"
  * and the Scheme type "numeric" (which encompasses float and int.)
@@ -79,8 +74,14 @@
  */
 
 
-static void pspec_set_default_file (GParamSpec *pspec, const gchar *filepath);
-static void append_int_repr_from_gvalue (GString *result_string, GValue *gvalue);
+static void pspec_set_default_file         (GParamSpec  *pspec,
+                                            const gchar *filepath);
+static void append_int_repr_from_gvalue    (GString     *result_string,
+                                            GValue      *gvalue);
+static void append_scheme_repr_of_c_string (const gchar *user_string,
+                                            GString     *result_string);
+
+static GString *script_fu_arg_get_blurb    (SFArg *arg);
 
 /* Free any allocated members.
  * Somewhat hides what members of the SFArg struct are allocated.
@@ -91,9 +92,11 @@ void
 script_fu_arg_free (SFArg *arg)
 {
   g_free (arg->label);
+  g_free (arg->property_name);
 
   switch (arg->type)
     {
+    /* Integer ID's: primitives not needing free. */
     case SF_IMAGE:
     case SF_DRAWABLE:
     case SF_LAYER:
@@ -104,7 +107,14 @@ script_fu_arg_free (SFArg *arg)
     case SF_TOGGLE:
       break;
 
-    case SF_VALUE:
+    case SF_BRUSH:
+    case SF_FONT:
+    case SF_GRADIENT:
+    case SF_PALETTE:
+    case SF_PATTERN:
+      sf_resource_arg_free (arg);
+      break;
+
     case SF_STRING:
     case SF_TEXT:
       g_free (arg->default_value.sfa_value);
@@ -120,36 +130,6 @@ script_fu_arg_free (SFArg *arg)
       g_free (arg->value.sfa_file.filename);
       break;
 
-    /* FUTURE: font..gradient could all use the same code.
-     * Since the type in the union are all the same: gchar*.
-     * That is, group these cases with SF_VALUE.
-     * But this method should go away altogether.
-     */
-    case SF_FONT:
-      g_free (arg->default_value.sfa_font);
-      g_free (arg->value.sfa_font);
-      break;
-
-    case SF_PALETTE:
-      g_free (arg->default_value.sfa_palette);
-      g_free (arg->value.sfa_palette);
-      break;
-
-    case SF_PATTERN:
-      g_free (arg->default_value.sfa_pattern);
-      g_free (arg->value.sfa_pattern);
-      break;
-
-    case SF_GRADIENT:
-      g_free (arg->default_value.sfa_gradient);
-      g_free (arg->value.sfa_gradient);
-      break;
-
-    case SF_BRUSH:
-      g_free (arg->default_value.sfa_brush);
-      g_free (arg->value.sfa_brush);
-      break;
-
     case SF_OPTION:
       g_slist_free_full (arg->default_value.sfa_option.list,
                          (GDestroyNotify) g_free);
@@ -161,7 +141,17 @@ script_fu_arg_free (SFArg *arg)
     }
 }
 
-/* Reset: copy the default value to current value. */
+/* Reset: copy the default value to current value.
+ * This is only for old-style scripts and interface:
+ * new-style scripts keep args in a ProcedureConfig, which is resettable.
+ * Invoked when user chooses "Reset" button in the old-style dialog.
+ *
+ * Old-style reset is to the "factory default"
+ * i.e. the default declared in the script.
+ * New-style dialog also allows reset to "initial values"
+ * which are the values when the dialog appeared,
+ * which can be values from a previous run.
+ */
 void
 script_fu_arg_reset (SFArg *arg, gboolean should_reset_ids)
 {
@@ -176,6 +166,7 @@ script_fu_arg_reset (SFArg *arg, gboolean should_reset_ids)
     case SF_CHANNEL:
     case SF_VECTORS:
     case SF_DISPLAY:
+
       if (should_reset_ids)
         {
           /* !!! Use field name "sfa_image"; all these cases have same type in union.
@@ -187,15 +178,24 @@ script_fu_arg_reset (SFArg *arg, gboolean should_reset_ids)
 
       break;
 
+    case SF_BRUSH:
+    case SF_FONT:
+    case SF_GRADIENT:
+    case SF_PALETTE:
+    case SF_PATTERN:
+      if (should_reset_ids)
+        sf_resource_arg_reset (arg);
+      break;
+
     case SF_COLOR:
-      value->sfa_color = default_value->sfa_color;
+      g_clear_object (&value->sfa_color);
+      value->sfa_color = gegl_color_duplicate (default_value->sfa_color);
       break;
 
     case SF_TOGGLE:
       value->sfa_toggle = default_value->sfa_toggle;
       break;
 
-    case SF_VALUE:
     case SF_STRING:
     case SF_TEXT:
       g_free (value->sfa_value);
@@ -212,35 +212,6 @@ script_fu_arg_reset (SFArg *arg, gboolean should_reset_ids)
       value->sfa_file.filename = g_strdup (default_value->sfa_file.filename);
       break;
 
-    /* FUTURE: font..gradient could all use the same code.
-     * Since the type in the union are all the same: gchar*.
-     * That is, group these cases with SF_VALUE.
-     */
-    case SF_FONT:
-      g_free (value->sfa_font);
-      value->sfa_font = g_strdup (default_value->sfa_font);
-      break;
-
-    case SF_PALETTE:
-      g_free (value->sfa_palette);
-      value->sfa_palette = g_strdup (default_value->sfa_palette);
-      break;
-
-    case SF_PATTERN:
-      g_free (value->sfa_pattern);
-      value->sfa_pattern = g_strdup (default_value->sfa_pattern);
-      break;
-
-    case SF_GRADIENT:
-      g_free (value->sfa_gradient);
-      value->sfa_gradient = g_strdup (default_value->sfa_gradient);
-      break;
-
-    case SF_BRUSH:
-      g_free (value->sfa_brush);
-      value->sfa_brush = g_strdup (default_value->sfa_brush);
-      break;
-
     case SF_OPTION:
       value->sfa_option.history = default_value->sfa_option.history;
       break;
@@ -251,6 +222,131 @@ script_fu_arg_reset (SFArg *arg, gboolean should_reset_ids)
     }
 }
 
+/* The type of a function that declares a formal, resource argument to a PDB procedure. */
+typedef void (*ResourceArgDeclareFunc) (
+  GimpProcedure*, const gchar*, const gchar*, const gchar*,
+  gboolean, GimpResource*, gboolean, GParamFlags);
+
+
+
+static void
+script_fu_add_resource_arg_default_from_context (
+  GimpProcedure          *procedure,
+  const gchar            *name,
+  const gchar            *nick,
+  const gchar            *blurb,
+  ResourceArgDeclareFunc  func)
+{
+  /* Default from context, dynamically: default:NULL, default_from_context:TRUE */
+  (*func) (procedure, name, nick, blurb,
+            FALSE,  /* none OK */
+            NULL,   /* default */
+            TRUE,   /* default_from_context */
+            G_PARAM_READWRITE);
+}
+
+
+static void
+script_fu_add_resource_arg_with_default (
+  GimpProcedure          *procedure,
+  const gchar            *name,
+  const gchar            *nick,
+  const gchar            *blurb,
+  ResourceArgDeclareFunc  func,
+  GimpResource           *default_resource)
+{
+  (*func) (procedure, name, nick, blurb,
+            FALSE,  /* none OK */
+            default_resource,
+            FALSE,   /* default_from_context */
+            G_PARAM_READWRITE);
+}
+
+
+/* Formally declare a resource arg of a PDB procedure.
+ * From the given SFArg.
+ */
+static void
+script_fu_add_resource_arg (
+  GimpProcedure          *procedure,
+  const gchar            *name,
+  const gchar            *nick,
+  const gchar            *blurb,
+  SFArg                  *arg,
+  ResourceArgDeclareFunc  func)
+{
+  /* Switch on script author's declared resource name. */
+  gchar *declared_name_of_default = sf_resource_arg_get_name_default (arg);
+
+  if (g_strcmp0 (declared_name_of_default, "from context") == 0 ||
+      g_utf8_strlen (declared_name_of_default, 256) == 0
+     )
+    {
+      /* Author declared default name is empty or the special "from context". */
+      script_fu_add_resource_arg_default_from_context (procedure, name, nick, blurb, func);
+    }
+  else
+    {
+      /* Author declared default name should name a resource.
+       * Declare the formal arg to default to the named resource.
+       *
+       * The declared name should be untranslated.
+       * Note that names might not be unique e.g. for fonts.
+       * Note that some resources have translated names e.g. generated gradients.
+       *
+       * The default resource could still be NULL if the declared name does not match.
+       * Warn in that case, to the console, at procedure creation time.
+       * Note we still pass none_OK:FALSE meaning the procedure expects
+       * a non-NULL resource at call time (after GUI.)
+       * In interactive mode, the GUI may default from context when passed a NULL default.
+       */
+      GimpResource *default_resource = sf_resource_arg_get_default (arg);
+
+      if (default_resource == NULL)
+        {
+          g_warning ("%s declared resource name is invalid %s", G_STRFUNC, declared_name_of_default);
+          script_fu_add_resource_arg_default_from_context (procedure, name, nick, blurb, func);
+        }
+      else
+        {
+          /* Declared name is valid name of resource. */
+          script_fu_add_resource_arg_with_default (procedure, name, nick, blurb, func,
+                                                   default_resource);
+        }
+    }
+}
+
+/* Get the blurb of an argument.
+ *
+ * The blurb becomes the tooltip for widgets
+ * and the description for the arg in the PDB.
+ *
+ * ScriptFu does not let authors declare blurb.
+ * Instead we derive the blurb from a "label",
+ * replacing the mnemonic/accelerator mark char.
+ * An author declares a label i.e. menu item, with optional accelerator mark.
+ * The label is a translated text.
+ * SF also uses label for nick of the argument's param spec.
+ *
+ * For this implementation, the widget tooltip is redundant with the widget label,
+ * but at least the PDB description is not empty.
+ * The PDB description must not be empty because the author also cannot declare
+ * the name (SF generates it) and the generated name is meaningless to a reader.
+ * FUTURE: let author's declare a separate, translated blurb.
+ * FUTURE: let author's declare a name for the arg (instead of generating one.)
+ *
+ * Caller owns the result.
+ */
+static GString*
+script_fu_arg_get_blurb (SFArg *arg)
+{
+  GString *result = g_string_new (arg->label);
+
+  g_string_replace (result, "_", "", 0);
+  return result;
+}
+
+
 /* Return param spec that describes the arg.
  * Convert instance of SFArg to instance of GParamSpec.
  *
@@ -266,137 +362,115 @@ script_fu_arg_reset (SFArg *arg, gboolean should_reset_ids)
  * this should convey what SFArg denotes about desired widget kind,
  * but it doesn't fully do that yet.
  */
-GParamSpec *
-script_fu_arg_get_param_spec (SFArg       *arg,
-                              const gchar *name,
-                              const gchar *nick)
+void
+script_fu_arg_add_argument (SFArg         *arg,
+                            GimpProcedure *procedure,
+                            const gchar   *name,
+                            const gchar   *nick)
 {
-  GParamSpec * pspec = NULL;
+  GString               *blurb_gstring = script_fu_arg_get_blurb (arg);
+  gchar                 *blurb         = blurb_gstring->str;
+  GimpFileChooserAction  action        = GIMP_FILE_CHOOSER_ACTION_ANY;
 
   switch (arg->type)
     {
       /* No defaults for GIMP objects: Image, Item subclasses, Display */
     case SF_IMAGE:
-      pspec = gimp_param_spec_image (name,
-                                     nick,
-                                     arg->label,
-                                     TRUE,  /* None is valid. */
-                                     G_PARAM_READWRITE);
+      gimp_procedure_add_image_argument (procedure, name,
+                                         nick, blurb,
+                                         TRUE,  /* None is valid. */
+                                         G_PARAM_READWRITE);
       break;
 
     case SF_DRAWABLE:
-      pspec = gimp_param_spec_drawable (name,
-                                        nick,
-                                        arg->label,
+      gimp_procedure_add_drawable_argument (procedure, name,
+                                            nick, blurb,
+                                            TRUE,
+                                            G_PARAM_READWRITE);
+      break;
+
+    case SF_LAYER:
+      gimp_procedure_add_layer_argument (procedure, name,
+                                         nick, blurb,
+                                         TRUE,
+                                         G_PARAM_READWRITE);
+      break;
+
+    case SF_CHANNEL:
+      gimp_procedure_add_channel_argument (procedure, name,
+                                           nick, blurb,
+                                           TRUE,
+                                           G_PARAM_READWRITE);
+      break;
+
+    case SF_VECTORS:
+      gimp_procedure_add_path_argument (procedure, name,
+                                        nick, blurb,
                                         TRUE,
                                         G_PARAM_READWRITE);
       break;
 
-    case SF_LAYER:
-      pspec = gimp_param_spec_layer (name,
-                                     nick,
-                                     arg->label,
-                                     TRUE,
-                                     G_PARAM_READWRITE);
-      break;
-
-    case SF_CHANNEL:
-      pspec = gimp_param_spec_channel (name,
-                                       nick,
-                                       arg->label,
-                                       TRUE,
-                                       G_PARAM_READWRITE);
-      break;
-
-    case SF_VECTORS:
-      pspec = gimp_param_spec_vectors (name,
-                                       nick,
-                                       arg->label,
-                                       TRUE,
-                                       G_PARAM_READWRITE);
-      break;
-
     case SF_DISPLAY:
-      pspec = gimp_param_spec_display (name,
-                                       nick,
-                                       arg->label,
-                                       TRUE,
-                                       G_PARAM_READWRITE);
+      gimp_procedure_add_display_argument (procedure, name,
+                                           nick, blurb,
+                                           TRUE,
+                                           G_PARAM_READWRITE);
       break;
 
     case SF_COLOR:
-      /* Pass address of default color i.e. instance of GimpRGB.
-       * Color is owned by ScriptFu and exists for lifetime of SF process.
-       */
-      pspec = gimp_param_spec_rgb (name,
-                                   nick,
-                                   arg->label,
-                                   TRUE,  /* is alpha relevant */
-                                   &arg->default_value.sfa_color,
-                                   G_PARAM_READWRITE);
-      /* FUTURE: Default not now appear in PDB browser, but appears in widgets? */
+      {
+        GeglColor *color = sf_color_arg_get_default_color (arg);
+
+        gimp_procedure_add_color_argument (procedure, name, nick, blurb, TRUE,
+                                           color, G_PARAM_READWRITE);
+        g_object_unref (color);
+      }
       break;
 
     case SF_TOGGLE:
       /* Implicit conversion from gint32 to gboolean. */
-      pspec = g_param_spec_boolean (name,
-                                    nick,
-                                    arg->label,
-                                    arg->default_value.sfa_toggle,
-                                    G_PARAM_READWRITE);
+      gimp_procedure_add_boolean_argument (procedure, name,
+                                           nick, blurb,
+                                           arg->default_value.sfa_toggle,
+                                           G_PARAM_READWRITE);
       break;
 
     /* FUTURE special widgets for multiline text.
      * script-fu-interface does, but GimpProcedureDialog does not.
      */
-    case SF_VALUE:
     case SF_STRING:
     case SF_TEXT:
-      pspec = g_param_spec_string (name,
-                                   nick,
-                                   arg->label,
-                                   arg->default_value.sfa_value,
-                                   G_PARAM_READWRITE);
+      gimp_procedure_add_string_argument (procedure, name,
+                                          nick, blurb,
+                                          arg->default_value.sfa_value,
+                                          G_PARAM_READWRITE);
       break;
 
     /* Subclasses of GimpResource.  Special widgets. */
     case SF_FONT:
-      pspec = gimp_param_spec_font (name,
-                                    nick,
-                                    arg->label,
-                                    FALSE,  /* none OK */
-                                    G_PARAM_READWRITE | GIMP_PARAM_NO_VALIDATE);
+      /* Avoid compiler warning: cast a type specific function to a generic function. */
+      script_fu_add_resource_arg (procedure, name, nick, blurb, arg,
+          (ResourceArgDeclareFunc) gimp_procedure_add_font_argument);
       break;
+
     case SF_PALETTE:
-      pspec = gimp_param_spec_palette (name,
-                                       nick,
-                                       arg->label,
-                                       FALSE,  /* none OK */
-                                       G_PARAM_READWRITE | GIMP_PARAM_NO_VALIDATE);
+      script_fu_add_resource_arg (procedure, name, nick, blurb, arg,
+          (ResourceArgDeclareFunc) gimp_procedure_add_palette_argument);
       break;
 
     case SF_PATTERN:
-      pspec = gimp_param_spec_pattern (name,
-                                       nick,
-                                       arg->label,
-                                       FALSE,  /* none OK */
-                                       G_PARAM_READWRITE | GIMP_PARAM_NO_VALIDATE);
+      script_fu_add_resource_arg (procedure, name, nick, blurb, arg,
+          (ResourceArgDeclareFunc) gimp_procedure_add_pattern_argument);
       break;
 
     case SF_GRADIENT:
-      pspec = gimp_param_spec_gradient (name,
-                                        nick,
-                                        arg->label,
-                                        FALSE,  /* none OK */
-                                        G_PARAM_READWRITE | GIMP_PARAM_NO_VALIDATE);
+      script_fu_add_resource_arg (procedure, name, nick, blurb, arg,
+          (ResourceArgDeclareFunc) gimp_procedure_add_gradient_argument);
       break;
 
     case SF_BRUSH:
-      pspec = gimp_param_spec_brush (name,
-                                     nick,
-                                     arg->label,
-                                     FALSE,  /* none OK */
-                                     G_PARAM_READWRITE | GIMP_PARAM_NO_VALIDATE);
+      script_fu_add_resource_arg (procedure, name, nick, blurb, arg,
+          (ResourceArgDeclareFunc) gimp_procedure_add_brush_argument);
       break;
 
     case SF_ADJUSTMENT:
@@ -405,57 +479,79 @@ script_fu_arg_get_param_spec (SFArg       *arg,
        * Decimal places == 0 means type integer, else float
        */
       if (arg->default_value.sfa_adjustment.digits == 0)
-        pspec = g_param_spec_int (name, nick, arg->label,
-                                  arg->default_value.sfa_adjustment.lower,
-                                  arg->default_value.sfa_adjustment.upper,
-                                  arg->default_value.sfa_adjustment.value,
-                                  G_PARAM_READWRITE);
+        gimp_procedure_add_int_argument (procedure, name, nick, blurb,
+                                         arg->default_value.sfa_adjustment.lower,
+                                         arg->default_value.sfa_adjustment.upper,
+                                         arg->default_value.sfa_adjustment.value,
+                                         G_PARAM_READWRITE);
       else
-        pspec = g_param_spec_double (name, nick, arg->label,
-                                     arg->default_value.sfa_adjustment.lower,
-                                     arg->default_value.sfa_adjustment.upper,
-                                     arg->default_value.sfa_adjustment.value,
-                                     G_PARAM_READWRITE);
+        gimp_procedure_add_double_argument (procedure, name, nick, blurb,
+                                            arg->default_value.sfa_adjustment.lower,
+                                            arg->default_value.sfa_adjustment.upper,
+                                            arg->default_value.sfa_adjustment.value,
+                                            G_PARAM_READWRITE);
       break;
 
     case SF_FILENAME:
+      action = GIMP_FILE_CHOOSER_ACTION_OPEN;
     case SF_DIRNAME:
-      pspec = g_param_spec_object (name,
-                                   nick,
-                                   arg->label,
-                                   G_TYPE_FILE,
-                                   G_PARAM_READWRITE |
-                                   GIMP_PARAM_NO_VALIDATE);
-      pspec_set_default_file (pspec, arg->default_value.sfa_file.filename);
-      /* FUTURE: Default not now appear in PDB browser, but appears in widgets? */
+        {
+          GParamSpec *pspec = NULL;
+
+          if (action == GIMP_FILE_CHOOSER_ACTION_ANY)
+            action = GIMP_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+
+          gimp_procedure_add_file_argument (procedure, name,
+                                            nick, blurb,
+                                            action, TRUE, NULL,
+                                            G_PARAM_READWRITE |
+                                            GIMP_PARAM_NO_VALIDATE);
+          pspec = gimp_procedure_find_argument (procedure, name);
+          pspec_set_default_file (pspec, arg->default_value.sfa_file.filename);
+        }
       break;
 
+      /* Not necessary to have a more specific pspec:
+       * pspec = gimp_param_spec_config_path (...GIMP_TYPE_CONFIG_PATH...)
+       */
+
     case SF_ENUM:
       /* history is the last used value AND the default. */
-      pspec = g_param_spec_enum (name,
-                                 nick,
-                                 arg->label,
-                                 g_type_from_name (arg->default_value.sfa_enum.type_name),
-                                 arg->default_value.sfa_enum.history,
-                                 G_PARAM_READWRITE);
+      gimp_procedure_add_enum_argument (procedure, name,
+                                        nick, blurb,
+                                        g_type_from_name (arg->default_value.sfa_enum.type_name),
+                                        arg->default_value.sfa_enum.history,
+                                        G_PARAM_READWRITE);
       break;
 
     case SF_OPTION:
-      pspec = g_param_spec_int (name,
-                                nick,
-                                arg->label,
-                                0,        /* Always zero based. */
-                                g_slist_length (arg->default_value.sfa_option.list),
-                                arg->default_value.sfa_option.history,
-                                G_PARAM_READWRITE);
+      gimp_procedure_add_int_argument (procedure, name,
+                                       nick, blurb,
+                                       0,        /* Always zero based. */
+                                       g_slist_length (arg->default_value.sfa_option.list) - 1,
+                                       arg->default_value.sfa_option.history,
+                                       G_PARAM_READWRITE);
       /* FUTURE: Model values not now appear in PDB browser NOR in widgets? */
       /* FUTURE: Does not show a combo box widget ??? */
       break;
     }
 
-  return pspec;
+  g_string_free (blurb_gstring, TRUE);
 }
 
+/* Warn of an error in a GFile argument
+ * and append a repr of an unknown file to the result_string,
+ * which is a scheme text being built.
+ * The warning is to the console.
+ * The warning may be innocuous if the script handles the case.
+ */
+static void
+sf_append_file_error_repr (gchar *err_message, GString *result_string )
+{
+  g_warning ("%s", err_message);
+  /* Represent unknown file by literal for empty string: "" */
+  g_string_append_printf (result_string, "\"\"");
+}
 
 /* Append a Scheme representation of the arg value from the given gvalue.
  * Append to a Scheme text to be interpreted.
@@ -482,6 +578,14 @@ script_fu_arg_append_repr_from_gvalue (SFArg       *arg,
     case SF_CHANNEL:
     case SF_VECTORS:
     case SF_DISPLAY:
+
+    /* The GValue is a GObject of type inheriting GimpResource, having id prop */
+    case SF_BRUSH:
+    case SF_FONT:
+    case SF_GRADIENT:
+    case SF_PALETTE:
+    case SF_PATTERN:
+
       {
         GObject *object = g_value_get_object (gvalue);
         gint     id     = -1;
@@ -495,34 +599,39 @@ script_fu_arg_append_repr_from_gvalue (SFArg       *arg,
 
     case SF_COLOR:
       {
-        GimpRGB color;
-        guchar  r, g, b;
+        /* Since v3 PDB procedures traffic in GeglColor.
+         * But ScriptFu still represents as RGB uint8 in a list.
+         */
+        GeglColor *color;
 
-        gimp_value_get_rgb (gvalue, &color);
-        gimp_rgb_get_uchar (&color, &r, &g, &b);
-        g_string_append_printf (result_string, "'(%d %d %d)",
-                                (gint) r, (gint) g, (gint) b);
+        color = g_value_get_object (gvalue);
+        if (color)
+          {
+            gchar *repr = sf_color_get_repr_from_gegl_color (color);
+            g_string_append (result_string, repr);
+            g_free (repr);
+          }
+        else
+          {
+            gchar *msg = "Invalid color object in gvalue.";
+            g_warning ("%s", msg);
+            g_string_append_printf (result_string, "\"%s\"", msg);
+          }
       }
       break;
 
     case SF_TOGGLE:
-      g_string_append_printf (result_string, (g_value_get_boolean (gvalue) ?
-                                  "TRUE" : "FALSE"));
-      break;
-
-    case SF_VALUE:
-      g_string_append (result_string, g_value_get_string (gvalue));
+      if (is_interpret_v3_dialect ())
+        {
+          g_string_append (result_string, (g_value_get_boolean (gvalue) ? "#t" : "#f"));
+        }
+      else
+        g_string_append (result_string, (g_value_get_boolean (gvalue) ? "TRUE" : "FALSE"));
       break;
 
     case SF_STRING:
     case SF_TEXT:
-      {
-        gchar *tmp;
-
-        tmp = script_fu_strescape (g_value_get_string (gvalue));
-        g_string_append_printf (result_string, "\"%s\"", tmp);
-        g_free (tmp);
-      }
+      append_scheme_repr_of_c_string (g_value_get_string (gvalue), result_string);
       break;
 
     case SF_FILENAME:
@@ -542,25 +651,30 @@ script_fu_arg_append_repr_from_gvalue (SFArg       *arg,
                  * I.E. not g_file_get_path, which can return NULL.
                  */
                 gchar *filepath = g_file_get_parse_name (file);
+                gchar *esc_path = script_fu_strescape (filepath);
+
                 /* assert filepath not null. */
-                /* Not escape special chars for whitespace or double quote. */
-                g_string_append_printf (result_string, "\"%s\"", filepath);
+                g_string_append_printf (result_string, "\"%s\"", esc_path);
                 g_free (filepath);
+                g_free (esc_path);
               }
             else
               {
-                gchar *msg = "Invalid GFile in gvalue.";
-                g_warning ("%s", msg);
-                g_string_append_printf (result_string, "\"%s\"", msg);
+                /* This is usually a C NULL.
+                 * Some file chooser widgets may not support a default file
+                 * and/or may return NULL when the user doesn't choose a file.
+                 */
+                sf_append_file_error_repr ("Unknown file in GFile to script.",
+                                           result_string);
               }
           }
         else
           {
-            gchar *msg = "Expecting GFile in gvalue.";
-            g_warning ("%s", msg);
-            g_string_append_printf (result_string, "\"%s\"", msg);
+            /* Programming error: GValue should hold a GFile. */
+            sf_append_file_error_repr ("Expecting GFile in GValue to script.",
+                                       result_string);
           }
-        /* Ensure appended a filepath string OR an error string.*/
+        /* Ensure appended a filepath string OR a string repr of an unknown file. */
       }
       break;
 
@@ -580,25 +694,6 @@ script_fu_arg_append_repr_from_gvalue (SFArg       *arg,
       }
       break;
 
-    case SF_FONT:
-    case SF_PALETTE:
-    case SF_PATTERN:
-    case SF_GRADIENT:
-    case SF_BRUSH:
-      {
-        /* The GValue is a GObject of type inheriting GimpResource */
-        GimpResource *resource;
-        gchar        *name = NULL;
-
-        resource = g_value_get_object (gvalue);
-
-        if (resource)
-          name = gimp_resource_get_name (resource);
-
-        g_string_append_printf (result_string, "\"%s\"", name);
-      }
-      break;
-
     case SF_OPTION:
       append_int_repr_from_gvalue (result_string, gvalue);
       break;
@@ -647,33 +742,36 @@ script_fu_arg_append_repr_from_self (SFArg       *arg,
       g_string_append_printf (result_string, "%d", arg_value->sfa_image);
       break;
 
+    case SF_BRUSH:
+    case SF_FONT:
+    case SF_GRADIENT:
+    case SF_PALETTE:
+    case SF_PATTERN:
+      {
+        gchar *repr = sf_resource_arg_get_repr (arg);
+        g_string_append (result_string, repr);
+        g_free (repr);
+      }
+      break;
+
     case SF_COLOR:
       {
-        guchar r, g, b;
-
-        gimp_rgb_get_uchar (&arg_value->sfa_color, &r, &g, &b);
-        g_string_append_printf (result_string, "'(%d %d %d)",
-                                (gint) r, (gint) g, (gint) b);
+        gchar *repr = sf_color_get_repr (arg_value->sfa_color);
+        g_string_append (result_string, repr);
+        g_free (repr);
       }
       break;
 
     case SF_TOGGLE:
-      g_string_append (result_string, arg_value->sfa_toggle ? "TRUE" : "FALSE");
-      break;
-
-    case SF_VALUE:
-      g_string_append (result_string, arg_value->sfa_value);
+      if (is_interpret_v3_dialect ())
+        g_string_append (result_string, arg_value->sfa_toggle ? "#t" : "#f");
+      else
+        g_string_append (result_string, arg_value->sfa_toggle ? "TRUE" : "FALSE");
       break;
 
     case SF_STRING:
     case SF_TEXT:
-      {
-        gchar *tmp;
-
-        tmp = script_fu_strescape (arg_value->sfa_value);
-        g_string_append_printf (result_string, "\"%s\"", tmp);
-        g_free (tmp);
-      }
+      append_scheme_repr_of_c_string (arg_value->sfa_value, result_string);
       break;
 
     case SF_ADJUSTMENT:
@@ -688,33 +786,7 @@ script_fu_arg_append_repr_from_self (SFArg       *arg,
 
     case SF_FILENAME:
     case SF_DIRNAME:
-      {
-        gchar *tmp;
-
-        tmp = script_fu_strescape (arg_value->sfa_file.filename);
-        g_string_append_printf (result_string, "\"%s\"", tmp);
-        g_free (tmp);
-      }
-      break;
-
-    case SF_FONT:
-      g_string_append_printf (result_string, "\"%s\"", arg_value->sfa_font);
-      break;
-
-    case SF_PALETTE:
-      g_string_append_printf (result_string, "\"%s\"", arg_value->sfa_palette);
-      break;
-
-    case SF_PATTERN:
-      g_string_append_printf (result_string, "\"%s\"", arg_value->sfa_pattern);
-      break;
-
-    case SF_GRADIENT:
-      g_string_append_printf (result_string, "\"%s\"", arg_value->sfa_gradient);
-      break;
-
-    case SF_BRUSH:
-      g_string_append_printf (result_string, "\"%s\"", arg_value->sfa_brush);
+      append_scheme_repr_of_c_string (arg_value->sfa_file.filename, result_string);
       break;
 
     case SF_OPTION:
@@ -731,12 +803,12 @@ script_fu_arg_append_repr_from_self (SFArg       *arg,
 /* Array the size of the enum
  * Counts names generated per SF type per generator session.
  */
-static gint arg_count[SF_DISPLAY] = { 0, };
+static gint arg_count[SF_DISPLAY + 1] = { 0, };
 
 void
 script_fu_arg_reset_name_generator (void)
 {
-  for (guint i = 0; i<SF_DISPLAY; i++)
+  for (guint i = 0; i <= SF_DISPLAY; i++)
     arg_count[i] = 0;
 }
 
@@ -755,7 +827,7 @@ script_fu_arg_reset_name_generator (void)
  * Thus name meets uniquity for names of properties of one object.
  *
  * !!! GimpImageProcedures already have properties for convenience arguments,
- * e.g. a property named "image" "n_drawables" and "drawables"
+ * e.g. a property named "image" and "drawables"
  * So we avoid that name clash by starting with "otherImage"
  *
  * The name means nothing to human readers of the spec.
@@ -764,6 +836,8 @@ script_fu_arg_reset_name_generator (void)
  * The returned string is owned by the generator, a constant.
  * The caller need not copy it,
  * but usually does by creating a GParamSpec.
+ *
+ * As a side effect, a copy of the string is kept in arg->property_name.
  */
 void
 script_fu_arg_generate_name_and_nick (SFArg        *arg,
@@ -807,10 +881,6 @@ script_fu_arg_generate_name_and_nick (SFArg        *arg,
       name = "toggle";
       break;
 
-    case SF_VALUE:
-      name = "value";
-      break;
-
     case SF_STRING:
       name = "string";
       break;
@@ -872,13 +942,15 @@ script_fu_arg_generate_name_and_nick (SFArg        *arg,
 
   arg_count[arg->type]++;
 
+  /* Side effect: remember the generated name. */
+  arg->property_name = g_strdup (numbered_name);
+
   *returned_name = numbered_name;
 
-  /* nick is what the script author said describes the arg */
+  /* nick is what the script author declared for menu label */
   *returned_nick = arg->label;
 }
 
-
 /* Set the default of a GParamSpec to a GFile for a path string.
  * The GFile is allocated and ownership is transferred to the GParamSpec.
  * The GFile is only a name and a so-named file might not exist.
@@ -886,13 +958,14 @@ script_fu_arg_generate_name_and_nick (SFArg        *arg,
 static void
 pspec_set_default_file (GParamSpec *pspec, const gchar *filepath)
 {
-  GValue  gvalue = G_VALUE_INIT;
-  GFile  *gfile  = NULL;
+  GFile *gfile  = NULL;
 
-  g_value_init (&gvalue, G_TYPE_FILE);
-  gfile = g_file_new_for_path (filepath);
-  g_value_set_object (&gvalue, gfile);
-  g_param_value_set_default (pspec, &gvalue);
+  if (filepath != NULL && strlen (filepath) > 0)
+    gfile = g_file_new_for_path (filepath);
+
+  gimp_param_spec_object_set_default (pspec, G_OBJECT (gfile));
+
+  g_clear_object (&gfile);
 }
 
 /* Append a string repr of an integer valued gvalue to given GString.
@@ -914,3 +987,35 @@ append_int_repr_from_gvalue (GString *result_string, GValue *gvalue)
       g_string_append (result_string, "1");
     }
 }
+
+
+/* For the given C string IN, which can be user input so almost anything,
+ * append the Scheme repr to the GString OUT.
+ *
+ * The Scheme repr is:
+ *    - escaped certain characters
+ *    - surrounded by double quotes.
+ *
+ * When user input string is NULL, appends just a pair of quotes
+ * i.e. the Scheme representation of an empty string.
+ *
+ * Caller owns the result_string, and it is a GString, appendable.
+ */
+static void
+append_scheme_repr_of_c_string (const gchar *user_string, GString *result_string)
+{
+  gchar *tmp;
+
+  if (user_string == NULL)
+    {
+      g_string_append (result_string, "\"\"");
+    }
+  else
+    {
+      /* strescape requires user_string not null. */
+      tmp = script_fu_strescape (user_string);
+      g_string_append_printf (result_string, "\"%s\"", tmp);
+      g_free (tmp);
+    }
+}
+
